
const string UN_CONFIG_CPC_TESTOBJECT_DPT_NAME = "CPC_TestObject";

const unsigned UN_CONFIG_CPC_TESTOBJECT_LENGTH = 15;

const unsigned UN_CONFIG_CPC_TESTOBJECT_UNIT					= 1;
const unsigned UN_CONFIG_CPC_TESTOBJECT_FORMAT					= 2;
const unsigned UN_CONFIG_CPC_TESTOBJECT_RANGEMAX				= 3;
const unsigned UN_CONFIG_CPC_TESTOBJECT_RANGEMIN				= 4;
const unsigned UN_CONFIG_CPC_TESTOBJECT_DEADBAND				= 5;
const unsigned UN_CONFIG_CPC_TESTOBJECT_DEADBAND_TYPE			= 6;
const unsigned UN_CONFIG_CPC_TESTOBJECT_ARCHIVE_ACTIVE			= 7;
const unsigned UN_CONFIG_CPC_TESTOBJECT_ARCHIVE_TIME_FILTER		= 8;
const unsigned UN_CONFIG_CPC_TESTOBJECT_ADDRESS_STSREG01		= 9;
const unsigned UN_CONFIG_CPC_TESTOBJECT_ADDRESS_EVSTSREG01		= 10;
const unsigned UN_CONFIG_CPC_TESTOBJECT_ADDRESS_POSST			= 11;
const unsigned UN_CONFIG_CPC_TESTOBJECT_ADDRESS_AUPOSRST		= 12;
const unsigned UN_CONFIG_CPC_TESTOBJECT_ADDRESS_MANREG01		= 13;
const unsigned UN_CONFIG_CPC_TESTOBJECT_ADDRESS_MPOSR			= 14;
const unsigned UN_CONFIG_CPC_TESTOBJECT_NORMAL_POSITION   = 15;


const unsigned UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH		= 7;
const unsigned UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_MASKEVENT	= 1;
const unsigned UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_PARAMETERS	= 2;
const unsigned UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_MASTER_NAME	= 3;
const unsigned UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_PARENTS		= 4;
const unsigned UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_CHILDREN		= 5;
const unsigned UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_TYPE			= 6;
const unsigned UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_SECOND_ALIAS	= 7;

mapping CPC_TestObjectConfig_getConfig() {
	mapping mConfig;
	mapping mProps;

    mappingClear(mProps);
    mProps["address"] 		  = ".ProcessInput.StsReg01";
    mProps["dataType"] 		  = CPC_UINT16;
    mConfig["StsReg01"] 		= mProps;

    mappingClear(mProps);
    mProps["hasArchive"] 	  = true;
    mProps["address"] 		  = ".ProcessInput.evStsReg01";
    mProps["dataType"] 		  = CPC_INT32;
    mConfig["evStsReg01"] 	= mProps;

	mappingClear(mProps);
    mProps["hasUnit"] 		  = true;
    mProps["hasFormat"]		  = true;
    mProps["hasSmooth"] 	  = true;
    mProps["hasArchive"] 	  = true;
    mProps["hasPvRange"] 	  = true;
    mProps["address"] 		  = ".ProcessInput.PosSt";
    mProps["dataType"] 		  = CPC_FLOAT;
    mConfig["PosSt"] 		    = mProps;

	mappingClear(mProps);
    mProps["hasSmooth"] 		= true;
    mProps["hasUnit"] 		  = true;
    mProps["hasFormat"]		  = true;
    mProps["address"] 		  = ".ProcessInput.AuPosRSt";
    mProps["dataType"] 		  = CPC_FLOAT;
    mConfig["AuPosRSt"] 		= mProps;

	mappingClear(mProps);
    mProps["address"]      = ".ProcessInput.AlSt";
    mProps["dpe"]          = "StsReg01";
    mProps["bitPosition"]  = CPC_StsReg01_ALST;
    mProps["isAlarm"]      = true;
    mConfig["AlSt"]        = mProps;

    mappingClear(mProps);
    mProps["address"]      = ".ProcessInput.AlUnAck";
    mProps["dpe"]          = "StsReg01";
    mProps["bitPosition"]  = CPC_StsReg01_ALUNACK;
    mConfig["AlUnAck"]     = mProps;

	mappingClear(mProps);
    mProps["address"] 		  = ".ProcessOutput.ManReg01";
    mProps["dataType"] 		  = CPC_UINT16;
    mConfig["ManReg01"] 		= mProps;

    mappingClear(mProps);
    mProps["address"] 		  = ".ProcessOutput.MPosR";
    mProps["dataType"] 		  = CPC_FLOAT;
    mConfig["MPosR"] 		    = mProps;

	return mConfig;
}

dyn_string CPC_TestObjectConfig_getParamNames() {
    return makeDynString(CPC_PARAMS_FSPOSON, CPC_PARAMS_MRESTART, CPC_PARAMS_RSTARTFS, CPC_PARAMS_HLDRIVE);
}

CPC_TestObjectConfig_ExportConfig(dyn_string dsDpList, dyn_string &dsExceptionInfo) {
    string sObject = UN_CONFIG_CPC_TESTOBJECT_DPT_NAME;
    int iAmount = dynlen(dsDpList);

    for (int i = 1; i <= iAmount; i++) {
        dyn_string dsDpParameters;
        string sCurrentDp = dsDpList[i];

        if (!dpExists(sCurrentDp)) { continue; }

        unExportDevice_getAllCommonParameters(sCurrentDp, sObject, dsDpParameters);
        cpcExportGenericFunctions_getUnit(sCurrentDp, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getFormat(sCurrentDp, dsDpParameters, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getRange(sCurrentDp, dsDpParameters, dsExceptionInfo, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getDeadband(sCurrentDp, dsDpParameters, dsExceptionInfo, ".ProcessInput.PosSt");
        cpcExportGenericFunctions_getArchive(sCurrentDp, dsDpParameters);
        cpcExportGenericFunctions_processAddress(sCurrentDp, sObject, "StsReg01", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(sCurrentDp, sObject, "evStsReg01", 	true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(sCurrentDp, sObject, "PosSt", 		true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(sCurrentDp, sObject, "AuPosRSt",    true, dsDpParameters);
        cpcExportGenericFunctions_processAddress(sCurrentDp, sObject, "ManReg01", 	false, dsDpParameters);
        cpcExportGenericFunctions_processAddress(sCurrentDp, sObject, "MPosR", 		false, dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(sCurrentDp, UN_CONFIG_EXPORT_ARCHIVE_1, makeDynString(), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(sCurrentDp, UN_CONFIG_EXPORT_ARCHIVE_3, makeDynString("ProcessInput.evStsReg01"), dsDpParameters);
        cpcExportGenericFunctions_getArchiveNameForDpes(sCurrentDp, UN_CONFIG_EXPORT_ARCHIVE_2, makeDynString("ProcessInput.PosSt"), dsDpParameters);
        cpcExportGenericFunctions_getMaskEvent(sCurrentDp, dsDpParameters);
        cpcExportGenericFunctions_getParameters(sCurrentDp, dsDpParameters, dsExceptionInfo);
        cpcExportGenericFunctions_getMetainfo(sCurrentDp, dsDpParameters, dsExceptionInfo);
        unExportDevice_writeDsStringToFile(dsDpParameters, UN_PARAMETER_DELIMITER);
    }
}

cpcButton_TestButton(string sDeviceName, string sDpType, bool bButtonEnabled, bool bSelected, dyn_string dsData, dyn_string &dsExceptionInfo) {
    dsExceptionInfo = makeDynString("generic");
}

cpcButton_DT_TestButton(string sDeviceName, string sDpType, bool bButtonEnabled, bool bSelected, dyn_string dsData, dyn_string &dsExceptionInfo) {
    dsExceptionInfo = makeDynString("specific");
}

// Test setup helping variables and functions
string TEST_DP_NAME = "my_test_dp";

dyn_string makeEmptyDynString(int iAmount) {
    dyn_string dsResult;

    for (int i = 1; i <= iAmount; i++) {
        dynAppend(dsResult, "");
    }
    return dsResult;
}

void createBinaryAlert() {
    dpSet(TEST_DP_NAME + ".ProcessInput.AlSt:_alert_hdl.._type", DPCONFIG_ALERT_BINARYSIGNAL,
          TEST_DP_NAME + ".ProcessInput.AlSt:_alert_hdl.._ok_range", 0,
          TEST_DP_NAME + ".ProcessInput.AlSt:_alert_hdl.._class", "_cpcAnalog_Ack.",
          TEST_DP_NAME + ".ProcessInput.AlSt:_alert_hdl.._orig_hdl", TRUE);
}

void createAlert() {
    dpSet(TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.._type", DPCONFIG_ALERT_NONBINARYSIGNAL,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.1._type", DPDETAIL_RANGETYPE_MINMAX,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.2._type", DPDETAIL_RANGETYPE_MINMAX,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.3._type", DPDETAIL_RANGETYPE_MINMAX,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.4._type", DPDETAIL_RANGETYPE_MINMAX,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.5._type", DPDETAIL_RANGETYPE_MINMAX,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.1._u_limit", 1.0,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.2._l_limit", 1.0,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.2._u_limit", 2.0,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.3._l_limit", 2.0,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.3._u_limit", 3.0,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.4._l_limit", 3.0,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.4._u_limit", 4.0,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.5._l_limit", 4.0,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.1._l_incl", TRUE,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.1._u_incl", FALSE,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.2._l_incl", TRUE,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.2._u_incl", FALSE,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.3._l_incl", TRUE,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.3._u_incl", FALSE,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.4._l_incl", TRUE,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.4._u_incl", FALSE,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.5._l_incl", TRUE,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.5._u_incl", TRUE,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.1._class", "_cpcAnalogLL.",
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.2._class", "_cpcAnalogL.",
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.4._class", "_cpcAnalogH.",
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.5._class", "_cpcAnalogHH.",
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.._orig_hdl", TRUE);
}
