#uses "fwUnitTestComponentAsserts.ctl"
#uses "CPC_declarations.ctl"
#uses "S7Constant_declarations.ctl"
#uses "fwConfigs/fwPeriphAddress.ctl"
#uses "cpcConfigGenericFunctions.ctl"
#uses "cpcDpFunc.ctl"
#uses "test/unCPC6/cpcTestObjectConfig.ctl"

void test_cpcButton_isAckEnabled_CPC_ProcessControlObject(){
    assertFalse(cpcButton_isAckEnabled("device_name", "CPC_ProcessControlObject", 0, true, 0, false), "Assertion failed for invalid StsReg01");
    assertFalse(cpcButton_isAckEnabled("device_name", "CPC_ProcessControlObject", 0, false, 0, true), "Assertion failed for invalid StsReg02");
    assertFalse(cpcButton_isAckEnabled("device_name", "CPC_ProcessControlObject", 0, false, 5000, false), "Assertion failed for active SoftLDSt");
    assertTrue(cpcButton_isAckEnabled("device_name", "CPC_ProcessControlObject", 32, false, 0, false), "Assertion failed for blank StsReg02");
    assertTrue(cpcButton_isAckEnabled("device_name", "CPC_ProcessControlObject", 67, false, 1000, false), "Assertion failed for inactive SoftLDSt");
}

void test_cpcButton_isAckEnabled_CPC_MassFlowController(){
    assertFalse(cpcButton_isAckEnabled("device_name", "CPC_MassFlowController", 0, true, 0, false), "Assertion failed for invalid StsReg01");
    assertFalse(cpcButton_isAckEnabled("device_name", "CPC_MassFlowController", 0, false, 0, true), "Assertion failed for invalid StsReg02");
    assertFalse(cpcButton_isAckEnabled("device_name", "CPC_MassFlowController", 0, false, 5000, false), "Assertion failed for active SoftLDSt");
    assertTrue(cpcButton_isAckEnabled("device_name", "CPC_MassFlowController", 32, false, 0, false), "Assertion failed for blank StsReg02");
    assertTrue( cpcButton_isAckEnabled("device_name", "CPC_MassFlowController", 67, false, 1000, false), "Assertion failed for inactive SoftLDSt");
}

void test_cpcButton_isAckEnabled_CPC_OtherDevices(){
    assertFalse(cpcButton_isAckEnabled("device_name", "CPC_DigitalInput", 0, true, 0, false), "Assertion failed for invalid StsReg01");
    assertFalse(cpcButton_isAckEnabled("device_name", "CPC_DigitalInput", 0, false, 0, true), "Assertion failed for invalid StsReg02");
    assertFalse(cpcButton_isAckEnabled("device_name", "CPC_DigitalInput", 0, false, 5000, false), "Assertion failed for active SoftLDSt");
    assertTrue(cpcButton_isAckEnabled("device_name", "CPC_DigitalInput", 32, false, 0, false), "Assertion failed for blank StsReg02");
    assertTrue( cpcButton_isAckEnabled("device_name", "CPC_DigitalInput", 67, false, 1000, false), "Assertion failed for inactive SoftLDSt");
}

void test_cpcButton_IsAllowRestartEnabled_CPC_ProcessControlObject() {
    assertFalse(cpcButton_IsAllowRestartEnabled("device_name", "CPC_ProcessControlObject", 163, true, 1134, false, false),
                "Assertion failed for invalid StsReg01");
    assertFalse(cpcButton_IsAllowRestartEnabled("device_name", "CPC_ProcessControlObject", 167, false, 1234, true, false),
                "Assertion failed for invalid StsReg02");
    assertTrue(cpcButton_IsAllowRestartEnabled("device_name", "CPC_ProcessControlObject", 40, false, 0, false, false),
               "Assertion failed for blank StsReg01 and StsReg02");
    assertTrue(cpcButton_IsAllowRestartEnabled("device_name", "CPC_ProcessControlObject", 0, false, 0, false, true),
               "Assertion failed for RstartFS and blank StsReg02");
    assertFalse(cpcButton_IsAllowRestartEnabled("device_name", "CPC_ProcessControlObject", 512, false, 0, false, false),
                "Assertion failed for active FuStopAl");
    assertTrue(cpcButton_IsAllowRestartEnabled("device_name", "CPC_ProcessControlObject", 0, false, 1024, false, false),
               "Assertion failed for blank StsReg01 and inactive SoftLDSt");
    assertTrue(cpcButton_IsAllowRestartEnabled("device_name", "CPC_ProcessControlObject", 1024, false, 0, false, false),
               "Assertion failed for inactive FuStopAl and blank StsReg02");
    assertFalse(cpcButton_IsAllowRestartEnabled("device_name", "CPC_ProcessControlObject", 0, false, 5000, false, false),
                "Assertion failed for active SoftLDSt");
}

void test_cpcButton_IsAllowRestartEnabled_CPC_OtherDevices() {
    assertFalse(cpcButton_IsAllowRestartEnabled("device_name", "CPC_MassFlowController", 167, true, 1234, false, false),
                "Assertion failed for invalid StsReg01");
    assertFalse(cpcButton_IsAllowRestartEnabled("device_name", "CPC_MassFlowController", 167, false, 1234, true, false),
                "Assertion failed for invalid StsReg02");
    assertTrue(cpcButton_IsAllowRestartEnabled("device_name", "CPC_MassFlowController", 0, false, 0, false, true),
               "Assertion failed for RstartFS and blank StsReg02");
    assertFalse(cpcButton_IsAllowRestartEnabled("device_name", "CPC_MassFlowController", 0, false, 1024, false, false),
                "Assertion failed for active FuStopISt");
    assertTrue(cpcButton_IsAllowRestartEnabled("device_name", "CPC_MassFlowController", 0, false, 1000, false, false),
               "Assertion failed for inactive FuStopISt and SoftLDSt");
    assertFalse(cpcButton_IsAllowRestartEnabled("device_name", "CPC_MassFlowController", 0, false, 5000, false, false),
                "Assertion failed for active SoftLDSt");
}

void test_cpcButton_ButtonSetState() {
    dyn_string dsUserAccess, dsData, dsExceptionInfo;
    dyn_string dsButtons = makeDynString("TestButton");
    string sDeviceName;

    dsUserAccess = makeDynString("");
    dsData = makeDynString("", "");
    sDeviceName = "devname";
    cpcButton_ButtonSetState(dsButtons, dsUserAccess, sDeviceName, "", dsData, dsExceptionInfo);
    assertEqual(makeDynString("generic"), dsExceptionInfo, "Assertion failed for empty DpType");
    dynClear(dsExceptionInfo);

    cpcButton_ButtonSetState(dsButtons, dsUserAccess, sDeviceName, "DT1", dsData, dsExceptionInfo);
    assertEqual(makeDynString("generic"), dsExceptionInfo, "Assertion failed for DT1");
    dynClear(dsExceptionInfo);

    cpcButton_ButtonSetState(dsButtons, dsUserAccess, sDeviceName, "DT", dsData, dsExceptionInfo);
    assertEqual(makeDynString("specific"), dsExceptionInfo, "Assertion failed for DT");
    dynClear(dsExceptionInfo);
}
