#uses "fwUnitTestComponentAsserts.ctl"
#uses "CPC_declarations.ctl"
#uses "S7Constant_declarations.ctl"
#uses "fwConfigs/fwPeriphAddress.ctl"
#uses "cpcConfigGenericFunctions.ctl"
#uses "test/unCPC6/cpcTestObjectConfig.ctl"

void test_setup() {
    dpCreate(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
}

void test_teardown() {
   dpDelete(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
}

void test_splitCommaSeparatedString() {
    assertEqual(makeDynString(), splitCommaSeparatedString(""), "Assertion failed for empty string");
    assertEqual(makeDynString("a"), splitCommaSeparatedString("a"), "Assertion failed for plain string");
    assertEqual(makeDynString("a", "b"), splitCommaSeparatedString("a,b"), "Assertion failed for two values");
    assertEqual(makeDynString("a", "b"), splitCommaSeparatedString("a,b,"), "Assertion failed for comma ending");
    assertEqual(makeDynString("a", "b", ""), splitCommaSeparatedString("a,b,,"), "Assertion failed for empty string comma separated");
    assertEqual(makeDynString("", "", ""), splitCommaSeparatedString(",,,"), "Assertion failed for three empty strings");
}

void test_cpcConfigGenericFunctions_translateDataTypeForS7Check() {
    // Explicit data types
    assertEqual(UN_CONFIG_S7_PLC_DATATYPE_WORD, cpcConfigGenericFunctions_translateDataTypeForS7Check(CPC_INT16, ""), "Assertion failed for int16");
    assertEqual(UN_CONFIG_S7_PLC_DATATYPE_DWORD, cpcConfigGenericFunctions_translateDataTypeForS7Check(CPC_INT32, ""), "Assertion failed for int32");
    assertEqual(UN_CONFIG_S7_PLC_DATATYPE_WORD, cpcConfigGenericFunctions_translateDataTypeForS7Check(CPC_UINT16, ""), "Assertion failed for uint16");
    assertEqual(UN_CONFIG_S7_PLC_DATATYPE_BOOL, cpcConfigGenericFunctions_translateDataTypeForS7Check(CPC_BOOL, ""), "Assertion failed for bool");
    assertEqual(UN_CONFIG_S7_PLC_DATATYPE_FLOAT, cpcConfigGenericFunctions_translateDataTypeForS7Check(CPC_FLOAT, ""), "Assertion failed for float");  // DB10.DBD0

    // Implicit data types
    assertEqual(UN_CONFIG_S7_PLC_DATATYPE_WORD, cpcConfigGenericFunctions_translateDataTypeForS7Check(0, "DB10.DBW0"), "Assertion failed for DBW");
    assertEqual(UN_CONFIG_S7_PLC_DATATYPE_BOOL, cpcConfigGenericFunctions_translateDataTypeForS7Check(0, "DB10.DBX0.0"), "Assertion failed for DBX");
    assertEqual(UN_CONFIG_S7_PLC_DATATYPE_BYTE, cpcConfigGenericFunctions_translateDataTypeForS7Check(0, "DB10.DBB0"), "Assertion failed for DBB");
}

void test_cpcConfigGenericFunctions_translateDataType() {
    // Modbus
    assertEqual(PVSS_MODBUS_INT16, cpcConfigGenericFunctions_translateDataType(UN_PLC_DPTYPE, CPC_INT16), "Assertion failed for modbus int16");
    assertEqual(PVSS_MODBUS_INT32, cpcConfigGenericFunctions_translateDataType(UN_PLC_DPTYPE, CPC_INT32), "Assertion failed for modbus int32");
    assertEqual(PVSS_MODBUS_UINT16, cpcConfigGenericFunctions_translateDataType(UN_PLC_DPTYPE, CPC_UINT16), "Assertion failed for modbus uint16");
    assertEqual(PVSS_MODBUS_BOOL, cpcConfigGenericFunctions_translateDataType(UN_PLC_DPTYPE, CPC_BOOL), "Assertion failed for modbus bool");
    assertEqual(PVSS_MODBUS_FLOAT, cpcConfigGenericFunctions_translateDataType(UN_PLC_DPTYPE, CPC_FLOAT), "Assertion failed for modbus float");

    // S7
    assertEqual(PVSS_S7_INT16, cpcConfigGenericFunctions_translateDataType(S7_PLC_DPTYPE, CPC_INT16), "Assertion failed for S7 int16");
    assertEqual(PVSS_S7_INT32, cpcConfigGenericFunctions_translateDataType(S7_PLC_DPTYPE, CPC_INT32), "Assertion failed for S7 int32");
    assertEqual(PVSS_S7_UINT16, cpcConfigGenericFunctions_translateDataType(S7_PLC_DPTYPE, CPC_UINT16), "Assertion failed for S7 uint16");
    assertEqual(PVSS_S7_BOOL, cpcConfigGenericFunctions_translateDataType(S7_PLC_DPTYPE, CPC_BOOL), "Assertion failed for S7 bool");
    assertEqual(PVSS_S7_FLOAT, cpcConfigGenericFunctions_translateDataType(S7_PLC_DPTYPE, CPC_FLOAT), "Assertion failed for S7 float");

    // OPCUA
    assertEqual(OPCUA_INT16, cpcConfigGenericFunctions_translateDataType(CPC_OPCUA_DPTYPE, CPC_INT16), "Assertion failed for OPCUA int16");
    assertEqual(OPCUA_INT32, cpcConfigGenericFunctions_translateDataType(CPC_OPCUA_DPTYPE, CPC_INT32), "Assertion failed for OPCUA int32");
    assertEqual(OPCUA_UINT16, cpcConfigGenericFunctions_translateDataType(CPC_OPCUA_DPTYPE, CPC_UINT16), "Assertion failed for OPCUA uint16");
    assertEqual(OPCUA_BOOL, cpcConfigGenericFunctions_translateDataType(CPC_OPCUA_DPTYPE, CPC_BOOL), "Assertion failed for OPCUA bool");
    assertEqual(OPCUA_FLOAT, cpcConfigGenericFunctions_translateDataType(CPC_OPCUA_DPTYPE, CPC_FLOAT), "Assertion failed for OPCUA float");

    // TODO BacNet
    assertEqual(800, cpcConfigGenericFunctions_translateDataType(CPC_BACNET_DPTYPE, CPC_INT16), "Assertion failed for BacNet int16");
    assertEqual(800, cpcConfigGenericFunctions_translateDataType(CPC_BACNET_DPTYPE, CPC_INT32), "Assertion failed for BacNet int32");
    assertEqual(800, cpcConfigGenericFunctions_translateDataType(CPC_BACNET_DPTYPE, CPC_UINT16), "Assertion failed for BacNet uint16");
    assertEqual(800, cpcConfigGenericFunctions_translateDataType(CPC_BACNET_DPTYPE, CPC_BOOL), "Assertion failed for BacNet bool");
    assertEqual(800, cpcConfigGenericFunctions_translateDataType(CPC_BACNET_DPTYPE, CPC_FLOAT), "Assertion failed for BacNet float");

    // TODO SNMP
    assertEqual(661, cpcConfigGenericFunctions_translateDataType(CPC_SNMP_DPTYPE, CPC_INT16), "Assertion failed for SNMP int16");
    assertEqual(661, cpcConfigGenericFunctions_translateDataType(CPC_SNMP_DPTYPE, CPC_INT32), "Assertion failed for SNMP int32");
    assertEqual(661, cpcConfigGenericFunctions_translateDataType(CPC_SNMP_DPTYPE, CPC_UINT16), "Assertion failed for SNMP uint16");
    assertEqual(661, cpcConfigGenericFunctions_translateDataType(CPC_SNMP_DPTYPE, CPC_BOOL), "Assertion failed for SNMP bool");
    assertEqual(661, cpcConfigGenericFunctions_translateDataType(CPC_SNMP_DPTYPE, CPC_FLOAT), "Assertion failed for SNMP float");

    // TODO IEC104
    assertEqual(PVSS_S7_INT16, cpcConfigGenericFunctions_translateDataType(CPC_IEC104_DPTYPE, CPC_INT16), "Assertion failed for IEC104 int16");
    assertEqual(PVSS_S7_INT32, cpcConfigGenericFunctions_translateDataType(CPC_IEC104_DPTYPE, CPC_INT32), "Assertion failed for IEC104 int32");
    assertEqual(PVSS_S7_UINT16, cpcConfigGenericFunctions_translateDataType(CPC_IEC104_DPTYPE, CPC_UINT16), "Assertion failed for IEC104 uint16");
    assertEqual(PVSS_S7_BOOL, cpcConfigGenericFunctions_translateDataType(CPC_IEC104_DPTYPE, CPC_BOOL), "Assertion failed for IEC104 bool");
    assertEqual(PVSS_S7_FLOAT, cpcConfigGenericFunctions_translateDataType(CPC_IEC104_DPTYPE, CPC_FLOAT), "Assertion failed for IEC104 float");

    // Default
    assertEqual(PVSS_S7_INT16, cpcConfigGenericFunctions_translateDataType("", CPC_INT16), "Assertion failed for default int16");
    assertEqual(PVSS_S7_INT32, cpcConfigGenericFunctions_translateDataType("", CPC_INT32), "Assertion failed for default int32");
    assertEqual(PVSS_S7_UINT16, cpcConfigGenericFunctions_translateDataType("", CPC_UINT16), "Assertion failed for default uint16");
    assertEqual(PVSS_S7_BOOL, cpcConfigGenericFunctions_translateDataType("", CPC_BOOL), "Assertion failed for default bool");
    assertEqual(PVSS_S7_FLOAT, cpcConfigGenericFunctions_translateDataType("", CPC_FLOAT), "Assertion failed for default float");
}

void test_cpcConfigGenericFunctions_processArchives() {
    int iValuesToSkip = 6;
    dyn_string dsConfigs;

    // No archive
    cpcConfigGenericFunctions_processArchives(1, false, dsConfigs);
    assertEmpty(dsConfigs, "Assertion failed for no archive");

    // Empty configs
    dsConfigs = makeEmptyDynString(iValuesToSkip + dynMax(
        makeDynInt(UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL, UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG, UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT)));
    cpcConfigGenericFunctions_processArchives(1, true, dsConfigs);
    assertEqual("", dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL], "Assertion failed for bool archive of empty configs");
    assertEqual("", dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG], "Assertion failed for analog archive of empty configs");
    assertEqual("", dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT], "Assertion failed for event archive of empty configs");

    // Set configs
    dsConfigs[iValuesToSkip + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] = "bool";
    dsConfigs[iValuesToSkip + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] = "analog";
    dsConfigs[iValuesToSkip + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] = "event";
    cpcConfigGenericFunctions_processArchives(iValuesToSkip, true, dsConfigs);
    assertEqual("bool", dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_BOOL], "Assertion failed for bool archive of set configs");
    assertEqual("analog", dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG], "Assertion failed for analog archive of set configs");
    assertEqual("event", dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT], "Assertion failed for event archive of set configs");
}

void test_cpcConfigGenericFunctions_alertWithMail() {
    dyn_string dsExceptionInfo;

    // No category, false value
    cpcGenericDpFunctions_setDeviceProperty(TEST_DP_NAME, "ALERT_NAME_NO_MAIL", "FALSE", dsExceptionInfo);
    assertTrue(cpcConfigGenericFunctions_alertWithMail(TEST_DP_NAME, "ALERT_NAME"), "Assertion failed for false no mail");

    // No category, true value
    cpcGenericDpFunctions_setDeviceProperty(TEST_DP_NAME, "ALERT_NAME_NO_MAIL", "TRUE", dsExceptionInfo);
    assertFalse(cpcConfigGenericFunctions_alertWithMail(TEST_DP_NAME, "ALERT_NAME"), "Assertion failed for no mail");

    // Category, false value
    cpcGenericDpFunctions_setDeviceProperty(TEST_DP_NAME, "CATEGORY_ALERT_NAME_NO_MAIL", "FALSE", dsExceptionInfo);
    assertEqual(cpcConfigGenericFunctions_alertWithMail(TEST_DP_NAME, "ALERT_NAME"),
                cpcConfigGenericFunctions_alertWithMail(TEST_DP_NAME, "ALERT_NAME", "CATEGORY"),
                "Assertion failed for false category no mail");

    // Category, true value
    cpcGenericDpFunctions_setDeviceProperty(TEST_DP_NAME, "CATEGORY_ALERT_NAME_NO_MAIL", "TRUE", dsExceptionInfo);
    assertFalse(cpcConfigGenericFunctions_alertWithMail(TEST_DP_NAME, "ALERT_NAME", "CATEGORY"), "Assertion failed for category no mail");
}

void test_cpcConfigGenericFunctions_createAlarmDescription() {
    string sDomain = UN_CONFIG_CPC_TESTOBJECT_DPT_NAME + "_DOMAIN";
    string sNature = UN_CONFIG_CPC_TESTOBJECT_DPT_NAME + "_NATURE";
    string sAlias = UN_CONFIG_CPC_TESTOBJECT_DPT_NAME + "_ALIAS";
    string sDescription = UN_CONFIG_CPC_TESTOBJECT_DPT_NAME + "_DESCRIPTION";
    string sAlarmName = UN_CONFIG_CPC_TESTOBJECT_DPT_NAME + "_ALARM";
    string sDefaultPanel = UN_CONFIG_CPC_TESTOBJECT_DPT_NAME + "_DEFAULT_PANEL";
    dyn_string dsDescription, dsExpectedDescription;

    dsExpectedDescription[UN_CONFIG_ALARM_DOMAIN]       = sDomain;
    dsExpectedDescription[UN_CONFIG_ALARM_NATURE]       = sNature;
    dsExpectedDescription[UN_CONFIG_ALARM_ALIAS]        = sAlias;
    dsExpectedDescription[UN_CONFIG_ALARM_DESCRIPTION]  = sDescription;
    dsExpectedDescription[UN_CONFIG_ALARM_NAME]         = sAlarmName;
    dsExpectedDescription[UN_CONFIG_ALARM_DEFAULTPANEL] = sDefaultPanel;

    cpcConfigGenericFunctions_createAlarmDescription(sDomain, sNature, sAlias, sDescription, sAlarmName, sDefaultPanel, dsDescription);
    assertEqual(dsExpectedDescription, dsDescription, "Assertion failed for alarm description");
}

void test_cpcConfigGenericFunctions_get5AlertConfig() {
    dyn_float dfLevelLimits;
    dyn_string dsLevelsAlarms, dsLevelsAlertsClass;

    // All thresholds
    cpcConfigGenericFunctions_get5AlertConfig("0.25", "1", "1", "1", true, true, dfLevelLimits, dsLevelsAlarms, dsLevelsAlertsClass);
    assertEqual("0.25 | 1 | 1 | 1", dfLevelLimits, "Assertion failed for limits");
    assertEqual("LL | L | Ok | H | HH", dsLevelsAlarms, "Assertion failed for alarms");
    assertEqual("_cpcAnalogLL_Ack_Mail. | _cpcAnalogL_Mail. |  | _cpcAnalogH_Mail. | _cpcAnalogHH_Ack_Mail.", dsLevelsAlertsClass, "Assertion failed for classes");
    dynClear(dfLevelLimits);
    dynClear(dsLevelsAlarms);
    dynClear(dsLevelsAlertsClass);

    // Alarm thresholds only
    cpcConfigGenericFunctions_get5AlertConfig("0.25", "", "", "1", false, false, dfLevelLimits, dsLevelsAlarms, dsLevelsAlertsClass);
    assertEqual("0.25 | 1", dfLevelLimits, "Assertion failed for limits");
    assertEqual("LL | Ok | HH", dsLevelsAlarms, "Assertion failed for alarms");
    assertEqual("_cpcAnalogLL. |  | _cpcAnalogHH.", dsLevelsAlertsClass, "Assertion failed for classes");
    dynClear(dfLevelLimits);
    dynClear(dsLevelsAlarms);
    dynClear(dsLevelsAlertsClass);

    // No threasholds
    cpcConfigGenericFunctions_get5AlertConfig("", "", "", "", false, false, dfLevelLimits, dsLevelsAlarms, dsLevelsAlertsClass);
    assertEmpty(dfLevelLimits, "Assertion failed for limits");
    assertEqual("Ok", dsLevelsAlarms, "Assertion failed for alarms");
    assertEqual("", dsLevelsAlertsClass, "Assertion failed for classes");
    dynClear(dfLevelLimits);
    dynClear(dsLevelsAlarms);
    dynClear(dsLevelsAlertsClass);

    // Wrong value
    cpcConfigGenericFunctions_get5AlertConfig("a", "2.a", "3EXP5", "", false, false, dfLevelLimits, dsLevelsAlarms, dsLevelsAlertsClass);
    assertEqual("0 | 2 | 3", dfLevelLimits, "Assertion failed for limits");
    assertEqual("LL | L | Ok | H", dsLevelsAlarms, "Assertion failed for alarms");
    assertEqual("_cpcAnalogLL. | _cpcAnalogL. |  | _cpcAnalogH.", dsLevelsAlertsClass, "Assertion failed for classes");
}

void test_cpcConfigGenericFunctions_getAlertClass() {
    // No acknowledge, no SMS
    assertEqual("_cpcAnalog", cpcConfigGenericFunctions_getAlertClass(false, false), "Assertion failed for standard alarm");

    // Acknowledge
    assertEqual("_cpcAnalogLL_Ack", cpcConfigGenericFunctions_getAlertClass(true, false, "LL"), "Assertion failed for acknowledge alarm");

    // Acknowledge and SMS
    assertEqual("_cpcAnalogH_Ack_Mail", cpcConfigGenericFunctions_getAlertClass(true, true, "H"), "Assertion failed for acknowledge and SMS alarm");

    // SMS
    assertEqual("_cpcAnalogL_Mail", cpcConfigGenericFunctions_getAlertClass(false, true, "L"), "Assertion failed for SMS alarm");
}

void test_cpcConfigGenericFunctions_getConstant() {
    // Device constant
    assertEqual(UN_CONFIG_CPC_TESTOBJECT_LENGTH, cpcConfigGenericFunctions_getConstant(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "LENGTH"), "Assertion failed for constant");

    // Wrong device constant
    assertEqual(-1, cpcConfigGenericFunctions_getConstant(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "MissedProperty"), "Assertion failed for wrong constant");

    // Wrong device
    assertEqual(-1, cpcConfigGenericFunctions_getConstant("CPC_MissedObject", "TESTPROPERTY"), "Assertion failed for wrong device");
}

void test_cpcConfigGenericFunctions_getConstantByName() {
    // Valid constant
    assertEqual(UN_ACCESS_RIGHTS_ADMIN, cpcConfigGenericFunctions_getConstantByName("UN_ACCESS_RIGHTS", "ADMIN"), "Assertion failed for constant");

    // Wrong constant
    assertEqual(-1, cpcConfigGenericFunctions_getConstantByName("UN_ACCESS_RIGHTS", "wrong"), "Assertion failed for wrong constat");
}

void test_cpcConfigGenericFunctions_getConstantPrefix() {
    // Any frontend
    assertEqual("UN_CONFIG_", cpcConfigGenericFunctions_getConstantPrefix(""), "Assertion failed for empty frontend");
    assertEqual("UN_CONFIG_", cpcConfigGenericFunctions_getConstantPrefix(UN_PLC_DPTYPE), "Assertion failed for UN frontend");
    assertEqual("UN_CONFIG_", cpcConfigGenericFunctions_getConstantPrefix(S7_PLC_DPTYPE), "Assertion failed for S7 frontend");
    assertEqual("UN_CONFIG_", cpcConfigGenericFunctions_getConstantPrefix("SOFT"), "Assertion failed for SOFT frontend");
    assertEqual("UN_CONFIG_", cpcConfigGenericFunctions_getConstantPrefix("CMW"), "Assertion failed for CMW frontend");
}

void test_cpcConfigGenericFunctions_getStringConstantByName() {
    // Valid constant
    assertEqual(UN_ACCESS_RIGHTS_ADMIN, cpcConfigGenericFunctions_getStringConstantByName("UN_ACCESS_RIGHTS_ADMIN"), "Assertion failed for constant");

    // Wrong constant
    assertEqual(CPC_UNDEFINED_CONSTANT_VALUE, cpcConfigGenericFunctions_getStringConstantByName("CPC_ACCESS_RIGHTS_wrong"), "Assertion failed for wrong constant");
}

void test_cpcConfigGenericFunctions_getConstantDPLength() {
    // Valid device type
    assertEqual(UN_CONFIG_CPC_TESTOBJECT_LENGTH, cpcConfigGenericFunctions_getConstantDPLength(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME), "Assertion failed for device type");

    // Wrong device type
    assertEqual(-1, cpcConfigGenericFunctions_getConstantDPLength("CPC_MissedObject"), "Assertion failed for wrong device type");
}

void test_cpcConfigGenericFunctions_getConstantDPAdditionalLength() {
    // Valid device type
    assertEqual(UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH, cpcConfigGenericFunctions_getConstantDPAdditionalLength(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME), "Assertion failed for device type");

    // Wrong device type
    assertEqual(0, cpcConfigGenericFunctions_getConstantDPAdditionalLength("CPC_MissedObject"), "Assertion failed for wrong device type");
}

void test_cpcConfigGenericFunctions_getConstantDPEAdress() {
    // Valid device type address
    assertEqual(UN_CONFIG_CPC_TESTOBJECT_ADDRESS_POSST, cpcConfigGenericFunctions_getConstantDPEAdress(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "POSST"), "Assertion failed for device type address");

    // Wrong device type
    assertEqual(-1, cpcConfigGenericFunctions_getConstantDPEAdress("CPC_MissedObject", "TESTADDRESS"), "Assertion failed for wrong device type");

    // Wrong address
    assertEqual(-1, cpcConfigGenericFunctions_getConstantDPEAdress(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "MissAddress"), "Assertion failed for wrong address");
}

void test_cpcConfigGenericFunctions_getConstantDPEProperty() {
    // Length without archive
    assertEqual(UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH,
                cpcConfigGenericFunctions_getConstantDPEProperty(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "" , false , "LENGTH"),
                "Assertion failed for length without archive");

    // Length with archive
    assertEqual(UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH,
                cpcConfigGenericFunctions_getConstantDPEProperty(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "" , true , "LENGTH"),
                "Assertion failed for length with archive");

    // Address PosSt
    assertEqual(UN_CONFIG_CPC_TESTOBJECT_ADDRESS_POSST,
                cpcConfigGenericFunctions_getConstantDPEProperty(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "ADDRESS" , true , "POSST"),
                "Assertion failed for address");

    // MaskEvent without archive
    assertEqual(UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_MASKEVENT,
                cpcConfigGenericFunctions_getConstantDPEProperty(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "" , false , "MASKEVENT"),
                "Assertion failed for MaskEvent without archive");

    // MaskEvent with archive
    assertEqual(UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_MASKEVENT + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH,
                cpcConfigGenericFunctions_getConstantDPEProperty(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "" , true , "MASKEVENT"),
                "Assertion failed for MaskEvent with archive");

    // Unit without archive
    assertEqual(UN_CONFIG_CPC_TESTOBJECT_UNIT, cpcConfigGenericFunctions_getConstantDPEProperty(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "" , false , "UNIT"),
                "Assertion failed for unit without archive");

    // Unit with archive
    assertEqual(UN_CONFIG_CPC_TESTOBJECT_UNIT, cpcConfigGenericFunctions_getConstantDPEProperty(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "" , true , "UNIT"),
                "Assertion failed for unit with archive");

    // Wrong address and property
    assertEqual(-1, cpcConfigGenericFunctions_getConstantDPEProperty(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "Missing" , true , "Missing"),
                "Assertion failed for wrong address and property");
}

void test_cpcConfigGenericFunctions_getDeviceParamNames() {
    dyn_string dsParams = makeDynString ("PFSPosOn", "PEnRstart", "PRstartFS", "PHLDrive");

    // Valid device type
    dyn_string dsResult = cpcConfigGenericFunctions_getDeviceParamNames(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
    assertEqual(dynlen(dsResult), dynlen(dsParams), "Assertion failed number of parameters returned");
    if (dynlen(dsResult) == dynlen(dsParams)) {
        for (int i = 1; i <= dynlen(dsResult); i++) {
            assertTrue(dynContains(dsParams, dsResult[i]), "Assertion failed for parameters returned containing " + dsResult[i]);
        }
    }

    // Wrong device type
    assertEqual("", cpcConfigGenericFunctions_getDeviceParamNames("CPC_MissedObject"), "Assertion failed for parameters returned by wrong DpType");
}

void test_cpcConfigGenericFunctions_getParametersMapping() {
    mapping mParameters;
    bool bHasArchive;
    dyn_string dsConfigs = makeEmptyDynString(UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);

    mParameters["a"] = "b";
    mParameters["c"] = "";
    mParameters["d"] = "e";
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_PARAMETERS] = "a=b,c,d=e";

    assertEqual(mParameters, cpcConfigGenericFunctions_getParametersMapping(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, bHasArchive), "Assertion failed for parameters");
}

void test_cpcConfigGenericFunctions_getDPEConfig() {
    mapping mAddresses = cpcConfigGenericFunctions_getDPEConfig(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
    dyn_string dsKeys2;
    dyn_string dsKeys = mappingKeys(mAddresses);

    for (int i =1; i<= dynlen(dsKeys);i++) {
        dsKeys2 = mappingKeys(mAddresses[dsKeys[i]]);
        assertTrue(mappingHasKey(mAddresses[dsKeys[i]], "dataType"), "Assertion failed for " + dsKeys[i] + " not containing dataType");
        assertTrue(dynlen(dsKeys2) >= 10, "Assertion failed for " + dsKeys[i] + " not containing at least 10 keys");
    }
}

void test_cpcConfigGenericFunctions_getRange() {
    string sRangeMax, sRangeMin;
    dyn_string dsConfigs = makeEmptyDynString(UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);

    // Empty range
    cpcConfigGenericFunctions_getRange(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "PosSt", sRangeMax, sRangeMin, false);
    assertEqual("", sRangeMax, "Assertion failed for empty max config");
    assertEqual("", sRangeMin, "Assertion failed for empty min config");

    // Int range
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_RANGEMAX] = "16";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_RANGEMIN] = "17";
    cpcConfigGenericFunctions_getRange(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "PosSt", sRangeMax, sRangeMin, false);
    assertEqual("16", sRangeMax, "Assertion failed for int max config");
    assertEqual("17", sRangeMin, "Assertion failed for int min config");

    // Float range
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_RANGEMAX] = "56.9";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_RANGEMIN] = "-10";
    cpcConfigGenericFunctions_getRange(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "PosSt", sRangeMax, sRangeMin, false);
    assertEqual("56.9", sRangeMax, "Assertion failed for float max config");
    assertEqual("-10", sRangeMin, "Assertion failed for float min config");

}

void test_cpcConfigGenericFunctions_getShortTypeName() {
    // CPC prefix
    assertEqual(cpcConfigGenericFunctions_getShortTypeName("CPC_Test"), "Test", "Assertion failed for CPC prefix");

    // No CPC prefix
    assertEqual(cpcConfigGenericFunctions_getShortTypeName("Test"), "Test", "Assertion failed for no CPC prefix");
}

void test_cpcConfigGenericFunctions_getFrontEndType() {
    dyn_string dsConfigs = makeEmptyDynString(UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);

    // UN
    dsConfigs[UN_CONFIG_OBJECTGENERAL_PLC_TYPE] = UN_CONFIG_PREMIUM;
    assertEqual(UN_PLC_DPTYPE, cpcConfigGenericFunctions_getFrontEndType(dsConfigs), "Assertion failed for Premium PLC");

    // S7
    dsConfigs[UN_CONFIG_OBJECTGENERAL_PLC_TYPE] = UN_CONFIG_S7_300;
    assertEqual(S7_PLC_DPTYPE, cpcConfigGenericFunctions_getFrontEndType(dsConfigs), "Assertion failed for 300 PLC");

    // Other FE
    dsConfigs[UN_CONFIG_OBJECTGENERAL_PLC_TYPE] = "ANYFRONTEND";
    assertEqual("ANYFRONTEND", cpcConfigGenericFunctions_getFrontEndType(dsConfigs), "Assertion failed for Undefined FrontEnd");
}

void test_cpcConfigGenericFunctions_createGlobalConst() {
    dyn_string dsExceptionInfo;

    // Create global const
    if (globalExists("g_mCpcConst")) {
        removeGlobal("g_mCpcConst");
    }
    assertTrue(cpcConfigGenericFunctions_createGlobalConst(dsExceptionInfo), "Assertion failed for creating global constants");
    assertEmpty(dsExceptionInfo, "Assertion failed for creating global constants");

    // Global const already exists
    assertFalse(cpcConfigGenericFunctions_createGlobalConst(dsExceptionInfo), "Assertion failed for overwriting global constants");
    assertError("ERROR", "Error creating global variable: g_mCpcConst", dsExceptionInfo);
    removeGlobal("g_mCpcConst");
}

void test_cpcConfigGenericFunctions_initConstants() {
    if (globalExists("g_mCpcConst")) {
        removeGlobal("g_mCpcConst");
    }
    cpcConfigGenericFunctions_initConstants();
    assertTrue(globalExists("g_mCpcConst"), "Assertion failed for initializing global constants");
}

void test_cpcConfigGenericFunctions_isOldSoftFe() {
    // No Soft FE
    assertFalse(cpcConfigGenericFunctions_isOldSoftFe("NoSoftFE"), "Assertion failed for wrong FrontEnd");

    // No new Soft FE mapping
    if (!globalExists("g_mCpcConst")) {
        addGlobal("g_mCpcConst");
    } else if (mappingHasKey(g_mCpcConst, "NEW_SOFT_FE")) {
        g_mCpcConst.remove("NEW_SOFT_FE");
    }
    assertTrue(cpcConfigGenericFunctions_isOldSoftFe(SOFT_FE_DPTYPE), "Assertion failed for no new Soft FE in global constants");

    // Explicit old Soft FE
    g_mCpcConst["NEW_SOFT_FE"] = FALSE;
    assertTrue(cpcConfigGenericFunctions_isOldSoftFe(SOFT_FE_DPTYPE), "Assertion failed for explicit no new Soft FE in global constants");

    // Explicit new Soft FE
    g_mCpcConst["NEW_SOFT_FE"] = TRUE;
    assertFalse(cpcConfigGenericFunctions_isOldSoftFe(SOFT_FE_DPTYPE), "Assertion failed for explicit new Soft FE in global constants");
    removeGlobal("g_mCpcConst");
}

void test_cpcConfigGenericFunctions_check5RangesAlert() {
    int iAlarmActivePos = 1;
    dyn_string dsExceptionInfo;
    dyn_string dsConfigs = makeEmptyDynString(UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);
    dyn_int diLimits = makeDynInt(1, 2, 3);

    // Wrong alarm value
    dsConfigs[UN_CONFIG_COMMON_LENGTH + iAlarmActivePos] = "missbool";
    cpcConfigGenericFunctions_check5RangesAlert(dsConfigs, iAlarmActivePos, diLimits, dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkBool: the value 'missbool' is not correct for the ALARM_ACTIVE", dsExceptionInfo);

    // Wrong diLimits length
    dsConfigs[UN_CONFIG_COMMON_LENGTH + iAlarmActivePos] = "false";
    cpcConfigGenericFunctions_check5RangesAlert(dsConfigs, iAlarmActivePos, diLimits, dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_check5RangesAlert: wrong amount of alert limits: 3 instead of 4.", dsExceptionInfo);

    // Equal limits
    diLimits = makeDynInt(2, 3, 4, 5);
    dsConfigs[UN_CONFIG_COMMON_LENGTH + diLimits[1]] = "4.5";
    dsConfigs[UN_CONFIG_COMMON_LENGTH + diLimits[2]] = "4.5";
    cpcConfigGenericFunctions_check5RangesAlert(dsConfigs, iAlarmActivePos, diLimits, dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_check5RangesAlert: limits are not consistent ([limit#2] 4.5 >= 4.5 [limit#1])", dsExceptionInfo);

    // Wrong limit value
    dsConfigs[UN_CONFIG_COMMON_LENGTH + diLimits[1]] = "4.5";
    dsConfigs[UN_CONFIG_COMMON_LENGTH + diLimits[2]] = "missedFloat";
    dsConfigs[UN_CONFIG_COMMON_LENGTH + diLimits[3]] = "6.5";
    dsConfigs[UN_CONFIG_COMMON_LENGTH + diLimits[4]] = "7.5";
    cpcConfigGenericFunctions_check5RangesAlert(dsConfigs, iAlarmActivePos, diLimits, dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_check5RangesAlert: wrong limit #3: missedFloat", dsExceptionInfo);

    // Valid limits
    dsConfigs[UN_CONFIG_COMMON_LENGTH + diLimits[1]] = "4.5";
    dsConfigs[UN_CONFIG_COMMON_LENGTH + diLimits[2]] = "5.5";
    dsConfigs[UN_CONFIG_COMMON_LENGTH + diLimits[3]] = "6.5";
    dsConfigs[UN_CONFIG_COMMON_LENGTH + diLimits[4]] = "7.5";
    cpcConfigGenericFunctions_check5RangesAlert(dsConfigs, iAlarmActivePos, diLimits, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for valid limits");
}

void test_cpcConfigGenericFunctions_checkAddressCMW() {
    dyn_string dsExceptionInfo;

    // Wrong address
    string sAddress = "MissedAddress";
    cpcConfigGenericFunctions_checkAddressCMW(sAddress,dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkAddressCMW: address 'MissedAddress' does not follow format TYPE|TRANSFORMATION_TYPE|POLL_GROUP|ADDRESS_CONFIG|FILTER_NAME|FILTER_VALUE|LLC", dsExceptionInfo);

    // Wrong address type
    sAddress = "8|2|2|1|1|1|1";
    cpcConfigGenericFunctions_checkAddressCMW(sAddress,dsExceptionInfo);
    assertError("ERROR", "unConfigCMW_checkType: type (spont, poll, SQ) 8", dsExceptionInfo);

    // Wrong address config
    sAddress = "2|2|2|1$2|1|1|1";
    cpcConfigGenericFunctions_checkAddressCMW(sAddress,dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkAddressCMW: wrong CMW address  '1$2'", dsExceptionInfo);

    // Right address
    sAddress = "2|2|2|1$2$3|1|1|1";
    cpcConfigGenericFunctions_checkAddressCMW(sAddress,dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for valid address");
}

void test_cpcConfigGenericFunctions_checkAddressSNMP() {
    dyn_string dsExceptionInfo;

    // Wrong address format
    string sAddress = "$$$$$";
    cpcConfigGenericFunctions_checkAddressSNMP(sAddress, dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkAddressSNMP: wrong address format: $$$$$", dsExceptionInfo);

    // Wrong address
    sAddress = "WrongAddress";
    cpcConfigGenericFunctions_checkAddressSNMP(sAddress, dsExceptionInfo);
    assertError("ERROR", " fwPeriphAddressSNMP_check: Invalid OID \"WrongAddress\". OIDs must always begin with \"1.3.6.1\"", dsExceptionInfo);

    // Wrong address subindex
    sAddress = "1.3.6.1$$a$";
    cpcConfigGenericFunctions_checkAddressSNMP(sAddress, dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkAddressSNMP: wrong subindex:a", dsExceptionInfo);

    // Wrong address subindex value
    sAddress = "1.3.6.1$$-1$";
    cpcConfigGenericFunctions_checkAddressSNMP(sAddress, dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkAddressSNMP: wrong subindex value: -1", dsExceptionInfo);

    // Wrong address transformation
    sAddress = "1.3.6.1$WrongType$$";
    cpcConfigGenericFunctions_checkAddressSNMP(sAddress, dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkAddressSNMP: wrong transformation type: WrongType", dsExceptionInfo);

    // Valid address
    sAddress = "1.3.6.1$$$";
    cpcConfigGenericFunctions_checkAddressSNMP(sAddress, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for valid address");
}

void test_cpcConfigGenericFunctions_checkBool() {
    bool bRes;
    dyn_string dsExceptionInfo;
    dyn_string dsTrueCases = makeDynString("true", "1", "yes", "y", "True", "TRUE", "YES", "Y");

    // Valid true cases
    for (int i = 1; i <= dynlen(dsTrueCases); i++) {
        cpcConfigGenericFunctions_checkBool("test", dsTrueCases[i], bRes, dsExceptionInfo);
        assertTrue(bRes, "Assertion failed for valid true value");
        assertEmpty(dsExceptionInfo, "Assertion failed for valid true value");
    }

    // Valid false cases
    dyn_string dsFalseCases = makeDynString("false", "0", "no", "n", "False", "FALSE", "NO", "N");
    for (int i = 1; i <= dynlen(dsFalseCases); i++) {
        cpcConfigGenericFunctions_checkBool("test", dsFalseCases[i], bRes, dsExceptionInfo);
        assertFalse(bRes, "Assertion failed for valid false value");
        assertEmpty(dsExceptionInfo, "Assertion failed for valid false value");
    }

    // Wrong value
    cpcConfigGenericFunctions_checkBool("test", "ok", bRes, dsExceptionInfo);
    assertFalse(bRes, "Assertion failed for invalid value");
    assertError("ERROR", "cpcConfigGenericFunctions_checkBool: the value 'ok' is not correct for the test", dsExceptionInfo);
}

void test_cpcConfigGenericFunctions_checkConfigLength() {
    dyn_string dsConfigs;
    bool bHasArchive;

    // Wrong config length
    assertFalse(cpcConfigGenericFunctions_checkConfigLength(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, dsConfigs, bHasArchive), "Assertion failed for wrong configs length");

    // Config length with archive
    dsConfigs = makeEmptyDynString(UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH);
    assertTrue(cpcConfigGenericFunctions_checkConfigLength(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, dsConfigs, bHasArchive), "Assertion failed for configs with archive length");
    assertTrue(bHasArchive, "Assertion failed for archives detection in configs");

    // Config length without archive
    dsConfigs = makeEmptyDynString(UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);
    assertTrue(cpcConfigGenericFunctions_checkConfigLength(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, dsConfigs, bHasArchive), "Assertion failed for configs without archive length");
    assertFalse(bHasArchive, "Assertion failed for no archives detection in configs");
}

void test_cpcConfigGenericFunctions_checkDeviceAddress() {
    string sDeviceType = UN_CONFIG_CPC_TESTOBJECT_DPT_NAME;
    string sAddressName = "POSST";
    mapping mAddrProps;
    dyn_string dsExceptionInfo;
    dyn_string dsConfigs = makeEmptyDynString(UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);

    // UN Wrong address
    cpcConfigGenericFunctions_checkDeviceAddress(dsConfigs, "_UnPlc", sDeviceType, sAddressName, mAddrProps, false, dsExceptionInfo);
    assertError("ERROR", "unConfigGenericFunctions_checkUnsigned: bad data.", dsExceptionInfo);

    // UN Valid address
    cpcConfigGenericFunctions_checkDeviceAddress(dsConfigs, "_UnPlc", sDeviceType, "1097", mAddrProps, false, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for valid UN address");

    // S7 Wrong address
    mAddrProps["dataType"] = 1;
    dsConfigs = makeEmptyDynString (UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);
    cpcConfigGenericFunctions_checkDeviceAddress(dsConfigs, "S7_PLC", sDeviceType, sAddressName, mAddrProps, false, dsExceptionInfo);
    assertError("ERROR", "unConfigGenericFunctions_checkAddressS7:POSST bad data.", dsExceptionInfo);

    // S7 Valid address
    cpcConfigGenericFunctions_checkDeviceAddress(dsConfigs, "S7_PLC", sDeviceType, "DB10.DBW10", mAddrProps, false, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for valid S7 address");

    // Valid frontend CMW
    dsConfigs = makeEmptyDynString(UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);
    cpcConfigGenericFunctions_checkDeviceAddress(dsConfigs, "CMW", sDeviceType, "2|2|2|1$2$3|1|1|1", mAddrProps, false, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for valid CMW address");

    // Wrong frontend
    cpcConfigGenericFunctions_checkDeviceAddress(dsConfigs, "", sDeviceType, sAddressName, mAddrProps, false, dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkDeviceAddress: unknown front end ", dsExceptionInfo);
}

void test_cpcConfigGenericFunctions_checkDeviceConfig() {
    string sDeviceType = UN_CONFIG_CPC_TESTOBJECT_DPT_NAME;
    dyn_string dsConfigs, dsExceptionInfo;

    // Empty line
    cpcConfigGenericFunctions_checkDeviceConfig(sDeviceType, dsConfigs, dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkDeviceConfig: CPC_TestObject importation line is less that minimum", dsExceptionInfo);

    // Wrong length line
    dsConfigs = makeEmptyDynString(UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH + 1);
    cpcConfigGenericFunctions_checkDeviceConfig(sDeviceType, dsConfigs, dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkDeviceConfig: CPC_TestObject expected length 32(+3) differs from config length 33", dsExceptionInfo);

    // Right line
    dsConfigs = makeEmptyDynString(UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_DEADBAND_TYPE] = UN_CONFIG_DEADBAND_NONE;
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_RANGEMAX] = 0;
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_RANGEMIN] = 0;
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_PARAMETERS] = strjoin(CPC_TestObjectConfig_getParamNames(), "=,") + "=";
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_MASKEVENT] = 0;
    cpcConfigGenericFunctions_checkDeviceConfig(sDeviceType, dsConfigs, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for valid device config length");

    // Wrong normal position of alarm
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_NORMAL_POSITION] = 50;
    cpcConfigGenericFunctions_checkDeviceConfig(sDeviceType, dsConfigs, dsExceptionInfo);
    assertError("ERROR", "unConfigGenericFunctions_checkNormalPosition:" + getCatStr("unGeneration", "BADDATA"), dsExceptionInfo);
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_NORMAL_POSITION] = "";

    // Wrong deadband
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_DEADBAND_TYPE] = UN_CONFIG_DEADBAND_VALUE;
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_DEADBAND] = "WrongValue";
    cpcConfigGenericFunctions_checkDeviceConfig(sDeviceType, dsConfigs, dsExceptionInfo);
    assertError("ERROR", "unConfigGenericFunctions_checkFloat:" + getCatStr("unGeneration", "BADDATA"), dsExceptionInfo);
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_DEADBAND_TYPE] = UN_CONFIG_DEADBAND_NONE;

    // Wrong range
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_RANGEMIN] = 1;
    cpcConfigGenericFunctions_checkDeviceConfig(sDeviceType, dsConfigs, dsExceptionInfo);
    assertError("ERROR", "unConfigGenericFunctions_checkRange:" + getCatStr("unGeneration", "BADDATA"), dsExceptionInfo);
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_RANGEMIN] = 0;

    // Wrong format
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_FORMAT] = "|WrongValue";
    cpcConfigGenericFunctions_checkDeviceConfig(sDeviceType, dsConfigs, dsExceptionInfo);
    assertError("ERROR", "unConfigGenericFunctions_checkFormat: WRONGVALUE" + getCatStr("unGeneration", "DEBUGBADFORMAT"), dsExceptionInfo);
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_FORMAT] = "";

    // Wrong archive
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ARCHIVE_ACTIVE] = UN_CONFIG_ARCHIVE_ACTIVE;
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ARCHIVE_TIME_FILTER] = "WrongValue";
    cpcConfigGenericFunctions_checkDeviceConfig(sDeviceType, dsConfigs, dsExceptionInfo);
    assertError("ERROR", "unConfigGenericFunctions_checkFloat:" + getCatStr("unGeneration", "BADDATA"), dsExceptionInfo);
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ARCHIVE_ACTIVE] = "";

    // Wrong parameters
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_PARAMETERS] = "";
    cpcConfigGenericFunctions_checkDeviceConfig(sDeviceType, dsConfigs, dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkParameters: device should contain '" + CPC_TestObjectConfig_getParamNames()[1] + "' parameter.", dsExceptionInfo);
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_PARAMETERS] = strjoin(CPC_TestObjectConfig_getParamNames(), "=,") + "=";

    // Wrong maskevent
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_MASKEVENT] = "";
    cpcConfigGenericFunctions_checkDeviceConfig(sDeviceType, dsConfigs, dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkBool: the value '' is not correct for the MASKEVENT", dsExceptionInfo);
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_MASKEVENT] = 0;
}

void test_cpcConfigGenericFunctions_checkMapping() {
    dyn_string dsExceptionInfo;

    cpcConfigGenericFunctions_checkMapping(makeDynString(), "-", dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for - in mapping");
    cpcConfigGenericFunctions_checkMapping(makeDynString(), "", dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for empty mapping");
    cpcConfigGenericFunctions_checkMapping(makeDynString(), "1=a", dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for assignment in mapping");
    cpcConfigGenericFunctions_checkMapping(makeDynString(), "1=a,2=b", dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for multiple assignment in mapping");
    cpcConfigGenericFunctions_checkMapping(makeDynString(), "1=", dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkMapping: bad data.", dsExceptionInfo);
    cpcConfigGenericFunctions_checkMapping(makeDynString(), "1=a,2", dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkMapping: bad data.", dsExceptionInfo);
    cpcConfigGenericFunctions_checkMapping(makeDynString(), "z=a", dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkMapping: bad data.", dsExceptionInfo);
}

void test_cpcConfigGenericFunctions_checkParameters() {
    dyn_string dsExceptionInfo;

    cpcConfigGenericFunctions_checkParameters("", "", dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for empty parameters");
    cpcConfigGenericFunctions_checkParameters("", "a=b,c=d", dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for multiple assignment in parameters");
    cpcConfigGenericFunctions_checkParameters("", "a=", dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for empty assignment in parameters");
    cpcConfigGenericFunctions_checkParameters("", "a=b=c", dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkParameters: bad data.", dsExceptionInfo);
    cpcConfigGenericFunctions_checkParameters("", "a=b,a=c", dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkParameters: duplicate in parameters", dsExceptionInfo);
    cpcConfigGenericFunctions_checkParameters(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "PFSPosOn=FALSE,PEnRstart=FALSE,PRstartFS=FALSE,PHLDrive=FALSE", dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for multiple assignment in parameters with device type");
    cpcConfigGenericFunctions_checkParameters(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "PEnRstart=FALSE,PRstartFS=FALSE,PHLDrive=FALSE", dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkParameters: device should contain 'PFSPosOn' parameter.", dsExceptionInfo);
}

void test_cpcConfigGenericFunctions_checkMsgConv() {
    string sConversion;
    dyn_string dsExceptionInfo;

    // No conversion
    cpcConfigGenericFunctions_checkMsgConv(sConversion, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for no conversion");

    // Wrong conversion type
    sConversion = "WrongConversion";
    cpcConfigGenericFunctions_checkMsgConv(sConversion, dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkMsgConv: wrong type of transformation: wrongconversion" + ", " + getCatStr("unGeneration", "BADINPUTS"), dsExceptionInfo);

    // Linear wrong parameters
    sConversion = "linear";
    cpcConfigGenericFunctions_checkMsgConv(sConversion, dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkMsgConv: wrong set of parameters for linear transformation: " + getCatStr("unGeneration", "BADINPUTS"), dsExceptionInfo);

    // Linear wrong first parameter
    sConversion = "linear|a|b";
    cpcConfigGenericFunctions_checkMsgConv(sConversion, dsExceptionInfo);
    assertEqual("cpcConfigGenericFunctions_checkMsgConv: wrong float value: a, " + getCatStr("unGeneration", "BADINPUTS"), dsExceptionInfo[dynlen(dsExceptionInfo) - 1],
                "Assertion failed for wrong first value of linear conversion");
    assertError("ERROR", "unConfigGenericFunctions_checkFloat: bad data.", dsExceptionInfo);

    // Linear wrong second parameter
    sConversion = "linear|1.0|b";
    cpcConfigGenericFunctions_checkMsgConv(sConversion, dsExceptionInfo);
    assertEqual("cpcConfigGenericFunctions_checkMsgConv: wrong float value: b, " + getCatStr("unGeneration", "BADINPUTS"), dsExceptionInfo[dynlen(dsExceptionInfo) - 1],
                "Assertion failed for wrong second value of linear conversion");
    assertError("ERROR", "unConfigGenericFunctions_checkFloat: bad data.", dsExceptionInfo);

    // Linear conversion
    sConversion = "linear|1.0|2.0";
    cpcConfigGenericFunctions_checkMsgConv(sConversion, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for valid linear conversion");
}

void test_cpcConfigGenericFunctions_checkArchiveConfig() {
    dyn_string dsExceptionInfo;
    dyn_string dsConfigs;

    // Empty configs
    cpcConfigGenericFunctions_checkArchiveConfig(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, dsConfigs, dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_checkArchiveConfig:" + getCatStr("unGeneration", "BADLENGTH"), dsExceptionInfo);

    // Non existing archives
    dsConfigs = makeEmptyDynString(UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_LENGTH);

    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_BOOL] = "bool";
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_ANALOG] = "analog";
    dsConfigs[UN_CONFIG_COMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_ADDITIONAL_ARCHIVE_EVENT] = "event";
    cpcConfigGenericFunctions_checkArchiveConfig(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, dsConfigs, dsExceptionInfo);
    assertError("ERROR", "unConfigGenericFunctions_checkArchive: wrong archive name (bool=FALSE), (analog=FALSE), (event=FALSE)", dsExceptionInfo);
}

void test_cpcConfigGenericFunctions_setRange() {
    string sDpe = TEST_DP_NAME + ".ProcessInput.PosSt";
    float fRangeMin, fRangeMax;
    mapping mKeys = cpcConfigGenericFunctions_getDPEConfig(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
    mapping mRegProperties = mKeys["PosSt"];
    dyn_string dsExceptionInfo;
    dyn_string dsConfigs = makeEmptyDynString(UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);

    // No range defined
    dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] = TEST_DP_NAME;
    cpcConfigGenericFunctions_setRange(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "", mRegProperties, "", false, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for empty range");

    // Valid range
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_RANGEMAX] = "18";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_RANGEMIN] = "17";
    cpcConfigGenericFunctions_setRange(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "", mRegProperties, "", false, dsExceptionInfo);
    dpGet(sDpe + ":_pv_range.._min", fRangeMin);
    dpGet(sDpe + ":_pv_range.._max", fRangeMax);
    assertEqual("17", fRangeMin, "Assertion failed for min range");
    assertEqual("18", fRangeMax, "Assertion failed for max range");
    assertEmpty(dsExceptionInfo, "Assertion failed for valid range");

    // Equal range
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_RANGEMAX] = "14";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_RANGEMIN] = "14";
    cpcConfigGenericFunctions_setRange(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "", mRegProperties, "", false, dsExceptionInfo);
    dpGet(sDpe + ":_pv_range.._min", fRangeMin);
    dpGet(sDpe + ":_pv_range.._max", fRangeMax);
    assertEqual("17", fRangeMin, "Assertion failed for min equal range");
    assertEqual("18", fRangeMax, "Assertion failed for max equal range");
    assertEmpty(dsExceptionInfo, "Assertion failed for equal range");

    // Inverted range
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_RANGEMAX] = "13";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_RANGEMIN] = "14";
    cpcConfigGenericFunctions_setRange(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "", mRegProperties, "", false, dsExceptionInfo);
    assertError("ERROR", "_fwPvRange_checkInputConfiguration(): Minimum value must be less than maximum value for my_test_dp.ProcessInput.PosSt", dsExceptionInfo);
}

void test_cpcConfigGenericFunctions_setUnit() {
    string sDpe = TEST_DP_NAME + ".ProcessInput.PosSt";
    dyn_string dsConfigs = makeEmptyDynString(UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);
    dyn_string dsExceptionInfo;
    mapping mKeys = cpcConfigGenericFunctions_getDPEConfig(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
    mapping mRegProperties =  mKeys["PosSt"];

    dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] = TEST_DP_NAME;
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_UNIT] = "Test_Unit";
    cpcConfigGenericFunctions_setUnit(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "", mRegProperties, "", false, dsExceptionInfo);
    assertEqual("Test_Unit", dpGetUnit(sDpe), "Assertion failed for unit setting");
}

void test_cpcConfigGenericFunctions_setFormat() {
    string sDpe = TEST_DP_NAME + ".ProcessInput.PosSt";
    dyn_string dsConfigs, dsExceptionInfo;
    mapping mKeys = cpcConfigGenericFunctions_getDPEConfig(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
    mapping mRegProperties =  mKeys["PosSt"];

    dsConfigs = makeEmptyDynString(UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);
    dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] = TEST_DP_NAME;
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_FORMAT] = "##.###";
    cpcConfigGenericFunctions_setFormat(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "", mRegProperties, "", false, dsExceptionInfo);
    assertEqual("%6.3f", dpGetFormat(sDpe), "Assertion failed for format setting");
}

void test_cpcConfigGenericFunctions_setMaskEvent() {
    dyn_string dsExceptionInfo;
    bool iMask;

    dpGet(TEST_DP_NAME +".eventMask", iMask);
    assertFalse(iMask);

    cpcConfigGenericFunctions_setMaskEvent(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME , TEST_DP_NAME, true, dsExceptionInfo);
    dpGet(TEST_DP_NAME +".eventMask", iMask);
    assertTrue(iMask);
}

void test_cpcConfigGenericFunctions_setAddress() {
    string sDrvIdent, sDataType, sAddress, sValue;
    mapping mKeys = cpcConfigGenericFunctions_getDPEConfig(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
    mapping mRegProperties = mKeys["PosSt"];
    dyn_string dsExceptionInfo;
    dyn_string dsConfigs = makeEmptyDynString(UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);

    // UN
    dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER] = "1";
    dsConfigs[UN_CONFIG_OBJECTGENERAL_PLC_TYPE] = "MODBUS";
    dsConfigs[UN_CONFIG_OBJECTGENERAL_PLC_NUMBER] = "1";
    dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] = TEST_DP_NAME;
    dsConfigs[UN_CONFIG_OBJECTGENERAL_EVENT16] = "1000";

    // Value address
    cpcConfigGenericFunctions_setAddress(dsConfigs, "PosSt", mRegProperties, "value:50", UN_PLC_DPTYPE, dsExceptionInfo);
    dpGet(TEST_DP_NAME + ".ProcessInput.PosSt", sValue);
    assertEqual("50", sValue, "Assertion failed for value address");

    // TSPP address
    cpcConfigGenericFunctions_setAddress(dsConfigs, "PosSt", mRegProperties, "3", UN_PLC_DPTYPE, dsExceptionInfo);
    dpGet(TEST_DP_NAME + ".ProcessInput.PosSt:_address.._drv_ident", sDrvIdent);
    assertEqual("MODBUS", sDrvIdent, "Assertion failed for driver of UN address");
    dpGet(TEST_DP_NAME + ".ProcessInput.PosSt:_address.._datatype", sDataType);
    assertEqual(PVSS_MODBUS_FLOAT, sDataType, "Assertion failed for data type of UN address");
    dpGet(TEST_DP_NAME + ".ProcessInput.PosSt:_address.._reference", sAddress);
    assertEqual("U.1.1.3", sAddress, "Assertion failed for UN address");

    // S7
    dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER] = "1";
    dsConfigs[UN_CONFIG_OBJECTGENERAL_PLC_TYPE] = "S7_PLC";
    dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] = TEST_DP_NAME;
    dsConfigs[UN_CONFIG_OBJECTGENERAL_PLCNAME] = "PLC_NAME";
    cpcConfigGenericFunctions_setAddress(dsConfigs, "PosSt", mRegProperties, "DB10.DBW78", S7_PLC_DPTYPE, dsExceptionInfo);
    dpGet(TEST_DP_NAME + ".ProcessInput.PosSt:_address.._drv_ident", sDrvIdent);
    assertEqual("S7", sDrvIdent, "Assertion failed for driver of S7 address");
    dpGet(TEST_DP_NAME + ".ProcessInput.PosSt:_address.._datatype", sDataType);
    assertEqual(PVSS_S7_FLOAT, sDataType, "Assertion failed for data type of S7 address");
    dpGet(TEST_DP_NAME + ".ProcessInput.PosSt:_address.._reference", sAddress);
    assertEqual(sAddress, "PLC_NAME.DB10.DBW78", "Assertion failed for S7 address");

    // Wrong FrontEnd
    cpcConfigGenericFunctions_setAddress(dsConfigs, "", mRegProperties, 1, "", dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_setAddress: unknown front end ", dsExceptionInfo);

}

void test_cpcConfigGenericFunctions_setSmoothing() {
    int iConfigExists;
    float fSmoothValue;
    string sDpe = TEST_DP_NAME + ".ProcessInput.PosSt";
    mapping mKeys = cpcConfigGenericFunctions_getDPEConfig(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
    mapping mRegProperties = mKeys["PosSt"];
    dyn_string dsExceptionInfo;
    dyn_string dsConfigs = makeEmptyDynString(UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);

    dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] = TEST_DP_NAME;
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_DEADBAND] = "0.5";

    // Deadband value
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_DEADBAND_TYPE] = UN_CONFIG_DEADBAND_VALUE;
    cpcConfigGenericFunctions_setSmoothing(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "PosSt", mRegProperties, "_UnPlc", false, dsExceptionInfo);
    dpGet(sDpe + ":_smooth.._type", iConfigExists);
    dpGet(sDpe + ":_smooth.._std_tol", fSmoothValue);
    assertEqual(DPCONFIG_SMOOTH_SIMPLE_MAIN, iConfigExists, "Assertion failed for smooth type");
    assertEqual("0.5", fSmoothValue, "Assertion failed for smooth value");

    // Deadband relatif
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_DEADBAND_TYPE] = UN_CONFIG_DEADBAND_RELATIF;
    cpcConfigGenericFunctions_setSmoothing(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "PosSt", mRegProperties, "_UnPlc", false, dsExceptionInfo);
    dpGet(sDpe + ":_smooth.._type", iConfigExists);
    dpGet(sDpe + ":_smooth.._std_tol", fSmoothValue);
    assertEqual(DPCONFIG_SMOOTH_SIMPLE_MAIN, iConfigExists, "Assertion failed for smooth type");
    assertEqual("0.5", fSmoothValue, "Assertion failed for smooth value");

    // Deadband old/new
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_DEADBAND_TYPE] = UN_CONFIG_DEADBAND_OLD_NEW;
    cpcConfigGenericFunctions_setSmoothing(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "PosSt", mRegProperties, "_UnPlc", false, dsExceptionInfo);
    dpGet(sDpe + ":_smooth.._type", iConfigExists);
    dpGet(sDpe + ":_smooth.._std_tol", fSmoothValue);
    assertEqual(DPCONFIG_SMOOTH_SIMPLE_MAIN, iConfigExists, "Assertion failed for smooth type");
    assertEqual("0", fSmoothValue, "Assertion failed for smooth value");

    // No deadband
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_DEADBAND] = "";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_DEADBAND_TYPE] = "";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ARCHIVE_ACTIVE] = "-2";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ARCHIVE_TIME_FILTER] = "1";
    cpcConfigGenericFunctions_setSmoothing(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "PosSt", mRegProperties, "_UnPlc", false, dsExceptionInfo);
    dpGet(sDpe + ":_smooth.._type", iConfigExists);
    assertEqual(DPCONFIG_NONE, iConfigExists, "Assertion failed for smooth type");
}

void test_cpcConfigGenericFunctions_setDPEAlias() {
    string sDpe = TEST_DP_NAME + ".ProcessInput.PosSt";
    dyn_string dsExceptionInfo;

    // Right dpe
    cpcConfigGenericFunctions_setDPEAlias("My_Alias", sDpe, dsExceptionInfo);
    assertEqual("My_Alias.PosSt", dpGetAlias(TEST_DP_NAME + ".ProcessInput.PosSt"), "Assertion failed for alias of Dpe");

    // Wrong dpe
    sDpe = TEST_DP_NAME;
    cpcConfigGenericFunctions_setDPEAlias("My_Alias", sDpe, dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_setDPEAlias: can't set alias for my_test_dp", dsExceptionInfo);
}

void test_cpcConfigGenericFunctions_setArchive() {
    float fArchiveTimeFilter;
    string sArchiveActive;
    mapping mKeys = cpcConfigGenericFunctions_getDPEConfig(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
    mapping mRegProperties = mKeys["PosSt"];
    dyn_string dsExceptionInfo;
    dyn_string dsConfigs = makeEmptyDynString(UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);

    dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_ANALOG] = "ValueArchive_0000";
    dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] = TEST_DP_NAME;
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ARCHIVE_ACTIVE] = "VR,0.5,N";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ARCHIVE_TIME_FILTER] = "2";

    // Set PosSt archive
    cpcConfigGenericFunctions_setArchive(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "PosSt", mRegProperties, "_UnPlc", true, dsExceptionInfo);
    unExportDevice_getArchiveParameters(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, sArchiveActive, fArchiveTimeFilter);
    assertEqual("VR,0.5,N", sArchiveActive, "Assertion failed for archive of PosSt");

    // Set AlSt archive
    cpcConfigGenericFunctions_setArchive(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "AlSt", mRegProperties, "_UnPlc", true, dsExceptionInfo);
    unExportDevice_getArchiveParameters(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, sArchiveActive, fArchiveTimeFilter);
    assertEqual("VR,0.5,N", sArchiveActive, "Assertion failed for archive of AlSt");
}

void test_cpcConfigGenericFunctions_setCMWAddress() {
    string sDpe = TEST_DP_NAME + ".ProcessInput.PosSt";
    dyn_string dsResult, dsExceptionInfo;

    cpcConfigGenericFunctions_setCMWAddress(sDpe, "1|2|3|4$5$6|7|8|9", "1", dsExceptionInfo);
    unConfigCMW_getAddressConfig(sDpe, dsResult);
    assertEqual("0 | 2 |  | 4 | 5: | 6 | 7 | 8 | TRUE", dsResult, "Assertion failed for CMW address");
}

void test_cpcConfigGenericFunctions_setEvArchive() {
    bool bConfigExists, bArchiveClass, bIsActive;
    int iSmoothProcedure, iArchiveType;
    float fDeadband, fTimeInterval;
    dyn_string dsExceptionInfo;
    dyn_string dsConfigs = makeEmptyDynString(UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);

    dsConfigs[UN_CONFIG_OBJECTGENERAL_ARCHIVE_EVENT] = "ValueArchive_0000";
    dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] = TEST_DP_NAME;

    cpcConfigGenericFunctions_setEvArchive(dsConfigs, ".ProcessInput.AlSt", dsExceptionInfo);
    fwArchive_get(TEST_DP_NAME + ".ProcessInput.AlSt", bConfigExists, bArchiveClass, iArchiveType, iSmoothProcedure, fDeadband, fTimeInterval, bIsActive, dsExceptionInfo);
    assertTrue(bConfigExists, "Assertion failed for archive of alarm");
    assertTrue(bIsActive, "Assertion failed for active archive of alarm");
    assertEmpty(dsExceptionInfo, "Assertion failed for set/get archive of alarm");
}

void test_cpcConfigGenericFunctions_setDigitalAlert() {
    string sDpe = TEST_DP_NAME + ".ProcessInput.AlSt";
    bool bActive;
    dyn_string dsExceptionInfo;
    dyn_string dsDescription = makeDynString("AlarmDescription");
    dyn_string dsAlertTexts = makeDynString("Ok", "Bad");
    dyn_string dsAlertClasses = makeDynString("", "_cpcAnalog_Ack");

    // Set active alert
    cpcConfigGenericFunctions_setDigitalAlert(sDpe, dsDescription, dsAlertTexts, dsAlertClasses, true, true, true, dsExceptionInfo);
    dpGet(sDpe + ":_alert_hdl.._active", bActive);
    assertTrue(bActive, "Assertion failed for active alert");

    // Set inactive alert
    cpcConfigGenericFunctions_setDigitalAlert(sDpe, dsDescription, dsAlertTexts, dsAlertClasses, true, false, true, dsExceptionInfo);
    dpGet(sDpe + ":_alert_hdl.._active", bActive);
    assertFalse(bActive, "Assertion failed for inactive alert");
}

void test_cpcConfigGenericFunctions_setAlert() {
    string sDrvIdent, sDataType, sAddress;
    mapping mKeys = cpcConfigGenericFunctions_getDPEConfig(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
    mapping mRegProperties = mKeys["AlSt"];
    dyn_string dsResult, dsExceptionInfo;
    dyn_string dsConfigs = makeEmptyDynString(UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);

    dsConfigs[UN_CONFIG_OBJECTGENERAL_DRIVER] = "1";
    dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] = TEST_DP_NAME;

    // S7
    dsConfigs[UN_CONFIG_OBJECTGENERAL_PLC_TYPE] = "S7_PLC";
    dsConfigs[UN_CONFIG_OBJECTGENERAL_PLCNAME] = "PLC_NAME";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH - 1] = "DB10.DBW78"; // No AlSt constant
    cpcConfigGenericFunctions_setAlert(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "Test_Alert", mRegProperties, S7_PLC_DPTYPE, "AlSt", dsExceptionInfo);
    dpGet(TEST_DP_NAME + ".ProcessInput.AlSt:_address.._drv_ident", sDrvIdent);
    assertEqual("S7", sDrvIdent, "Assertion failed for S7 address driver");
    dpGet(TEST_DP_NAME + ".ProcessInput.AlSt:_address.._datatype", sDataType);
    assertEqual(PVSS_S7_BOOL, sDataType, "Assertion failed for S7 address data type");
    dpGet(TEST_DP_NAME + ".ProcessInput.AlSt:_address.._reference", sAddress);
    assertEqual(sAddress, "PLC_NAME.DB10.DBX78.6", "Assertion failed for S7 address");

    // UN
    dsConfigs[UN_CONFIG_OBJECTGENERAL_PLC_TYPE] = "MODBUS";
    dsConfigs[UN_CONFIG_OBJECTGENERAL_PLC_NUMBER] = "1";
    dsConfigs[UN_CONFIG_OBJECTGENERAL_EVENT16] = "1000";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH - 1] = "3"; // No AlSt constant
    cpcConfigGenericFunctions_setAlert(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "Test_Alert", mRegProperties, UN_PLC_DPTYPE, "AlSt", dsExceptionInfo);
    assertEmpty(dsExceptionInfo);
    dpGet(TEST_DP_NAME + ".ProcessInput.AlSt:_address.._drv_ident", sDrvIdent);
    assertEqual("MODBUS", sDrvIdent, "Assertion failed for UN address driver");
    dpGet(TEST_DP_NAME + ".ProcessInput.AlSt:_address.._datatype", sDataType);
    assertEqual(PVSS_MODBUS_BOOL, sDataType, "Assertion failed for UN address data type");
    dpGet(TEST_DP_NAME + ".ProcessInput.AlSt:_address.._reference", sAddress);
    assertEqual("U.1.1.3", sAddress, "Assertion failed for UN address");

    // CMW
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH - 1] = "1|2|3|4$5$6|7|8|9"; // No AlSt constant
    cpcConfigGenericFunctions_setAlert(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "Test_Alert", mRegProperties, CPC_CMW_DPTYPE, "AlSt", dsExceptionInfo);
    unConfigCMW_getAddressConfig(TEST_DP_NAME + ".ProcessInput.AlSt", dsResult);
    assertEqual("0 | 2 |  | 4 | 5: | 6 | 7 | 8 | TRUE", dsResult, "Assertion failed for CMW address");

    // OPC UA
    dpCreate("_OPCUA_1", "_OPCUA");
    dpCreate("_OPCUA_Sub_1", "_OPCUASubscription");

    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDRESS_STSREG01] = "1|2|3|4";
    cpcConfigGenericFunctions_setAlert(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "Test_Alert", mRegProperties, "OPCUA", "StsReg01", dsExceptionInfo);

    dpGet(TEST_DP_NAME + ".ProcessInput.AlSt:_address.._reference", sAddress);
    dpGet(TEST_DP_NAME + ".ProcessInput.AlSt:_address.._drv_ident", sDrvIdent);
    dpGet(TEST_DP_NAME + ".ProcessInput.AlSt:_address.._datatype", sDataType);
    assertEqual("OPCUA_1$OPCUA_Sub_1$1$1$ns=1;2=3", sAddress, "Assertion failed for OPCUA address");
    assertEqual("OPCUA", sDrvIdent, "Assertion failed for OPCUA address driver");
    assertEqual(OPCUA_DEFAULT, sDataType, "Assertion failed for OPCUA address data type");

    dpDelete("_OPCUA_Sub_1", "_OPCUASubscription");
    dpDelete("_OPCUA_1", "_OPCUA");

    // Wrong frontend
    cpcConfigGenericFunctions_setAlert(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "Test_Alert", mRegProperties, "missed_protocol", "AlSt", dsExceptionInfo);
    assertError("ERROR", "cpcConfigGenericFunctions_setAlert: unknown front end missed_protocol", dsExceptionInfo);
}

void test_cpcConfigGenericFunctions_setMetainfo() {
    dyn_string dsExceptionInfo, dsMetaInfo;
    dyn_string dsConfigs = makeEmptyDynString(UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);

    dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] = TEST_DP_NAME;
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_MASTER_NAME] = "MasterName";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_PARENTS] = "ParentsName";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_CHILDREN] = "ChildrenName";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_TYPE] = "TypeName";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_SECOND_ALIAS] = "SecondAliasName";

    cpcConfigGenericFunctions_setMetainfo(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, dsConfigs, false, dsExceptionInfo);
    cpcExportGenericFunctions_getMetainfo(TEST_DP_NAME, dsMetaInfo, dsExceptionInfo);
    assertEqual("MasterName | ParentsName | ChildrenName | TypeName | SecondAliasName", dsMetaInfo, "Assertion failed for Meta info");
}

void test_cpcConfigGenericFunctions_setMapping() {
    string sDpe = TEST_DP_NAME + ".statusInformation.configuration.deviceConfiguration";
    mapping mKeys = cpcConfigGenericFunctions_getDPEConfig(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
    mapping mRegProperties = mKeys["PosSt"];
    dyn_string dsExceptionInfo, dsConfig;
    dyn_string dsConfigs = makeEmptyDynString(UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);

    dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] = TEST_DP_NAME;

    // Valid mapping
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH - 1] = "Test_Element_Description"; // No mapping constant
    cpcConfigGenericFunctions_setMapping(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "PosSt", mRegProperties, "_UnPlc", false, dsExceptionInfo);
    dpGet(sDpe, dsConfig);
    assertEqual("DESC_PosSt:Test_Element_Description", dsConfig, "Assertion failed for valid mapping");
    dynClear(dsConfig);

    // Old mapping format
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH - 1] = "-"; // No mapping constant
    cpcConfigGenericFunctions_setMapping(dsConfigs, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "PosSt", mRegProperties, "_UnPlc", false, dsExceptionInfo);
    dpGet(sDpe, dsConfig);
    assertEqual("DESC_PosSt:", dsConfig, "Assertion failed for - mapping");
}

void test_cpcConfigGenericFunctions_setParameters() {
    string sDpe = TEST_DP_NAME + ".statusInformation.configuration.deviceConfiguration";
    dyn_string dsExceptionInfo, dsParameters;
    dyn_string dsConfigs = makeEmptyDynString(UN_CONFIG_ALLCOMMON_LENGTH + UN_CONFIG_CPC_TESTOBJECT_LENGTH + UN_CONFIG_CPC_TESTOBJECT_ADDITIONAL_LENGTH);

    dsConfigs[UN_CONFIG_OBJECTGENERAL_DPNAME] = TEST_DP_NAME;
    cpcConfigGenericFunctions_setParameters(UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, dsConfigs, "Test_Parameter1=1,Test_Parameter2,Test_Parameter2=2", dsExceptionInfo);
    dpGet(sDpe, dsParameters);
    assertEqual("PROP_Test_Parameter2:2 | PROP_Test_Parameter1:1", dsParameters, "Assertion failed for valid parameters");
}

void test_cpcConfigGenericFunctions_setMsgConv() {
    string sConversion;
    int iMsgConvType, iPolyGrade;
    float fPolyA, fPolyB;
    dyn_string dsExceptionInfo;
    dyn_string dsDpes = makeDynString(TEST_DP_NAME + ".ProcessInput.PosSt");

    // Empty conversion
    cpcConfigGenericFunctions_setMsgConv(sConversion, dsDpes, dsExceptionInfo);
    dpGet(dsDpes[1] + ":_msg_conv.1._type", iMsgConvType);
    assertEmpty(dsExceptionInfo, "Assertion failed for empty conversion");
    assertEqual(DPDETAIL_CONV_NONE, iMsgConvType, "Assertion failed for empty conversion");

    // Linear conversion
    sConversion = "linear|1.0|2.0";
    cpcConfigGenericFunctions_setMsgConv(sConversion, dsDpes, dsExceptionInfo);
    dpGet(dsDpes[1] + ":_msg_conv.1._type", iMsgConvType);
    dpGet(dsDpes[1] + ":_msg_conv.1._poly_grade", iPolyGrade);
    dpGet(dsDpes[1] + ":_msg_conv.1._poly_a", fPolyA);
    dpGet(dsDpes[1] + ":_msg_conv.1._poly_b", fPolyB);
    assertEmpty(dsExceptionInfo, "Assertion failed for linear conversion");
    assertEqual(DPDETAIL_CONV_POLY, iMsgConvType, "Assertion failed for linear conversion type");
    assertEqual(1, iPolyGrade, "Assertion failed for linear conversion grade");
    assertEqual(1.0, fPolyA, "Assertion failed for linear conversion first value");
    assertEqual(2.0, fPolyB, "Assertion failed for linear conversion second value");
}

void test_cpcConfigGenericFunctions_set5RangesAlert() {
    int iAlertType, iAlert1Type, iAlert2Type, iAlert3Type, iAlert4Type, iAlert5Type,
        iAlert1UpLimit, iAlert2LowLimit, iAlert2UpLimit, iAlert3LowLimit, iAlert3UpLimit, iAlert4LowLimit, iAlert4UpLimit, iAlert5LowLimit;
    bool bAlertActive, bAlert1UpInclude, bAlert2LowInclude, bAlert2UpInclude, bAlert3LowInclude, bAlert3UpInclude, bAlert4LowInclude, bAlert4UpInclude, bAlert5LowInclude;
    string sAlert1Class, sAlert2Class, sAlert3Class, sAlert4Class, sAlert5Class;
    string sSystemName = getSystemName();
    dyn_string dsConfigs, dsExceptionInfo;
    dyn_int diLimits = makeDynInt(1, 2, 3, 4);

    // Create alarm
    dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_DOMAIN] = "Domain";
    dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_NATURE] = "Nature";
    dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_ALIAS] = "Alias";
    dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_DESCRIPTION] = "Description";
    dsConfigs[UN_CONFIG_OBJECTGENERAL_LENGTH + UN_CONFIG_COMMON_DEFAULTPANEL] = "DefaultPanel";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diLimits[1]] = "5";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diLimits[2]] = "10";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diLimits[3]] = "15";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diLimits[4]] = "20";
    cpcConfigGenericFunctions_set5RangesAlert(dsConfigs, TEST_DP_NAME + ".ProcessInput.PosSt", "Description", TRUE, TRUE, TRUE, diLimits, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for alert setting");
    dpGet(TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.._type", iAlertType,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.1._type", iAlert1Type,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.2._type", iAlert2Type,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.3._type", iAlert3Type,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.4._type", iAlert4Type,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.5._type", iAlert5Type,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.1._class", sAlert1Class,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.2._class", sAlert2Class,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.3._class", sAlert3Class,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.4._class", sAlert4Class,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.5._class", sAlert5Class,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.1._u_limit", iAlert1UpLimit,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.2._l_limit", iAlert2LowLimit,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.2._u_limit", iAlert2UpLimit,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.3._l_limit", iAlert3LowLimit,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.3._u_limit", iAlert3UpLimit,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.4._l_limit", iAlert4LowLimit,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.4._u_limit", iAlert4UpLimit,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.5._l_limit", iAlert5LowLimit,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.1._u_incl", bAlert1UpInclude,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.2._l_incl", bAlert2LowInclude,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.2._u_incl", bAlert2UpInclude,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.3._l_incl", bAlert3LowInclude,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.3._u_incl", bAlert3UpInclude,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.4._l_incl", bAlert4LowInclude,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.4._u_incl", bAlert4UpInclude,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.5._l_incl", bAlert5LowInclude,
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.._active", bAlertActive);
    assertEqual(DPCONFIG_ALERT_NONBINARYSIGNAL, iAlertType, "Assertion failed for alert type");
    assertEqual(DPDETAIL_RANGETYPE_MINMAX, iAlert1Type, "Assertion failed for 1st alert type");
    assertEqual(DPDETAIL_RANGETYPE_MINMAX, iAlert2Type, "Assertion failed for 2nd alert type");
    assertEqual(DPDETAIL_RANGETYPE_MINMAX, iAlert3Type, "Assertion failed for 3rd alert type");
    assertEqual(DPDETAIL_RANGETYPE_MINMAX, iAlert4Type, "Assertion failed for 4th alert type");
    assertEqual(DPDETAIL_RANGETYPE_MINMAX, iAlert5Type, "Assertion failed for 5th alert type");
    assertEqual(sSystemName + "_cpcAnalogLL_Ack_Mail.", sAlert1Class, "Assertion failed for 1st alert class");
    assertEqual(sSystemName + "_cpcAnalogL_Mail.", sAlert2Class, "Assertion failed for 2nd alert class");
    assertEqual("", sAlert3Class, "Assertion failed for 3rd alert class");
    assertEqual(sSystemName + "_cpcAnalogH_Mail.", sAlert4Class, "Assertion failed for 4th alert class");
    assertEqual(sSystemName + "_cpcAnalogHH_Ack_Mail.", sAlert5Class, "Assertion failed for 5th alert class");
    assertEqual(5, iAlert1UpLimit, "Assertion failed for 1st alert upper limit");
    assertEqual(5, iAlert2LowLimit, "Assertion failed for 2nd alert lower limit");
    assertEqual(10, iAlert2UpLimit, "Assertion failed for 2nd alert upper limit");
    assertEqual(10, iAlert3LowLimit, "Assertion failed for 3rd alert lower limit");
    assertEqual(15, iAlert3UpLimit, "Assertion failed for 3rd alert upper limit");
    assertEqual(15, iAlert4LowLimit, "Assertion failed for 4th alert lower limit");
    assertEqual(20, iAlert4UpLimit, "Assertion failed for 4th alert upper limit");
    assertEqual(20, iAlert5LowLimit, "Assertion failed for 5th alert lower limit");
    assertTrue(bAlert1UpInclude != bAlert2LowInclude, "Assertion failed for 1st and 2nd alerts limit inclusion");
    assertTrue(bAlert2UpInclude != bAlert3LowInclude, "Assertion failed for 2nd and 3rd alerts limit inclusion");
    assertTrue(bAlert3UpInclude != bAlert4LowInclude, "Assertion failed for 3rd and 4th alerts limit inclusion");
    assertTrue(bAlert4UpInclude != bAlert5LowInclude, "Assertion failed for 4th and 5th alerts limit inclusion");
    assertTrue(bAlertActive, "Assertion failed for alert active");

    // Delete alarm
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diLimits[1]] = "";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diLimits[2]] = "";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diLimits[3]] = "";
    dsConfigs[UN_CONFIG_ALLCOMMON_LENGTH + diLimits[4]] = "";
    cpcConfigGenericFunctions_set5RangesAlert(dsConfigs, TEST_DP_NAME + ".ProcessInput.PosSt", "Description", TRUE, TRUE, TRUE, diLimits, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for alert deleting");
    dpGet(TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.._type", iAlertType);
    assertEqual(DPCONFIG_NONE, iAlertType, "Assertion failed for deleted alert type");
}
