#uses "fwUnitTestComponentAsserts.ctl"
#uses "CPC_declarations.ctl"
#uses "S7Constant_declarations.ctl"
#uses "fwConfigs/fwPeriphAddress.ctl"
#uses "cpcConfigGenericFunctions.ctl"
#uses "cpcDpFunc.ctl"
#uses "test/unCPC6/cpcTestObjectConfig.ctl"

void test_setup() {
    dpCreate(TEST_DP_NAME, "CPC_TestObject");
}

void test_teardown() {
    dpDelete(TEST_DP_NAME, "CPC_TestObject");
}


void test_cpcDigitalParameterDPFunc_getStsReg01Value() {
    string sDpe = TEST_DP_NAME + ".ProcessInput.StsReg01";

    dpSetWait(sDpe, 10);
    bit32 b32ManReg = 16; // Manual On Request. Ignoring first parameter
    assertEqual(1, cpcDigitalParameterDPFunc_getStsReg01Value(sDpe, b32ManReg), "Assertion failed for MOnR");
    b32ManReg = 32; // Manual Off Request. Ignoring first parameter.
    assertEqual(0, cpcDigitalParameterDPFunc_getStsReg01Value(sDpe, b32ManReg), "Assertion failed for MOffR");
    b32ManReg = 0; // No request, returning dpe (stsReg01)
    assertEqual(10, cpcDigitalParameterDPFunc_getStsReg01Value(sDpe, b32ManReg), "Assertion failed for StsReg01 value");
}

void test_cpcDpFunc_syncAcknowledge() {
    int iState;
    dyn_string dsDescription, dsAlertTexts, dsAlertClasses, dsExceptionInfo;
    string sDpe_Al = TEST_DP_NAME + ".ProcessInput.AlSt";
    string sDpe_Ack = TEST_DP_NAME + ".ProcessInput.AlUnAck";

    dsDescription = makeDynString("AlarmDescription");
    dsAlertTexts = makeDynString("Ok", "Bad");
    dsAlertClasses = makeDynString("", "_cpcAnalog_Ack");

    // Create Alarm
    cpcConfigGenericFunctions_setDigitalAlert(sDpe_Al, dsDescription, dsAlertTexts, dsAlertClasses, true, true, true, dsExceptionInfo);

    // Check the initial state of the alarm
    dpGet(sDpe_Al + ":_alert_hdl.._act_state", iState);
    assertEqual(0, iState, "Assertion failed for no alarm");

    // Activate the alarm
    dpSetWait(sDpe_Al, 1, sDpe_Ack, 1);
    dpGet(sDpe_Al + ":_alert_hdl.._act_state", iState);
    assertEqual(1, iState, "Assertion failed for active alarm");

    // Missed alarm
    dpSetWait(sDpe_Al, 0);
    dpGet(sDpe_Al + ":_alert_hdl.._act_state", iState);
    assertEqual(3, iState, "Assertion failed for unacknowledged inactive alarm");

    // Not fixed the ack propagation. After ack, still in state 3.
    dpSetWait(sDpe_Ack, 0);
    dpGet(sDpe_Al + ":_alert_hdl.._act_state", iState);
    assertEqual(3, iState, "Assertion failed for acknowledged inactive alarm before ack propagation");

    // Call the dp function manually.
    cpcDpFunc_syncAcknowledge(sDpe_Al, false);
    dpGet(sDpe_Al + ":_alert_hdl.._act_state", iState);
    assertEqual(0, iState, "Assertion failed for acknowledged inactive alarm after ack propagation");

    // Activate the alarm
    dpSetWait(sDpe_Al, 1, sDpe_Ack, 1);
    dpGet(sDpe_Al + ":_alert_hdl.._act_state", iState);
    assertEqual(1, iState, "Assertion failed for active alarm (second time)");

    // Ack the alarm doesn't work
    dpSetWait(sDpe_Ack, 0);
    dpGet(sDpe_Al + ":_alert_hdl.._act_state", iState);
    assertEqual(1, iState, "Assertion failed for acknowledged active alarm before ack propagation");

    // Call the function manually.
    cpcDpFunc_syncAcknowledge(sDpe_Al, false);
    dpGet(sDpe_Al + ":_alert_hdl.._act_state", iState);
    assertEqual(2, iState, "Assertion failed for acknowledged active alarm after ack propagation");

    // Alarm gone
    dpSetWait(sDpe_Al, 0);
    dpGet(sDpe_Al + ":_alert_hdl.._act_state", iState);
    assertEqual(0, iState, "Assertion failed for acknowledged inactive alarm");
}

void test_cpcDpFunc_fixAckPropogation() {
    int iState;
    dyn_string dsDescription, dsAlertTexts, dsAlertClasses, dsExceptionInfo;
    string sDpe_Al = TEST_DP_NAME + ".ProcessInput.AlSt";
    string sDpe_Ack = TEST_DP_NAME + ".ProcessInput.AlUnAck";

    dsDescription = makeDynString("AlarmDescription");
    dsAlertTexts = makeDynString("Ok", "Bad");
    dsAlertClasses = makeDynString("", "_cpcAnalog_Ack");

    // Create Alarm
    cpcConfigGenericFunctions_setDigitalAlert(sDpe_Al, dsDescription, dsAlertTexts, dsAlertClasses, true, true, true, dsExceptionInfo);

    // Fix Propagation
    cpcDpFunc_fixAckPropogation (TEST_DP_NAME, ".ProcessInput.AlSt");

    // Activate the alarm
    dpSetWait(sDpe_Al, 1, sDpe_Ack, 1);
    dpGet(sDpe_Al + ":_alert_hdl.._act_state", iState);
    assertEqual(1, iState, "Assertion failed for active alarm");

    // Missed alarm
    dpSetWait(sDpe_Al, 0);
    dpGet(sDpe_Al + ":_alert_hdl.._act_state", iState);
    assertEqual(3, iState, "Assertion failed for unacknowledged inactive alarm");

    // Ack the alarm and automatically go to state 0
    dpSetWait(sDpe_Ack, 0);
    dpGet(sDpe_Al + ":_alert_hdl.._act_state", iState);
    assertEqual(0, iState, "Assertion failed for acknowledged inactive alarm");

    // Activate the alarm
    dpSetWait(sDpe_Al, 1, sDpe_Ack, 1);
    dpGet(sDpe_Al + ":_alert_hdl.._act_state", iState);
    assertEqual(1, iState, "Assertion failed for active alarm (second time)");

    // Ack the alarm
    dpSetWait(sDpe_Ack, 0);
    dpGet(sDpe_Al + ":_alert_hdl.._act_state", iState);
    assertEqual(2, iState, "Assertion failed for acknowledged active alarm");

    // Remove the alarm and goes to state 0
    dpSetWait(sDpe_Al, 0);
    dpGet(sDpe_Al + ":_alert_hdl.._act_state", iState);
    assertEqual(0, iState, "Assertion failed for acknowledged inactive alarm (second time)");
}
