#uses "fwUnitTestComponentAsserts.ctl"
#uses "CPC_declarations.ctl"
#uses "S7Constant_declarations.ctl"
#uses "fwConfigs/fwPeriphAddress.ctl"
#uses "cpcConfigGenericFunctions.ctl"
#uses "test/unCPC6/cpcTestObjectConfig.ctl"

void test_setup() {
    dpCreate(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
}

void test_teardown() {
    dpDelete(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
}

void test_cpcExportGenericFunctions_getParameters() {
    dyn_string dsParameters, dsDpParameters, dsExceptionInfo;

    dynAppend(dsParameters, "PROP_Param1:Value1");
    dynAppend(dsParameters, "Param2:Value2");
    dynAppend(dsParameters, "PROP_Param3:Value3");
    dynAppend(dsParameters, "PROP_Param4:");
    dpSetWait(TEST_DP_NAME + ".statusInformation.configuration.deviceConfiguration", dsParameters);

    cpcExportGenericFunctions_getParameters(TEST_DP_NAME, dsDpParameters, dsExceptionInfo );
    assertEqual("Param4=,Param3=Value3,Param1=Value1", dsDpParameters, "Assertion failed for valid parameters");
}

void test_cpcExportGenericFunctions_getMetainfo() {
    dyn_string dsParameters, dsDpParameters, dsExceptionInfo;

    dynAppend(dsParameters, "MASTER_NAME:MasterName");
    dynAppend(dsParameters, "PARENTS:Parent1");
    dynAppend(dsParameters, "PARENTS:Parent2");
    dynAppend(dsParameters, "CHILDREN:Children");
    dynAppend(dsParameters, "TYPE:Type");
    dynAppend(dsParameters, "SECOND_ALIAS:SecondAlias");
    dpSetWait(TEST_DP_NAME + ".statusInformation.configuration.deviceConfiguration", dsParameters);

    cpcExportGenericFunctions_getMetainfo(TEST_DP_NAME, dsDpParameters, dsExceptionInfo);
    assertEqual("MasterName | Parent1,Parent2 | Children | Type | SecondAlias", dsDpParameters, "Assertion failed for valid meta info");
}

void test_cpcExportGenericFunctions_getUnit() {
    dyn_string dsUnit;
    string sDpe = TEST_DP_NAME + ".ProcessInput.PosSt";

    dpSetUnit(sDpe, "EUR");
    cpcExportGenericFunctions_getUnit(TEST_DP_NAME, dsUnit);
    assertEqual("EUR", dsUnit, "Assertion failed for unit");
}

void test_cpcExportGenericFunctions_getFormat() {
    dyn_string dsDpParameters;
    string sDpe = TEST_DP_NAME + ".ProcessInput.PosSt";

    dpSetFormat(sDpe, "%8.2f" );
    cpcExportGenericFunctions_getFormat(TEST_DP_NAME, dsDpParameters);
    assertEqual("#####.##", dsDpParameters, "Assertion failed for format");
}


void test_cpcExportGenericFunctions_getSimpleFormat() {
    dyn_string dsDpParameters;
    string sDpe = TEST_DP_NAME + ".ProcessInput.PosSt";

    dpSetFormat(sDpe + "", "%8.2f");
    cpcExportGenericFunctions_getSimpleFormat(TEST_DP_NAME, dsDpParameters);
    assertEqual("#####.##", dsDpParameters, "Assertion failed for format");
}

void test_cpcExportGenericFunctions_getRange() {
    dyn_string dsDpParameters, dsExceptionInfo;
    string sDpe = TEST_DP_NAME + ".ProcessInput.PosSt";
    int iResult = dpSet(sDpe + ":_pv_range.._type", 7,
                        sDpe + ":_pv_range.._min", 10,
                        sDpe + ":_pv_range.._max", 30,
                        sDpe + ":_pv_range.._neg", FALSE,
                        sDpe + ":_pv_range.._incl_min", TRUE,
                        sDpe + ":_pv_range.._incl_max", TRUE,
                        sDpe + ":_pv_range.._ignor_inv", FALSE);

    cpcExportGenericFunctions_getRange(TEST_DP_NAME, dsDpParameters, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for range getting");
    assertEqual("30.0 | 10.0", dsDpParameters, "Assertion failed for range");
}

void test_cpcExportGenericFunctions_getOutRange() {
    dyn_string dsDpParameters, dsExceptionInfo;
    string sDpe = TEST_DP_NAME + ".ProcessInput.PosSt";

    dpSetWait(sDpe + ":_pv_range.._type", 7,
              sDpe + ":_pv_range.._min", 10,
              sDpe + ":_pv_range.._max", 20,
              sDpe + ":_pv_range.._neg", FALSE,
              sDpe + ":_pv_range.._incl_min", TRUE,
              sDpe + ":_pv_range.._incl_max", TRUE,
              sDpe + ":_pv_range.._ignor_inv", FALSE);
    dpSetWait(sDpe + ":_msg_conv.._type", DPCONFIG_CONVERSION_RAW_TO_ENG_MAIN,
              sDpe + ":_msg_conv.1._type", DPDETAIL_CONV_POLY,
              sDpe + ":_msg_conv.1._poly_grade", 1,
              sDpe + ":_msg_conv.1._poly_a", 0.1,
              sDpe + ":_msg_conv.1._poly_b", 0.1);
    cpcExportGenericFunctions_getOutRange(TEST_DP_NAME, dsDpParameters, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for range getting");
    assertEqual("2.1 | 1.1", dsDpParameters, "Assertion failed for range");
}

void test_cpcExportGenericFunctions_getDeadband() {
    dyn_string dsDpParameters, dsExceptionInfo;
    string sDpe = TEST_DP_NAME + ".ProcessInput.PosSt";

    // No smoothing
    cpcExportGenericFunctions_getDeadband(TEST_DP_NAME, dsDpParameters, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for no smoothing");
    assertEqual("0 | 0", dsDpParameters, "Assertion failed for no smoothing");
    dynClear(dsDpParameters);

    // Relative value smoothing
    dpSetWait(sDpe + ":_smooth.._type", DPCONFIG_SMOOTH_SIMPLE_MAIN,
              sDpe + ":_smooth.._std_type", DPATTR_VALUE_REL_SMOOTH,
              sDpe + ":_smooth.._std_tol", 0.5);
    cpcExportGenericFunctions_getDeadband(TEST_DP_NAME, dsDpParameters, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for relative smoothing");
    assertEqual("0.5 | 2", dsDpParameters, "Assertion failed for relative smoothing");
    dynClear(dsDpParameters);

    // Absolute value smoothing
    dpSetWait(sDpe + ":_smooth.._std_type", DPATTR_VALUE_SMOOTH,
              sDpe + ":_smooth.._std_tol", 0.4);
    cpcExportGenericFunctions_getDeadband(TEST_DP_NAME, dsDpParameters, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for value smoothing");
    assertEqual("0.4 | 1", dsDpParameters, "Assertion failed for value smoothing");
    dynClear(dsDpParameters);

    // Old/new smoothing
    dpSetWait(sDpe + ":_smooth.._std_type", DPATTR_COMPARE_OLD_NEW,
              sDpe + ":_smooth.._std_tol", 0.3);
    cpcExportGenericFunctions_getDeadband(TEST_DP_NAME, dsDpParameters, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for old/new smoothing");
    assertEqual("0 | 3", dsDpParameters, "Assertion failed for old/new smoothing");
    dynClear(dsDpParameters);

    // Time or value smoothing
    dpSetWait(sDpe + ":_smooth.._std_type", 9,
              sDpe + ":_smooth.._std_tol", 0.2);
    cpcExportGenericFunctions_getDeadband(TEST_DP_NAME, dsDpParameters, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for time or value smoothing");
    assertEqual("0 | 0", dsDpParameters, "Assertion failed for time or value smoothing");
}

void test_cpcExportGenericFunctions_getArchive() {
    dyn_string dsDpParameters, dsExceptionInfo;
    string sDpe = TEST_DP_NAME + ".ProcessInput.PosSt";
    string sSystemName = getSystemName();

    // Check system name
    assertTrue(sSystemName != "", "Assertion failed for system name");

    // No archive
    cpcExportGenericFunctions_getArchive(TEST_DP_NAME, dsDpParameters);
    assertEqual("N | ", dsDpParameters, "Assertion failed for no archive");
    dynClear(dsDpParameters);

    // Value archive
    dpSetWait(sDpe + ":_archive.._type", DPCONFIG_DB_ARCHIVEINFO,
              sDpe + ":_archive.._archive", TRUE,
              sDpe + ":_archive.1._type", DPATTR_ARCH_PROC_VALARCH,
              sDpe + ":_archive.1._class", sSystemName + "_ValueArchive_6");
    cpcExportGenericFunctions_getArchive(TEST_DP_NAME, dsDpParameters);
    assertEqual("Y | 0", dsDpParameters, "Assertion failed for value archive");
}

void test_cpcExportGenericFunctions_getMaskEvent() {
    dyn_string dsDpParameters;

    dpSetWait(TEST_DP_NAME + ".eventMask", 32);
    cpcExportGenericFunctions_getMaskEvent(TEST_DP_NAME, dsDpParameters);
    assertEqual("32", dsDpParameters, "Assertion failed for mask event");
}

void test_cpcExportGenericFunctions_getDefaultValue() {
    dyn_string dsDpParameters;

    dpSetWait(TEST_DP_NAME + ".ProcessOutput.MPosR", 50.0);
    cpcExportGenericFunctions_getDefaultValue(TEST_DP_NAME, dsDpParameters, "MPosR");
    assertEqual("50", dsDpParameters, "Assertion failed for MPosR value");
}

void test_cpcExportGenericFunctions_getPIDName() {
    dyn_string dsDpParameters;

    dpSetDescription(TEST_DP_NAME + ".ProcessInput", "Name1;Description1;Name2;Description2;Name3;Description3");
    cpcExportGenericFunctions_getPIDName(TEST_DP_NAME, dsDpParameters);
    assertEqual("Description3", dsDpParameters, "Assertion failed for PIDName");
}

void test_cpcExportGenericFunctions_get5AlertClasses() {
    dyn_string dsExceptionInfo, dsClasses;
    string sSystemName = getSystemName();

    createAlert();
    cpcExportGenericFunctions_get5AlertClasses(TEST_DP_NAME, dsClasses, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for alarm classes");
    assertEqual(makeDynString(sSystemName + "_cpcAnalogLL.", sSystemName + "_cpcAnalogL.", sSystemName + "_cpcAnalogH.", sSystemName + "_cpcAnalogHH."), dsClasses,
                "Assertion failed for alarm classes");
}

void test_cpcExportGenericFunctions_get5AlertLimits() {
    dyn_string dsDpParameters;

    createAlert();
    cpcExportGenericFunctions_get5AlertLimits(TEST_DP_NAME, dsDpParameters);
    assertEqual(makeDynString(4.0, 3.0, 2.0, 1.0), dsDpParameters, "Assertion failed for alarm limits");
}

void test_cpcExportGenericFunctions_get5AlertAcknowledgeAlarmValue() {
    dyn_string dsExceptionInfo;

    // No ack
    createAlert();
    string sResult = cpcExportGenericFunctions_get5AlertAcknowledgeAlarmValue(TEST_DP_NAME, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for no acknowledge alarm");
    assertEqual("FALSE", sResult, "Assertion failed for no acknowledge alarm");

    // Ack
    dpSet(TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.1._class", "_cpcAnalogLL_Ack.");
    sResult = cpcExportGenericFunctions_get5AlertAcknowledgeAlarmValue(TEST_DP_NAME, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for acknowledge alarm");
    assertEqual("TRUE", sResult, "Assertion failed for acknowledge alarm");
}

void test_cpcExportGenericFunctions_get5AlertAcknowledgeAlarm() {
    dyn_string dsDpParameters, dsExceptionInfo;

    cpcExportGenericFunctions_get5AlertAcknowledgeAlarm(TEST_DP_NAME, dsDpParameters, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for no alarm");
    assertEqual(makeDynString("FALSE"), dsDpParameters, "Assertion failed for no alarm");
}

void test_cpcExportGenericFunctions_getAcknowledgeAlarmValue() {
    // No alert
    assertEqual("N", cpcExportGenericFunctions_getAcknowledgeAlarmValue(TEST_DP_NAME), "Assertion failed for no alarm");

    // Alert
    createAlert();
    assertEqual("N", cpcExportGenericFunctions_getAcknowledgeAlarmValue(TEST_DP_NAME), "Assertion failed for alarm");

    // Binary alert with Ack class
    createBinaryAlert();
    assertEqual("Y", cpcExportGenericFunctions_getAcknowledgeAlarmValue(TEST_DP_NAME, ".ProcessInput.AlSt"), "Assertion failed for binary alarm");
}

void test_cpcExportGenericFunctions_getAcknowledgeAlarm() {
    dyn_string dsDpParameters, dsExceptionInfo;

    cpcExportGenericFunctions_getAcknowledgeAlarm(TEST_DP_NAME, ".ProcessInput.PosSt", dsDpParameters, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for no alarm");
    assertEqual(makeDynString("N"), dsDpParameters, "Assertion failed for no alarm");
}

void test_cpcExportGenericFunctions_getAlarmActive() {
    dyn_string dsDpParameters;

    // No alarm
    cpcExportGenericFunctions_getAlarmActive(TEST_DP_NAME, dsDpParameters);
    assertEqual(makeDynString(FALSE), dsDpParameters, "Assertion failed for no alarm");
    dynClear(dsDpParameters);

    // Active alarm
    createAlert();
    dpSet(TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.._active", TRUE);
    cpcExportGenericFunctions_getAlarmActive(TEST_DP_NAME, dsDpParameters);
    assertEqual(makeDynString(TRUE), dsDpParameters, "Assertion failed for active alarm");
}

void test_cpcExportGenericFunctions_getBoolDefaultValue() {
    dyn_string dsDpParameters;

    dpSet(TEST_DP_NAME + ".ProcessOutput.MPosR", TRUE);
    cpcExportGenericFunctions_getBoolDefaultValue(TEST_DP_NAME, dsDpParameters, "MPosR");
    assertEqual(makeDynString(TRUE), dsDpParameters, "Assertion failed for true MPosR");
}

void test_cpcExportGenericFunctions_getCategories() {
    dyn_string dsDpParameters, dsExceptionInfo;

    cpcExportGenericFunctions_getCategories(TEST_DP_NAME, makeDynString(".ProcessInput.PosSt"), dsDpParameters, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for empty parameters");
    assertEqual(makeDynString(""), dsDpParameters);
}

void test_cpcExportGenericFunctions_getDigitalNormalPosition() {
    dyn_string dsDpParameters;

    // No alarm
    cpcExportGenericFunctions_getDigitalNormalPosition(TEST_DP_NAME, "", makeDynString(".ProcessInput.AlSt"), dsDpParameters);
    assertEqual(makeDynString(2), dsDpParameters, "Assertion failed for no alarm");
    dynClear(dsDpParameters);

    // Inactive alarm with ok range 0
    createBinaryAlert();
    cpcExportGenericFunctions_getDigitalNormalPosition(TEST_DP_NAME, "", makeDynString(".ProcessInput.AlSt"), dsDpParameters);
    assertEqual(makeDynString(3), dsDpParameters, "Assertion failed for inactive alarm");
    dynClear(dsDpParameters);

    // Active alarm with ok range 0
    dpSet(TEST_DP_NAME + ".ProcessInput.AlSt:_alert_hdl.._active", TRUE);
    cpcExportGenericFunctions_getDigitalNormalPosition(TEST_DP_NAME, "", makeDynString(".ProcessInput.AlSt"), dsDpParameters);
    assertEqual(makeDynString(0), dsDpParameters, "Assertion failed for active alarm");
    dynClear(dsDpParameters);

    // Inactive alarm with ok range 1
    dpSet(TEST_DP_NAME + ".ProcessInput.AlSt:_alert_hdl.._active", FALSE);
    dpSet(TEST_DP_NAME + ".ProcessInput.AlSt:_alert_hdl.._ok_range", 1);
    cpcExportGenericFunctions_getDigitalNormalPosition(TEST_DP_NAME, "", makeDynString(".ProcessInput.AlSt"), dsDpParameters);
    assertEqual(makeDynString(4), dsDpParameters, "Assertion failed for inactive ok alarm");
    dynClear(dsDpParameters);

    // Active alarm with ok range 1
    dpSet(TEST_DP_NAME + ".ProcessInput.AlSt:_alert_hdl.._active", TRUE);
    cpcExportGenericFunctions_getDigitalNormalPosition(TEST_DP_NAME, "", makeDynString(".ProcessInput.AlSt"), dsDpParameters);
    assertEqual(makeDynString(1), dsDpParameters, "Assertion failed for active ok alarm");
}

void test_cpcExportGenericFunctions_getDPEValue() {
    dyn_string dsDpParameters;

    // Unformatted value
    dpSet(TEST_DP_NAME + ".ProcessInput.PosSt", 50.0);
    cpcExportGenericFunctions_getDPEValue(TEST_DP_NAME, ".ProcessInput.PosSt", dsDpParameters);
    assertEqual(makeDynString(50.0), dsDpParameters, "Assertion failed for unformatted value");
    dynClear(dsDpParameters);

    // Formatted value
    cpcExportGenericFunctions_getDPEValue(TEST_DP_NAME, ".ProcessInput.PosSt", dsDpParameters, "%6.3f");
    assertEqual(makeDynString("50.000"), dsDpParameters, "Assertion failed for formatted value");
}

void test_cpcExportGenericFunctions_processAddress() {
    dyn_string dsDpParameters;

    // Value not set
    cpcExportGenericFunctions_processAddress(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "PosSt", TRUE, dsDpParameters);
    assertEqual(makeDynString(UN_CONFIG_EXPORT_NONE), dsDpParameters, "Assertion failed for no value");
    dynClear(dsDpParameters);

    // Value set
    dpSet(TEST_DP_NAME + ".ProcessInput.PosSt", 50.0);
    cpcExportGenericFunctions_processAddress(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, "PosSt", TRUE, dsDpParameters);
    assertEqual(makeDynString(cpcConfigGenericFunctions_sPARAM_DPE_VALUE + 50.0), dsDpParameters, "Assertion failed for set value");
}
