#uses "fwUnitTestComponentAsserts.ctl"
#uses "test/unCPC6/cpcTestObjectConfig.ctl"
#uses "cpcGenericDpFunctions.ctl"

void test_setup() {
    dpCreate(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
}

void test_teardown() {
    dpDelete(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
}

void test_cpcGenericDpFunctions_setDeviceProperty() {
    dyn_string dsConfiguration, dsExceptionInfo;

    cpcGenericDpFunctions_setDeviceProperty(TEST_DP_NAME, "TEST", "TEST_VALUE", dsExceptionInfo);
    dpGet(TEST_DP_NAME + ".statusInformation.configuration.deviceConfiguration", dsConfiguration);
    assertEmpty(dsExceptionInfo, "Assertion failed for device property setting");
    assertTrue(dynContains(dsConfiguration, "PROP_TEST:TEST_VALUE"), "Assertion failed for device property");
}

void test_cpcGenericDpFunctions_getDeviceProperty() {
    string sValue;
    dyn_string dsExceptionInfo;

    // Not valid property
    sValue = cpcGenericDpFunctions_getDeviceProperty(TEST_DP_NAME, "TEST", dsExceptionInfo, "WrongValue");
    assertEmpty(dsExceptionInfo, "Assertion failed for empty device property");
    assertEqual("WrongValue", sValue, "Assertion failed for empty device property");


    // Valid property
    dpSet(TEST_DP_NAME + ".statusInformation.configuration.deviceConfiguration", makeDynString("PROP_TEST:TEST_VALUE"));
    sValue = cpcGenericDpFunctions_getDeviceProperty(TEST_DP_NAME, "TEST", dsExceptionInfo, "WrongValue");
    assertEmpty(dsExceptionInfo, "Assertion failed for set device property");
    assertEqual("TEST_VALUE", sValue, "Assertion failed for set device property");
}

void test_cpcGenericDpFunctions_doesDevicePropertyExist() {
    bool bExists;
    dyn_string dsExceptionInfo;

    // Non existing property
    bExists = cpcGenericDpFunctions_doesDevicePropertyExist(TEST_DP_NAME, "TEST", dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for non existing property");
    assertFalse(bExists, "Assertion failed for non existing property");

    // Existing property
    dpSet(TEST_DP_NAME + ".statusInformation.configuration.deviceConfiguration", makeDynString("PROP_TEST:TEST_VALUE"));
    bExists = cpcGenericDpFunctions_doesDevicePropertyExist(TEST_DP_NAME, "TEST", dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for valid property");
    assertTrue(bExists, "Assertion failed for valid property");
}

void test_cpcConfigGenericFunctions_getAssociatedDevices() {
    string sDp = getSystemName() + TEST_DP_NAME + ".ProcessInput.PosSt";

    dpSetAlias(TEST_DP_NAME + ".ProcessInput.PosSt", TEST_DP_NAME);
    dpSet(TEST_DP_NAME + ".statusInformation.configuration.deviceConfiguration",
          makeDynString("CHILDREN:" + TEST_DP_NAME, "CHILDREN:" + TEST_DP_NAME, "MASTER_NAME:" + TEST_DP_NAME, "MASTER_NAME:WrongDevice", "PARENTS:" + TEST_DP_NAME));
    assertEqual(makeDynString(sDp, sDp, sDp, "", sDp), cpcConfigGenericFunctions_getAssociatedDevices(TEST_DP_NAME),
                "Assertion failed for associated devices");
}

void test_cpcGenericDpFunctions_getMapping() {
    dpSet(TEST_DP_NAME + ".statusInformation.configuration.deviceConfiguration", makeDynString("DESC_PosSt:TEST_VALUE"));
    assertEqual("TEST_VALUE", cpcGenericDpFunctions_getMapping(TEST_DP_NAME, "PosSt"), "Assertion failed for property mapping");
}
