#uses "fwUnitTestComponentAsserts.ctl"
#uses "test/unCPC6/cpcTestObjectConfig.ctl"
#uses "cpcGenericObject.ctl"

void test_setup() {
    dpCreate(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
}

void test_teardown() {
    dpDelete(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
}

void test_cpcGenericObject_StringToMapping() {
    mapping mExpectedPairs, mPairs;

    // Empty string
    cpcGenericObject_StringToMapping("", mPairs);
    assertEqual(mExpectedPairs, mPairs, "Assertion failed for empty mapping");

    // Parameters
    mExpectedPairs["a"] = "b";
    mExpectedPairs["d"] = "e";
    cpcGenericObject_StringToMapping("a=b,c,d=e", mPairs);
    assertEqual(mExpectedPairs, mPairs, "Assertion failed for valid mapping");
}

/* TODO fix these tests
void test_cpcGenericObject_SendbinaryOrder() {
    time tOriginal, tFinal;
    dyn_string dsExceptionInfo;

    dpGet(TEST_DP_NAME + ".ProcessOutput.ManReg01:_online.._stime", tOriginal);
    cpcGenericObject_SendBinaryOrder(TEST_DP_NAME, "ManReg01", 0, "", dsExceptionInfo);
    dpGet(TEST_DP_NAME + ".ProcessOutput.ManReg01:_online.._stime", tFinal);
    assertEmpty(dsExceptionInfo);
    assertTrue(tOriginal < tFinal);
}

void test_cpcGenericObject_SendManRegPulse() {
    bool bAlreadySent;
    time tOriginal, tFinal;
    dyn_string dsExceptionInfo;

    dpGet(TEST_DP_NAME + ".ProcessOutput.ManReg01:_online.._stime", tOriginal);
    cpcGenericObject_SendBinaryOrder(TEST_DP_NAME, "ManReg01", 0, dsExceptionInfo, bAlreadySent);
    dpGet(TEST_DP_NAME + ".ProcessOutput.ManReg01:_online.._stime", tFinal);
    assertEmpty(dsExceptionInfo);
    assertTrue(bAlreadySent);
    assertTrue(tOriginal < tFinal);
}*/

void test_cpcGenericObject_SystemInvalid() {
    // System integrity alarm disabled
    assertTrue(cpcGenericObject_SystemInvalid(FALSE, 0), "Assertion failed for disabled system integrity alarm");

    // Alarm active
    assertTrue(cpcGenericObject_SystemInvalid(TRUE, 1), "Assertion failed for active alarm");

    // No alarm
    assertFalse(cpcGenericObject_SystemInvalid(TRUE, 0), "Assertion failed for no alarm");
}

void test_cpcGenericObject_WidgetValidnessAnimation() {
    string sLetter, sColor;

    // Valid system and data
    cpcGenericObject_WidgetValidnessAnimation(TRUE, 0, FALSE, sLetter, sColor);
    assertEqual("", sLetter, "Assertion failed for letter of valid system and data");
    assertEqual("", sColor, "Assertion failed for color of valid system and data");

    // Data invalid
    cpcGenericObject_WidgetValidnessAnimation(TRUE, 0, TRUE, sLetter, sColor);
    assertEqual(CPC_WIDGET_TEXT_INVALID, sLetter, "Assertion failed for letter of invalid data");
    assertEqual("unDataNotValid", sColor, "Assertion failed for color of invalid data");

    // System invalid
    cpcGenericObject_WidgetValidnessAnimation(FALSE, 0, FALSE, sLetter, sColor);
    assertEqual(CPC_WIDGET_TEXT_OLD_DATA, sLetter, "Assertion failed for letter of invalid system");
    assertEqual("unDataNotValid", sColor, "Assertion failed for color of invalid system");
}

void test_cpcGenericObject_WidgetStsRegBitAnimation() {
    string sLetter, sColor;

    // StsReg Invalid
    cpcGenericObject_WidgetStsRegBitAnimation(0, TRUE, 0, FALSE, makeDynInt(), makeDynInt(), makeDynString(), makeDynString(), sLetter, sColor);
    assertEqual("", sLetter, "Assertion failed for letter of invalid StsReg");
    assertEqual("", sColor, "Assertion failed for color of invalid StsReg");

    // Valid letter and color
    cpcGenericObject_WidgetStsRegBitAnimation(0, FALSE, 2, FALSE, makeDynInt(0, 1), makeDynInt(1, 2),
                                              makeDynString("WrongLetter", "RightLetter"), makeDynString("WrongColor", "RightColor"), sLetter, sColor);
    assertEqual("RightLetter", sLetter, "Assertion failed for letter of valid data");
    assertEqual("RightColor", sColor, "Assertion failed for color of valid data");
}

void test_cpcGenericObject_WidgetAlarmTextAnimationDoubleStsReg() {
    string sLetter, sColor;

    // Wrong input
    cpcGenericObject_WidgetAlarmTextAnimationDoubleStsReg(0, 0, makeDynInt(), makeDynString(""), makeDynInt(), makeDynString(""), sLetter, sColor);
    assertEqual("", sLetter, "Assertion failed for letter of empty input");
    assertEqual("unFaceplate_AlarmActive", sColor, "Assertion failed for color of empty input");

    // Valid input
    cpcGenericObject_WidgetAlarmTextAnimationDoubleStsReg(2, 2, makeDynInt(0, 1), makeDynString("FirstWrongLetter", "FirstRightLetter"),
                                                          makeDynInt(0, 1), makeDynString("SecondWrongLetter", "SecondRightLetter"), sLetter, sColor);
    assertEqual("SecondRightLetter", sLetter, "Assertion failed for letter of valid input");
    assertEqual("unFaceplate_AlarmActive", sColor, "Assertion failed for color of valid input");
}

void test_cpcGenericObject_FormatValueWithUnit() {
    assertEqual("50.000 s", cpcGenericObject_FormatValueWithUnit(50, "%6.3f", "s"), "Assertion failed for formatted value");
}

void test_cpcGenericObject_addUnicosActionToMenu() {
    dyn_string dsMenuList;

    // Empty menu configuration
    cpcGenericObject_addUnicosActionToMenu(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, makeDynString(), makeDynString(), dsMenuList);
    assertEqual(makeDynString("PUSH_BUTTON, Unmask Event, " + CPC_POPUPMENU_UNMASKEVENT_NUMBER + ", 0", "SEPARATOR"), dsMenuList,
                "Assertion failed for empty menu configuration");
    dynClear(dsMenuList);

    // Added configuration
    cpcGenericObject_addUnicosActionToMenu(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME,
                                           makeDynString(UN_POPUPMENU_ACK_TEXT, UN_POPUPMENU_BLOCK_PLC_TEXT, CPC_POPUPMENU_MANUAL_IO_ERROR_BLOCK),
                                           makeDynString(), dsMenuList);
    assertEqual(makeDynString("PUSH_BUTTON, Unmask Event, " +  CPC_POPUPMENU_UNMASKEVENT_NUMBER + ", 0",
                              "SEPARATOR",
                              "PUSH_BUTTON, Ack. Alarm, " + CPC_POPUPMENU_ACK_NUMBER + ", 0",
                              "PUSH_BUTTON, Block PLC Alarm, " + CPC_POPUPMENU_BLOCK_PLC_NUMBER + ", 0",
                              "PUSH_BUTTON, Manual IO Error Block, " + CPC_POPUPMENU_MANUAL_IO_ERROR_NUMBER + ", 0"), dsMenuList,
                "Assertion failed for set menu configuration");
}

void test_cpcGenericObject_addDefaultUnicosActionToMenu() {
    dyn_string dsMenuList;
    string sAlias = unGenericDpFunctions_getAlias(TEST_DP_NAME);

    // Empty menu configuration
    cpcGenericObject_addDefaultUnicosActionToMenu(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, makeDynString(), makeDynString(), dsMenuList);

    // With UNICOS External Links component installed
    if (fwInstallation_checkInstalledComponent("unExternalLinks") != -1 && sAlias != "") {
      assertEqual(makeDynString("PUSH_BUTTON, Hierarchy, " + CPC_POPUPMENU_HIERARCHY_NUMBER + ", 1",
                                "PUSH_BUTTON, " + UN_POPUPMENU_DIAGNOSTIC_LABEL + ", " + UN_POPUPMENU_DIAGNOSTIC_ID + ", 1",
                                "PUSH_BUTTON, " + UN_POPUPMENU_INFO_LABEL + ", " + UN_POPUPMENU_INFO_ID + ", 0",
                                "PUSH_BUTTON, " + UN_POPUPMENU_SYNOPTIC_LABEL + ", " + UN_POPUPMENU_SYNOPTIC_ID + ", 0",
                                "PUSH_BUTTON, " + UN_POPUPMENU_COMMENTS_LABEL + ", " + UN_POPUPMENU_COMMENTS_ID + ", 1",
                                "PUSH_BUTTON, " + UN_POPUPMENU_DEVICECONFIG_LABEL + ", " + UN_POPUPMENU_DEVICECONFIG_ID + ", 1",
                                "PUSH_BUTTON, " + UN_POPUPMENU_SYNOPTICLOC_LABEL + ", " + UN_POPUPMENU_SYNOPTICLOC_ID + ", 1",
                                "External Links",
                                "PUSH_BUTTON, Equipment Information" + ", " + UN_POPUPMENU_INFOR_EAM_EQUIP + ", 1",
                                "PUSH_BUTTON, New Work Order" + ", " + UN_POPUPMENU_INFOR_EAM_WO + ", 1",
                                "PUSH_BUTTON, Location on Map" + ", " + UN_POPUPMENU_GIS_MAP + ", 1",
                                "CASCADE_BUTTON, LogBook, 1",
                                "LogBook"), dsMenuList,
                  "Assertion failed for empty menu configuration");
      dynClear(dsMenuList);

      // Added configuration
      dynAppend(dsMenuList, "First item");
      cpcGenericObject_addDefaultUnicosActionToMenu(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME,
                                                    makeDynString(UN_POPUPMENU_FACEPLATE_TEXT, CPC_POPUPMENU_ALARMS_TEXT, CPC_POPUPMENU_RECIPES_TEXT),
                                                    makeDynString(), dsMenuList);
      assertEqual(makeDynString("First item", "SEPARATOR",
                                "PUSH_BUTTON, Faceplate, " + CPC_POPUPMENU_FACEPLATE_NUMBER + ", 1",
                                "PUSH_BUTTON, Hierarchy, " + CPC_POPUPMENU_HIERARCHY_NUMBER + ", 1",
                                "PUSH_BUTTON, Alarms, " + CPC_POPUPMENU_ALARMS_NUMBER + ", 0",
                                "PUSH_BUTTON, " + CPC_POPUPMENU_RECIPES_TEXT + ", " + CPC_POPUPMENU_RECIPES + ", 1",
                                "PUSH_BUTTON, " + UN_POPUPMENU_DIAGNOSTIC_LABEL + ", " + UN_POPUPMENU_DIAGNOSTIC_ID + ", 1",
                                "PUSH_BUTTON, " + UN_POPUPMENU_INFO_LABEL + ", " + UN_POPUPMENU_INFO_ID + ", 0",
                                "PUSH_BUTTON, " + UN_POPUPMENU_SYNOPTIC_LABEL + ", " + UN_POPUPMENU_SYNOPTIC_ID + ", 0",
                                "PUSH_BUTTON, " + UN_POPUPMENU_COMMENTS_LABEL + ", " + UN_POPUPMENU_COMMENTS_ID + ", 1",
                                "PUSH_BUTTON, " + UN_POPUPMENU_DEVICECONFIG_LABEL + ", " + UN_POPUPMENU_DEVICECONFIG_ID + ", 1",
                                "PUSH_BUTTON, " + UN_POPUPMENU_SYNOPTICLOC_LABEL + ", " + UN_POPUPMENU_SYNOPTICLOC_ID + ", 1",
                                "External Links",
                                "PUSH_BUTTON, Equipment Information" + ", " + UN_POPUPMENU_INFOR_EAM_EQUIP + ", 1",
                                "PUSH_BUTTON, New Work Order" + ", " + UN_POPUPMENU_INFOR_EAM_WO + ", 1",
                                "PUSH_BUTTON, Location on Map" + ", " + UN_POPUPMENU_GIS_MAP + ", 1",
                                "CASCADE_BUTTON, LogBook, 1",
                                "LogBook"), dsMenuList,
                  "Assertion failed for set menu configuration");
    } else {
      assertEqual(makeDynString("PUSH_BUTTON, Hierarchy, " + CPC_POPUPMENU_HIERARCHY_NUMBER + ", 1",
                                "PUSH_BUTTON, " + UN_POPUPMENU_DIAGNOSTIC_LABEL + ", " + UN_POPUPMENU_DIAGNOSTIC_ID + ", 0",
                                "PUSH_BUTTON, " + UN_POPUPMENU_INFO_LABEL + ", " + UN_POPUPMENU_INFO_ID + ", 0",
                                "PUSH_BUTTON, " + UN_POPUPMENU_SYNOPTIC_LABEL + ", " + UN_POPUPMENU_SYNOPTIC_ID + ", 0",
                                "PUSH_BUTTON, " + UN_POPUPMENU_COMMENTS_LABEL + ", " + UN_POPUPMENU_COMMENTS_ID + ", 1",
                                "PUSH_BUTTON, " + UN_POPUPMENU_DEVICECONFIG_LABEL + ", " + UN_POPUPMENU_DEVICECONFIG_ID + ", 1",
                                "PUSH_BUTTON, " + UN_POPUPMENU_SYNOPTICLOC_LABEL + ", " + UN_POPUPMENU_SYNOPTICLOC_ID + ", 1"), dsMenuList,
                  "Assertion failed for empty menu configuration");
      dynClear(dsMenuList);

      // Added configuration
      dynAppend(dsMenuList, "First item");
      cpcGenericObject_addDefaultUnicosActionToMenu(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME,
                                                    makeDynString(UN_POPUPMENU_FACEPLATE_TEXT, CPC_POPUPMENU_ALARMS_TEXT, CPC_POPUPMENU_RECIPES_TEXT),
                                                    makeDynString(), dsMenuList);
      assertEqual(makeDynString("First item", "SEPARATOR",
                                "PUSH_BUTTON, Faceplate, " + CPC_POPUPMENU_FACEPLATE_NUMBER + ", 1",
                                "PUSH_BUTTON, Hierarchy, " + CPC_POPUPMENU_HIERARCHY_NUMBER + ", 1",
                                "PUSH_BUTTON, Alarms, " + CPC_POPUPMENU_ALARMS_NUMBER + ", 0",
                                "PUSH_BUTTON, " + CPC_POPUPMENU_RECIPES_TEXT + ", " + CPC_POPUPMENU_RECIPES + ", 1",
                                "PUSH_BUTTON, " + UN_POPUPMENU_DIAGNOSTIC_LABEL + ", " + UN_POPUPMENU_DIAGNOSTIC_ID + ", 0",
                                "PUSH_BUTTON, " + UN_POPUPMENU_INFO_LABEL + ", " + UN_POPUPMENU_INFO_ID + ", 0",
                                "PUSH_BUTTON, " + UN_POPUPMENU_SYNOPTIC_LABEL + ", " + UN_POPUPMENU_SYNOPTIC_ID + ", 0",
                                "PUSH_BUTTON, " + UN_POPUPMENU_COMMENTS_LABEL + ", " + UN_POPUPMENU_COMMENTS_ID + ", 1",
                                "PUSH_BUTTON, " + UN_POPUPMENU_DEVICECONFIG_LABEL + ", " + UN_POPUPMENU_DEVICECONFIG_ID + ", 1",
                                "PUSH_BUTTON, " + UN_POPUPMENU_SYNOPTICLOC_LABEL + ", " + UN_POPUPMENU_SYNOPTICLOC_ID + ", 1"), dsMenuList,
                  "Assertion failed for set menu configuration");
    }
}

void test_cpcGenericObject_UserCBGetButtonState() {
    dyn_string dsReturnData;

    // Empty StsReg01
    cpcGenericObject_UserCBGetButtonState(TEST_DP_NAME, "", dsReturnData);
    assertEqual(makeDynString(false, "", (bit32)0, true, (bit32)0, false), dsReturnData,
                "Assertion failed for empty StsReg01");

    // Set StsReg01
    dpSet(TEST_DP_NAME + ".ProcessInput.StsReg01", 1);
    cpcGenericObject_UserCBGetButtonState(TEST_DP_NAME, "", dsReturnData);
    assertEqual(makeDynString(false, "", (bit32)1, false, (bit32)0, false), dsReturnData,
                "Assertion failed for set StsReg01");
}

void test_cpcGenericObject_getCtrlLink() {
    string sControllerName = "my_test_controller";
    string sSystemName = getSystemName();

    // No device names
    assertEqual(makeDynString(), cpcGenericObject_getCtrlLink(sSystemName, makeDynString()), "Assertion failed for no devices");

    // Wrong device type
    assertEqual(makeDynString(), cpcGenericObject_getCtrlLink(sSystemName, makeDynString(TEST_DP_NAME)), "Assertion failed for wrong DpType");

    // Controller device type
    dpCreate(sControllerName, UN_CONFIG_CPC_CONTROLLER_DPT_NAME);
    dpSetAlias(sControllerName + ".", sControllerName);
    assertEqual(makeDynString(sSystemName + sControllerName + ".ProcessInput.StsReg01"), cpcGenericObject_getCtrlLink(sSystemName, makeDynString(sControllerName)),
                "Assertion failed for right DpType");
    dpDelete(sControllerName);
}

void test_cpcGenericObject_WidgetControlStateAnimation() {
    string sLetter, sColor;

    // Empty StsReg
    cpcGenericObject_WidgetControlStateAnimation(0, 0, makeDynString(), sLetter, sColor);
    assertEqual("", sLetter, "Assertion failed for letter of empty StsReg");
    assertEqual("unWidget_ControlStateManualLocal", sColor, "Assertion failed for color of empty StsReg");

    // Set StsReg01 no letter
    cpcGenericObject_WidgetControlStateAnimation((int)pow(2, CPC_StsReg01_POSST), 0, makeDynString("CPC_StsReg01_POSST"), sLetter, sColor);
    assertEqual(CPC_UNDEFINED_CONSTANT_VALUE, sLetter, "Assertion failed for letter of PosSt bit");
    assertEqual("unWidget_ControlStateManualLocal", sColor, "Assertion failed for color of PosSt bit");

    // Set StsReg01 letter
    cpcGenericObject_WidgetControlStateAnimation((int)pow(2, CPC_StsReg01_MMOST), 0, makeDynString("CPC_StsReg01_POSST", "CPC_StsReg01_MMOST"), sLetter, sColor);
    assertEqual("M", sLetter, "Assertion failed for letter of MMoSt bit");
    assertEqual("unWidget_ControlStateManualLocal", sColor, "Assertion failed for color of MMoSt bit");

    // Set StsReg02 no letter
    cpcGenericObject_WidgetControlStateAnimation(0, (int)pow(2, CPC_StsReg02_AUONRST), makeDynString("CPC_StsReg01_POSST", "CPC_StsReg02_AUONRST"), sLetter, sColor);
    assertEqual(CPC_UNDEFINED_CONSTANT_VALUE, sLetter, "Assertion failed for letter of AuOnRSt bit");
    assertEqual("unWidget_ControlStateManualLocal", sColor, "Assertion failed for color of AuOnRSt bit");

    // Set StsReg02 letter
    cpcGenericObject_WidgetControlStateAnimation(0, (int)pow(2, CPC_StsReg02_SOFTLDST), makeDynString("CPC_StsReg01_POSST", "CPC_StsReg02_AUONRST", "CPC_StsReg02_SOFTLDST"), sLetter, sColor);
    assertEqual("SL", sLetter, "Assertion failed for letter of SoftLDSt bit");
    assertEqual("unWidget_ControlStateManualLocal", sColor, "Assertion failed for color of SoftLDSt bit");

    // Set forced
    cpcGenericObject_WidgetControlStateAnimation((int)pow(2, CPC_StsReg01_FOMOST), 0, makeDynString("CPC_StsReg01_POSST", "CPC_StsReg01_MMOST", "CPC_StsReg01_FOMOST"), sLetter, sColor);
    assertEqual("F", sLetter, "Assertion failed for letter of FoMoSt bit");
    assertEqual("unWidget_ControlStateForced", sColor, "Assertion failed for color of FoMoSt bit");
}

void test_cpcGenericObject_StsRegConstantNameToValue() {
    // Empty name
    assertEqual(-1, cpcGenericObject_StsRegConstantNameToValue("", 0, 0), "Assertion failed for empty name");

    // Wrong prefix
    //assertEqual(-1, cpcGenericObject_StsRegConstantNameToValue("WrongPrefix", 0, 0));

    // Non existing constant
    assertEqual(-1, cpcGenericObject_StsRegConstantNameToValue("CPC_WrongConstant", 0, 0), "Assertion failed for non existing constant");

    // Wrong constant length
    assertEqual(-1, cpcGenericObject_StsRegConstantNameToValue("CPC_CONFIG_MASTER_NAME", 0, 0), "Assertion failed for wrong constant length");

    // Wrong second part of constant
    assertEqual(-1, cpcGenericObject_StsRegConstantNameToValue("CPC_CONFIG_PARENTS", 0, 0), "Assertion failed for no StsReg01 constant");

    // StsReg01 constant inactive
    assertEqual(0, cpcGenericObject_StsRegConstantNameToValue("CPC_StsReg01_POSST", 0, 0), "Assertion failed for inactive PosSt");

    // StsReg01 constant active
    assertEqual(1, cpcGenericObject_StsRegConstantNameToValue("CPC_StsReg01_POSST", (int)pow(2, CPC_StsReg01_POSST), 0), "Assertion failed for active PosSt");

    // StsReg02 constant inactive
    assertEqual(0, cpcGenericObject_StsRegConstantNameToValue("CPC_StsReg02_AUONRST", 0, 0), "Assertion failed for inactive AuOnRSt");

    // StsReg02 constant active
    assertEqual(1, cpcGenericObject_StsRegConstantNameToValue("CPC_StsReg02_AUONRST", 0, (int)pow(2, CPC_StsReg02_AUONRST)), "Assertion failed for inactive AuOnRSt");
}

void test_cpcGenericObject_SendBigPulse() {
    time tOriginal, tFinal;
    dyn_string dsExceptionInfo;


    dpGet(TEST_DP_NAME + ".ProcessOutput.ManReg01:_online.._stime", tOriginal);
    cpcGenericObject_SendBigPulse(TEST_DP_NAME, ".ProcessOutput.ManReg01", 0, dsExceptionInfo);
    dpGet(TEST_DP_NAME + ".ProcessOutput.ManReg01:_online.._stime", tFinal);
    assertEmpty(dsExceptionInfo, "Assertion failed for big pulse sending");
    assertTrue(tFinal > tOriginal, "Assertion failed for big pulse sending");
}

void test_cpcGenericObject_translateAlarmAcronym() {
    // Empty acronyms
    assertEqual("No type", cpcGenericObject_translateAlarmAcronym(makeDynString()), "Assertion failed for empty alarm acronym");

    // AL acronym
    assertEqual("No Interlock (AL)", cpcGenericObject_translateAlarmAcronym(makeDynString("AL1")), "Assertion failed for alarm acronym");

    // FS acronym
    assertEqual("Full Stop Interlock (FS)", cpcGenericObject_translateAlarmAcronym(makeDynString("FS1")), "Assertion failed for full stop acronym");

    // TS acronym
    assertEqual("Temp Stop Interlock (TS)", cpcGenericObject_translateAlarmAcronym(makeDynString("TS1")), "Assertion failed for temporary stop acronym");

    // SI acronym
    assertEqual("Start Interlock (SI)", cpcGenericObject_translateAlarmAcronym(makeDynString("SI1")), "Assertion failed for start interlock acronym");

    // Unknown acronym
    assertEqual("Unknown (Unknown1)", cpcGenericObject_translateAlarmAcronym(makeDynString("Unknown1")), "Assertion failed for invalid acronym");

    // Multiple acronyms
    assertEqual("Multiple (AL,FS,TS,SI,Un)", cpcGenericObject_translateAlarmAcronym(makeDynString("AL1", "FS1", "TS1", "SI1", "Unknown1")), "Assertion failed for multiple acronyms");
}

void test_cpcGenericObject_translateMultiAlarmAcronym() {
    assertEqual(makeDynString("Obj1 - No Interlock (AL)", "Obj2 - Full Stop Interlock (FS)"),
                cpcGenericObject_translateMultiAlarmAcronym(makeDynString("AL1", "FS1"), makeDynString("Obj1", "Obj2")),
                "Assertion failed for multiple acronyms");
}

void test_cpcGenericObject_GetAnalogAlarmType() {
    int iAlertType;
    dyn_bool dbRanges;
    dyn_string dsExceptionInfo;

    // Wrong dp
    dbRanges = cpcGenericObject_GetAnalogAlarmType(TEST_DP_NAME + "WrongDp", iAlertType, dsExceptionInfo);
    assertError("ERROR", "cpcGenericObject_GetAnalogAlarmType: " + TEST_DP_NAME + "WrongDp" + getCatStr("unGeneration", "DPDOESNOTEXIST"), dsExceptionInfo);

    // No alarm
    dbRanges = cpcGenericObject_GetAnalogAlarmType(TEST_DP_NAME, iAlertType, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for no alarm");
    assertEqual(DPCONFIG_NONE, iAlertType, "Assertion failed for no alarm type");
    assertEqual(makeDynBool(false, false, false, false), dbRanges, "Assertion failed for no alarm ranges");

    // Binary alarm
    createBinaryAlert();
    dbRanges = cpcGenericObject_GetAnalogAlarmType(TEST_DP_NAME + ".ProcessInput.AlSt", iAlertType, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for binary alarm");
    assertEqual(DPCONFIG_ALERT_BINARYSIGNAL, iAlertType, "Assertion failed for binary alarm type");
    assertEqual(makeDynBool(false, false, false, false), dbRanges, "Assertion failed for binary alarm ranges");

    // Analog alarm
    createAlert();
    dbRanges = cpcGenericObject_GetAnalogAlarmType(TEST_DP_NAME + ".ProcessInput.PosSt", iAlertType, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for analog alarm");
    assertEqual(DPCONFIG_ALERT_NONBINARYSIGNAL, iAlertType, "Assertion failed for analog alarm type");
    assertEqual(makeDynBool(true, true, true, true), dbRanges, "Assertion failed for analog alarm ranges");
}

void test_cpcGenericObject_isAlarmWithMail() {
    // Digital alert
    assertFalse(cpcGenericObject_isAlarmWithMail(TEST_DP_NAME + ".ProcessInput.AlSt", DPCONFIG_ALERT_BINARYSIGNAL), "Assertion failed for unconfigured digital alarm");

    createBinaryAlert();
    dpSet(TEST_DP_NAME + ".ProcessInput.AlSt:_alert_hdl.._class", "_cpcAnalog_Mail.");
    assertTrue(cpcGenericObject_isAlarmWithMail(TEST_DP_NAME + ".ProcessInput.AlSt", DPCONFIG_ALERT_BINARYSIGNAL), "Assertion failed for configured digital alarm");

    // Analog alert
    assertFalse(cpcGenericObject_isAlarmWithMail(TEST_DP_NAME + ".ProcessInput.PosSt", DPCONFIG_ALERT_NONBINARYSIGNAL), "Assertion failed for unconfigured analog alarm");

    createAlert();
    dpSet(TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.1._class", "_cpcAnalogLL_Mail.",
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.2._class", "_cpcAnalogL_Mail.",
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.4._class", "_cpcAnalogH_Mail.",
          TEST_DP_NAME + ".ProcessInput.PosSt:_alert_hdl.5._class", "_cpcAnalogHH_Mail.");
    assertTrue(cpcGenericObject_isAlarmWithMail(TEST_DP_NAME + ".ProcessInput.PosSt", DPCONFIG_ALERT_NONBINARYSIGNAL), "Assertion failed for configured analog alarm");
}

void test_cpcGenericObject_isInMailSMS() {
    // Not configured
    assertFalse(cpcGenericObject_isInMailSMS(TEST_DP_NAME), "Assertion failed for not configured mail");

    // Configured
    dpCreate("_unSystemAlarm_" + TEST_DP_NAME, "_UnSystemAlarm");
    assertTrue(cpcGenericObject_isInMailSMS(getSystemName() + "_unSystemAlarm_" + TEST_DP_NAME + ".alarm"), "Assertion failed for configured mail");
    dpDelete("_unSystemAlarm_" + TEST_DP_NAME);
}

void test_cpcGenericObject_GetAnimationFunction() {
    assertEqual("cpcWidget_WidgetWidgetTypeAnimation", cpcGenericObject_GetAnimationFunction("WidgetType"), "Assertion failed for widget animation function");
}

void test_cpcGenericObject_getDeviceWidget() {
    // No associated widgets
    dpSetAlias(TEST_DP_NAME + ".", TEST_DP_NAME);
    assertEqual("", cpcGenericObject_getDeviceWidget(TEST_DP_NAME), "Assertion failed for no associated widgets");

    // Associated widgets
    if (!dynContains(g_dsObjectList, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME)) {
      dynAppend(g_dsObjectList, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME);
    }
    g_ddsObjectWidgetList[dynContains(g_dsObjectList, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME)] = makeDynString("Widget_WrongWidget", "Widget_RightWidget");
    dpSetDescription(TEST_DP_NAME + ".ProcessInput", ";;;;;;;RightWidget");
    assertEqual("Widget_RightWidget", cpcGenericObject_getDeviceWidget(TEST_DP_NAME), "Assertion failed for associated widget");
    removeGlobal(g_dsObjectList);
}

void test_cpcGenericObject_getLinkedAlarms() {
    string sDAName = "my_test_da";
    string sAAName = "my_test_aa";
    string sSystemName = getSystemName();
    dyn_string dsAlarms, dsExceptionInfo;

    // No linked alarms
    dsAlarms = cpcGenericObject_getLinkedAlarms(TEST_DP_NAME, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for no linked alarms");
    assertEqual(makeDynString(), dsAlarms, "Assertion failed for no linked alarms");

    // Linked alarms
    dpCreate(sDAName, UN_CONFIG_CPC_DIGITALALARM_DPT_NAME);
    dpCreate(sAAName, UN_CONFIG_CPC_ANALOGALARM_DPT_NAME);
    dpSet(TEST_DP_NAME + ".statusInformation.configuration.deviceConfiguration", makeDynString("CHILDREN:" + sDAName, "CHILDREN:" + sAAName));
    dpSetAlias(sDAName + ".", sDAName);
    dpSetAlias(sAAName + ".", sAAName);
    dsAlarms = cpcGenericObject_getLinkedAlarms(TEST_DP_NAME, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for linked alarms");
    assertEqual(makeDynString(sSystemName + sDAName, sSystemName + sAAName), dsAlarms, "Assertion failed for linked alarms");

    dpDelete(sDAName);
    dpDelete(sAAName);
}

void test_cpcGenericObject_getLinkedObjects() {
    string sSystemName = getSystemName();
    dyn_string dsObjects, dsExceptionInfo;

    // No linked objects
    dsObjects = cpcGenericObject_getLinkedObjects(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for no linked objects");
    assertEqual(makeDynString(), dsObjects, "Assertion failed for no linked objects");

    // Linked objects
    dpSet(TEST_DP_NAME + ".statusInformation.configuration.deviceConfiguration", makeDynString("CHILDREN:" + TEST_DP_NAME));
    dpSetAlias(TEST_DP_NAME + ".", TEST_DP_NAME);
    dsObjects = cpcGenericObject_getLinkedObjects(TEST_DP_NAME, UN_CONFIG_CPC_TESTOBJECT_DPT_NAME, dsExceptionInfo);
    assertEmpty(dsExceptionInfo, "Assertion failed for linked objects");
    assertEqual(makeDynString(sSystemName + TEST_DP_NAME), dsObjects, "Assertion failed for linked objects");
}

void test_cpcGenericObject_WidgetWarningAnimationDoubleStsReg() {
    string sLetter, sColor;

    // Empty StsReg
    cpcGenericObject_WidgetWarningAnimationDoubleStsReg(0, 0, makeDynInt(), makeDynInt(), makeDynString(), makeDynString(), sLetter, sColor);
    assertEqual("", sLetter, "Assertion failed for letter of empty StsReg");
    assertEqual("cpcColor_Widget_Warning", sColor, "Assertion failed for color of empty StsReg");

    // StsReg01
    cpcGenericObject_WidgetWarningAnimationDoubleStsReg(2, 0, makeDynInt(0, 1), makeDynInt(0, 1), makeDynString("FirstWrongLetter", "FirstRightLetter"),
                                                        makeDynString("SecondWrongLetter", "SecondRightLetter"), sLetter, sColor);
    assertEqual("FirstRightLetter", sLetter, "Assertion failed for letter of StsReg01");
    assertEqual("cpcColor_Widget_Warning", sColor, "Assertion failed for color of StsReg01");

    // StsReg02
    cpcGenericObject_WidgetWarningAnimationDoubleStsReg(2, 2, makeDynInt(0, 1), makeDynInt(0, 1), makeDynString("FirstWrongLetter", "FirstRightLetter"),
                                                        makeDynString("SecondWrongLetter", "SecondRightLetter"), sLetter, sColor);
    assertEqual("SecondRightLetter", sLetter, "Assertion failed for letter of StsReg02");
    assertEqual("cpcColor_Widget_Warning", sColor, "Assertion failed for color of StsReg02");
}

void test_cpcGenericObject_valueInsideRange() {
    dyn_string dsExceptionInfo;

    // No range
    assertTrue(cpcGenericObject_valueInsideRange("", 50.0, TEST_DP_NAME + ".ProcessInput.PosSt", dsExceptionInfo), "Assertion failed for no range");
    assertEmpty(dsExceptionInfo, "Assertion failed for no range");

    // Value inside range
    dpSet(TEST_DP_NAME + ".ProcessInput.PosSt:_pv_range.._type", DPCONFIG_MINMAX_PVSS_RANGECHECK,
          TEST_DP_NAME + ".ProcessInput.PosSt:_pv_range.._max", 100.0,
          TEST_DP_NAME + ".ProcessInput.PosSt:_pv_range.._min", 0.0);
    assertTrue(cpcGenericObject_valueInsideRange("", 50.0, TEST_DP_NAME + ".ProcessInput.PosSt", dsExceptionInfo), "Assertion failed for value inside range");
    assertEmpty(dsExceptionInfo, "Assertion failed for value inside range");

    // Value outside range
    assertFalse(cpcGenericObject_valueInsideRange("", 150.0, TEST_DP_NAME + ".ProcessInput.PosSt", dsExceptionInfo), "Assertion failed for value outside range");
    assertTrue(dynContains(dsExceptionInfo, "Value " + 150.0 + " for  is outside the allowed range defined for " + TEST_DP_NAME + ".ProcessInput.PosSt"),
               "Assertion failed for value outside range exception");
}

void test_cpcGenericObject_setDefaultValue() {
    float fValue;
    mapping mDefDpeValues;
    dyn_string dsExceptionInfo;

    mDefDpeValues["MPosR"] = 50.0;

    // Wrong dp
    assertEqual(-1, cpcGenericObject_setDefaultValue(TEST_DP_NAME + "WrongDp", mDefDpeValues, dsExceptionInfo), "Assertion failed for setting of invalid Dp");
    assertEqual(makeDynString("DPE " + TEST_DP_NAME + "WrongDp.ProcessOutput.MPosR does not exist."), dsExceptionInfo, "Assertion failed for settinf of invalid Dp");
    dynClear(dsExceptionInfo);

    // Valid dp
    assertEqual(0, cpcGenericObject_setDefaultValue(TEST_DP_NAME, mDefDpeValues, dsExceptionInfo), "Assertion failed for setting of valid Dp");
    assertEmpty(dsExceptionInfo, "Assertion failed for setting of valid Dp");
    dpGet(TEST_DP_NAME + ".ProcessOutput.MPosR", fValue);
    assertEqual(50.0, fValue, "Assertion failed for setting of valid Dp");
}

void test_cpcGenericObject_getStatusBitValues() {
    string sByteNumber, sColorType, sBitType, sAlarmType;

    // Non existing bit
    assertEqual(-1, cpcGenericObject_getStatusBitValues("WRONGBIT", sByteNumber, sColorType, sBitType, sAlarmType), "Assertion failed for getting invalid bit");
    assertEqual("", sByteNumber, "Assertion failed for byte number of invalid bit");
    assertEqual("", sColorType, "Assertion failed for color of invalid bit");
    assertEqual("", sBitType, "Assertion failed for bit type of invalid bit");
    assertEqual("", sAlarmType, "Assertion failed for alarm type of invalid bit");

    // Bit from StsReg01
    assertEqual(0, cpcGenericObject_getStatusBitValues("AUMOST", sByteNumber, sColorType, sBitType, sAlarmType), "Assertion failed for getting StsReg01 bit");
    assertEqual("1", sByteNumber, "Assertion failed for byte number of StsReg01 bit");
    assertEqual("Faceplate", sColorType, "Assertion failed for color of StsReg01 bit");
    assertEqual("Status", sBitType, "Assertion failed for bit type of StsReg01 bit");
    assertEqual("NULL", sAlarmType, "Assertion failed for alarm type of StsReg01 bit");

    // Bit from StsReg02
    assertEqual(0, cpcGenericObject_getStatusBitValues("ALBW", sByteNumber, sColorType, sBitType, sAlarmType), "Assertion failed for getting StsReg02 bit");
    assertEqual("2", sByteNumber, "Assertion failed for byte number of StsReg02 bit");
    assertEqual("Faceplate", sColorType, "Assertion failed for color of StsReg02 bit");
    assertEqual("Warning", sBitType, "Assertion failed for bit type of StsReg02 bit");
    assertEqual("NULL", sAlarmType, "Assertion failed for alarm type of StsReg02 bit");

    // Bit from alarm
    assertEqual(0, cpcGenericObject_getStatusBitValues("IST", sByteNumber, sColorType, sBitType, sAlarmType), "Assertion failed for getting alarm bit");
    assertEqual("1", sByteNumber, "Assertion failed for byte number of alarm bit");
    assertEqual("StsReg01", sColorType, "Assertion failed for color of alarm bit");
    assertEqual("ALUNACK", sBitType, "Assertion failed for bit type of alarm bit");
    assertEqual("ISt", sAlarmType, "Assertion failed for alarm type of alarm bit");
}
