#uses "fwUnitTestComponentAsserts.ctl"
#uses "unShelvedAE/unShelvedAE.ctl"
#uses "unShelvedAE/unShelvedAE_TableParameters.ctl"

//------------------------------------------------------------------
void test_unShelvedAE_setupSuite(){
  _unShelvedAE_setNonAlarmsEvents();
  _unShelvedAE_setBlockedAlarms();
  _unShelvedAE_setMaskedAlarms();
  _unShelvedAE_setMaskedEvents();
}

//------------------------------------------------------------------
void test_unShelvedAE_getBlockedAlarms() {
  string sDp;
  dyn_string dsDp, dsType, dsTimestamp, dsTestBlockedAlarmDp, dsTestBlockedAlarmType, exceptionInfo;
  dyn_dyn_anytype ddResults;

  string sysName = getSystemName();
  dyn_string dsExpectedBlockedAlarmDp = makeDynString(sysName+"unShelved_testBlockedAlarm1", sysName+"unShelved_testBlockedAlarm2", sysName+"unShelved_testBlockedAlarm3");
  dyn_string dsExpectedBlockedAlarmType = makeDynString("Blocked Alarm", "Blocked Alarm", "Blocked Alarm");

  unShelvedAE_getBlockedAlarms("", ddResults, exceptionInfo);
  assertTrue(dynlen(ddResults)>0, "No result has returned for the blocked Alarm query call.");
  assertTrue(dynlen(exceptionInfo)==0, "An unexpected error has ocurred when processing the blocked Alarm query call.");

  unShelvedAE_getDpsBlockedAlarms(ddResults, dsDp, dsType, dsTimestamp);

  for (int i = 1; i <= dynlen(dsDp); i++) {
    sDp = dsDp[i];
    if (sDp.startsWith(sysName+"unShelved_test")) {
      dynAppend(dsTestBlockedAlarmDp, sDp);
      dynAppend(dsTestBlockedAlarmType, dsType[i]);
    }
  }

  assertEqual(dsExpectedBlockedAlarmDp, dsTestBlockedAlarmDp, "Test blocked alarms datapoints do not match with the expected result.");
  assertEqual(dsTestBlockedAlarmType, dsTestBlockedAlarmType, "Test blocked alarms types do not match with the expected result.");
}

//------------------------------------------------------------------
void test_unShelvedAE_getMaskedAlarms() {
  string sDp;
  dyn_string dsDp, dsType, dsTimestamp, dsTestMaskedAlarmDp, dsTestMaskedAlarmType, exceptionInfo;
  dyn_dyn_anytype ddResults;

  string sysName = getSystemName();
  dyn_string dsExpectedMaskedAlarmDp = makeDynString(sysName+"unShelved_testMaskedAlarm1", sysName+"unShelved_testMaskedAlarm2", sysName+"unShelved_testMaskedAlarm3");
  dyn_string dsExpectedMaskedAlarmType = makeDynString("Masked Alarm", "Masked Alarm", "Masked Alarm");

  unShelvedAE_getMaskedAlarms("", ddResults, exceptionInfo);
  assertTrue(dynlen(ddResults)>0, "No result has returned for the masked Alarm query call.");
  assertTrue(dynlen(exceptionInfo)==0, "An unexpected error has ocurred when processing the masked Alarm query call.");

  unShelvedAE_getDpsMaskedAlarms(ddResults, dsDp, dsType, dsTimestamp);

  for (int i = 1; i <= dynlen(dsDp); i++) {
    sDp = dsDp[i];
    if (sDp.startsWith(sysName+"unShelved_test")) {
      dynAppend(dsTestMaskedAlarmDp, sDp);
      dynAppend(dsTestMaskedAlarmType, dsType[i]);
    }
  }

  assertEqual(dsExpectedMaskedAlarmDp, dsTestMaskedAlarmDp, "Test masked alarms datapoints do not match with the expected result.");
  assertEqual(dsTestMaskedAlarmType, dsTestMaskedAlarmType, "Test masked alarms types do not match with the expected result.");
}

//------------------------------------------------------------------
void test_unShelvedAE_getMaskedEvents() {
  string sDp;
  dyn_string dsDp, dsType, dsTimestamp, dsTestMaskedEventDp, dsTestMaskedEventType, exceptionInfo;
  dyn_dyn_anytype ddResults;

  string sysName = getSystemName();
  dyn_string dsExpectedMaskedEventDp = makeDynString(sysName+"unShelved_testMaskedEvent1", sysName+"unShelved_testMaskedEvent2", sysName+"unShelved_testMaskedEvent3");
  dyn_string dsExpectedMaskedEventType = makeDynString("Masked Event", "Masked Event", "Masked Event");

  unShelvedAE_getMaskedEvents("", ddResults, exceptionInfo);
  assertTrue(dynlen(ddResults)>0, "No result has returned for the masked Event query call.");
  assertTrue(dynlen(exceptionInfo)==0, "An unexpected error has ocurred when processing the masked Event query call.");

  unShelvedAE_getDpsMaskedEvents(ddResults, dsDp, dsType, dsTimestamp);

  for (int i = 1; i <= dynlen(dsDp); i++) {
    sDp = dsDp[i];
    if (sDp.startsWith(sysName+"unShelved_test")) {
      dynAppend(dsTestMaskedEventDp, sDp);
      dynAppend(dsTestMaskedEventType, dsType[i]);
    }
  }

  assertEqual(dsExpectedMaskedEventDp, dsTestMaskedEventDp, "Test masked events datapoints do not match with the expected result.");
  assertEqual(dsTestMaskedEventType, dsTestMaskedEventType, "Test masked events types do not match with the expected result.");
}

//------------------------------------------------------------------
void test_unShelvedAE_getTableInfo() {
  dyn_string dsAlias, dsType, dsSystem, dsApplication, dsDescription, dsNature, dsComments, exceptionInfo;

  string sysname = getSystemName();

  dyn_string dsExpectedAlias = makeDynString("testBlockedAlarm1", "testBlockedAlarm2", "testBlockedAlarm3", "testMaskedAlarm1",
                                             "testMaskedAlarm2", "testMaskedAlarm3", "testMaskedEvent1", "testMaskedEvent2",
                                             "testMaskedEvent3");
  dyn_string dsExpectedType = makeDynString("Blocked Alarm", "Blocked Alarm", "Blocked Alarm", "Masked Alarm", "Masked Alarm",
                                            "Masked Alarm", "Masked Event", "Masked Event", "Masked Event");
  dyn_string dsExpectedSystem = makeDynString(sysname, sysname, sysname, sysname, sysname, sysname, sysname, sysname, sysname);
  dyn_string dsExpectedApplication = makeDynString("", "", "", "", "", "", "", "", "");
  dyn_string dsExpectedDescription = makeDynString("Blocked Alarm 1", "Blocked Alarm 2", "Blocked Alarm 3", "Masked Alarm 1",
                                                   "Masked Alarm 2", "Masked Alarm 3", "Masked Event 1", "Masked Event 2",
                                                   "Masked Event 3");
  dyn_string dsExpectedNature = makeDynString("", "", "", "", "", "", "", "", "");
  dyn_string dsExpectedDevComment = makeDynString("[2020.01.01 01:00:00] (jpabloro)   TestCommentBA1", "[2020.01.01 01:00:00] (jpabloro)   TestCommentBA2",
                                                  "[2020.01.01 01:00:00] (jpabloro)   TestCommentBA3", "[2021.01.01 01:00:00] (jpabloro)   TestCommentMA1",
                                                  "[2021.01.01 01:00:00] (jpabloro)   TestCommentMA2", "[2021.01.01 01:00:00] (jpabloro)   TestCommentMA3",
                                                  "[2022.01.01 01:00:00] (jpabloro)   TestCommentME1", "[2022.01.01 01:00:00] (jpabloro)   TestCommentME2",
                                                  "[2022.01.01 01:00:00] (jpabloro)   TestCommentME3");

  unShelvedAE_TableParameters tableParameters = new unShelvedAE_TableParameters(makeDynString(""), makeDynString(""), makeDynString(""), makeDynString(""),
                                                                                makeDynString(""), makeDynString(""), makeDynString(""), makeDynString(""));

  unShelvedAE_getTableInfo(true, true, true, tableParameters, exceptionInfo);

  dyn_string tableParamAliases = tableParameters.getAliases();
  dyn_string tableParamTypes = tableParameters.getTypes();
  dyn_string tableParamSystems = tableParameters.getSystems();
  dyn_string tableParamApplications = tableParameters.getApplications();
  dyn_string tableParamDescriptions = tableParameters.getDescriptions();
  dyn_string tableParamNatures = tableParameters.getNatures();
  dyn_string tableParamComments = tableParameters.getComments();

  for (int i = 1; i <= dynlen(dsExpectedAlias); i++) {
    int pos = dynContains(tableParamAliases, dsExpectedAlias[i]);
    if (pos > 0) {
      assertEqual(dsExpectedAlias[i], tableParamAliases[pos], "Aliases do not match with the expected results.");
      assertEqual(dsExpectedType[i], tableParamTypes[pos], "Types do not match with the expected results.");
      assertEqual(dsExpectedSystem[i], tableParamSystems[pos], "Systems do not match with the expected results.");
      assertEqual(dsExpectedApplication[i], tableParamApplications[pos], "Applications do not match with the expected results.");
      assertEqual(dsExpectedDescription[i], tableParamDescriptions[pos], "Descriptions do not match with the expected results.");
      assertEqual(dsExpectedNature[i], tableParamNatures[pos], "Nature types do not match with the expected results.");
      assertEqual(dsExpectedDevComment[i], tableParamComments[pos], "Device comments types do not match with the expected results.");
    }
  }
}

//------------------------------------------------------------------
void test_unShelvedAE_getFormattedComments() {
  string sFormatComment;
  dyn_string dsUnformattedComment = makeDynString("1970.03.12 07:30:20~jpabloro~TestDevComment~~Test~EmailTest~TRUE");
  string sExpectedFormattedComment = "[1970.03.12 07:30:20] (jpabloro)   TestDevComment";
  unShelvedAE_getFormattedComments(dsUnformattedComment, sFormatComment);
  assertEqual(sExpectedFormattedComment, sFormatComment, "Formatted comment does not match with the expected result.");
}

//------------------------------------------------------------------
void test_unShelvedAE_teardownSuite(){
  _unShelvedAE_deleteNonAlarmEventsDPs();
  _unShelvedAE_deleteBlockedAlarmsDPs();
  _unShelvedAE_deleteMaskedAlarmsDPs();
  _unShelvedAE_deleteMaskedEventsDPs();
}

//------------------------------------------------------------------
private void _unShelvedAE_setNonAlarmsEvents() {
  for(int i = 1; i <= 3; i++) {
    // Create datapoints for No alarms/events
    dpCreate("unShelved_testNonAlarmEvent" + i, "CPC_DigitalAlarm");
    // Set Non Alarms and events (eventMask has to be 1, otherwise it is considered an event)
    dpSet("unShelved_testNonAlarmEvent" + i + ".eventMask:_original.._value", 1);
  }
}

//------------------------------------------------------------------
private void _unShelvedAE_setBlockedAlarms() {
  dyn_string exceptionInfo;
  for(int i = 1; i <= 3; i++) {
    // Create datapoints for Blocked Alarms
    dpCreate("unShelved_testBlockedAlarm" + i, "CPC_DigitalAlarm");
    // Set Blocked Alarms values
    dpSet("unShelved_testBlockedAlarm" + i + ".ProcessInput.StsReg01:_original.._value", 16384,
          "unShelved_testBlockedAlarm" + i + ".eventMask:_original.._value", 1);
    // Set Blocked Alarms configuration
    unConfigGenericFunctions_setAlias("unShelved_testBlockedAlarm" + i, "testBlockedAlarm" + i, exceptionInfo);
    dpSetDescription("unShelved_testBlockedAlarm" + i + ".", "Blocked Alarm " + i);
    dpSetTimed(makeTime(2020,01,01,01,00), "unShelved_testBlockedAlarm" + i + ".statusInformation.deviceLog:_original.._value", "2020.01.01 01:00:00~jpabloro~TestCommentBA" + i + "~~TestApp~TestEmail~TRUE");
  }
}

//------------------------------------------------------------------
private void _unShelvedAE_setMaskedAlarms() {
  dyn_string exceptionInfo;
  dyn_mixed alarmObject;
  fwAlertConfig_objectCreateDiscrete(alarmObject, makeDynString("OK", "Bad"), makeDynString("*", "10"),
                                     makeDynString("", getSystemName()+"_unSystemIntegrityAlarm_lvl1."),
                                     "", makeDynString(""), "", false, makeDynBool(false,false), "",
                                     makeDynString("",""), exceptionInfo, false, false, "", false);
  for(int i = 1; i <= 3; i++) {
    // Create datapoints for Masked Alarms
    dpCreate("unShelved_testMaskedAlarm" + i, "CPC_DigitalAlarm");
    // Set Masked Alarms values
    dpSet("unShelved_testMaskedAlarm" + i + ".eventMask:_original.._value", 1);
    fwAlertConfig_objectSet("unShelved_testMaskedAlarm" + i + ".ProcessInput.ISt", alarmObject, exceptionInfo);
    unAlarmConfig_mask("unShelved_testMaskedAlarm" + i + ".ProcessInput.ISt", true, exceptionInfo);
    // Set Masked Alarms configuration
    unConfigGenericFunctions_setAlias("unShelved_testMaskedAlarm" + i, "testMaskedAlarm" + i, exceptionInfo);
    dpSetDescription("unShelved_testMaskedAlarm" + i + ".", "Masked Alarm " + i);
    dpSetTimed(makeTime(2021,01,01,01,00), "unShelved_testMaskedAlarm" + i + ".statusInformation.deviceLog:_original.._value", "2021.01.01 01:00:00~jpabloro~TestCommentMA" + i + "~~TestApp~TestEmail~TRUE");
  }
}

//------------------------------------------------------------------
private void _unShelvedAE_setMaskedEvents() {
  dyn_string exceptionInfo;
  for(int i = 1; i <= 3; i++) {
    // Create datapoints for Masked Events
    dpCreate("unShelved_testMaskedEvent" + i, "CPC_DigitalAlarm");
    // Set Masked Events values
    dpSet("unShelved_testMaskedEvent" + i + ".eventMask:_original.._value", 0);
    // Set Masked Events configuration
    unConfigGenericFunctions_setAlias("unShelved_testMaskedEvent" + i, "testMaskedEvent" + i, exceptionInfo);
    dpSetDescription("unShelved_testMaskedEvent" + i + ".", "Masked Event " + i);
    dpSetTimed(makeTime(2022,01,01,01,00), "unShelved_testMaskedEvent" + i + ".statusInformation.deviceLog:_original.._value", "2022.01.01 01:00:00~jpabloro~TestCommentME" + i + "~~TestApp~TestEmail~TRUE");
  }
}

//------------------------------------------------------------------
private void _unShelvedAE_deleteNonAlarmEventsDPs() {
  for(int i = 1; i <= 3; i++) {
    if(dpExists("unShelved_testNonAlarmEvent" + i))
      dpDelete("unShelved_testNonAlarmEvent" + i);
  }
}

//------------------------------------------------------------------
private void _unShelvedAE_deleteBlockedAlarmsDPs() {
  for(int i = 1; i <= 3; i++) {
    if(dpExists("unShelved_testBlockedAlarm" + i))
      dpDelete("unShelved_testBlockedAlarm" + i);
  }
}

//------------------------------------------------------------------
private void _unShelvedAE_deleteMaskedAlarmsDPs() {
  for(int i = 1; i <= 3; i++) {
    if(dpExists("unShelved_testMaskedAlarm" + i))
      dpDelete("unShelved_testMaskedAlarm" + i);
  }
}

//------------------------------------------------------------------
private void _unShelvedAE_deleteMaskedEventsDPs() {
  for(int i = 1; i <= 3; i++) {
    if(dpExists("unShelved_testMaskedEvent" + i))
      dpDelete("unShelved_testMaskedEvent" + i);
  }
}
