#uses "unSystemIntegrity/unSystemIntegrity_alarmDescription.ctl"

/**@name LIBRARY: unSystemIntegrity_unRcpPeriodicSaving.ctl

@author: Juan Manuel Pablo Rodriguez (BE-ICS-FT)

Creation Date: 07/01/2022

version 1.0

External Functions: 
	
Internal Functions: 
	
Purpose: 
	Library of the unRcpPeriodicSaving component used by the systemIntegrity.
	 	
Usage: Public

PVSS manager usage: CTRL (WCCOActrl)

Constraints:
	. data point type needed: _UnSystemIntegrity
	. data point: unRcpPeriodicSaving_systemIntegrity
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/

//--------------------------------------------------------------------------------------------------------------------------------
// constant declaration
//------------------
const string unRcpPeriodicSaving_pattern = "unRcpPeriodicSaving";

// indexes of settings in .config.data
const int c_unSystemIntegrity_unRcpPeriodicSaving_ConfigIndex_Count                 = 3;
const int c_unSystemIntegrity_unRcpPeriodicSaving_ConfigIndex_LoopDelay             = 1;
const int c_unSystemIntegrity_unRcpPeriodicSaving_ConfigIndex_MaxManagerBlockTime   = 2;
const int c_unSystemIntegrity_unRcpPeriodicSaving_ConfigIndex_MaxManagerRestarts    = 3;

// default values for .config.data
const int c_unSystemIntegrity_unRcpPeriodicSaving_DefaultConfig_LoopDelay           = 60;
const int c_unSystemIntegrity_unRcpPeriodicSaving_DefaultConfig_MaxManagerBlockTime = 300;
const int c_unSystemIntegrity_unRcpPeriodicSaving_DefaultConfig_MaxManagerRestarts  = 6;

// alert ranges of system integrity alarm datapoint
const int c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_OK                = 0;
const int c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_PMONWarning       = 10;
const int c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_MaxRetriesReached = 20;
const int c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_ManagerBlocked    = 30;
const int c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_ManagerStopped    = 40;
const int c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_NoDbConnection    = 50;
const int c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_FailurePeriodicSaving = 60;

// message indexes in unRcpPeriodicSaving catalogue
const int c_unSystemIntegrity_unRcpPeriodicSaving_Catalogue_ErrorNotFound     = 1;
const int c_unSystemIntegrity_unRcpPeriodicSaving_Catalogue_ManagerNotRunning = 2;
const int c_unSystemIntegrity_unRcpPeriodicSaving_Catalogue_ManagerIsBlocking = 3;
const int c_unSystemIntegrity_unRcpPeriodicSaving_Catalogue_KillingManager    = 4;
const int c_unSystemIntegrity_unRcpPeriodicSaving_Catalogue_StartingManager   = 5;
const int c_unSystemIntegrity_unRcpPeriodicSaving_Catalogue_MaxRetriesReached = 6;
const int c_unSystemIntegrity_unRcpPeriodicSaving_Catalogue_NoDbConnection    = 7;
const int c_unSystemIntegrity_unRcpPeriodicSaving_Catalogue_ReadPmonInfoFailed= 8;

//Default dedicated SI manager numbers
const int SYSTEMINTEGRITY_DEFAULT_MANAGER_NUMBER_UNRCPPERIODICSAVING = 42;
//------------------

// script variables declaration
dyn_string g_unSystemIntegrity_unRcpPeriodicSavingList;             // List of the unRcpPeriodicSaving manager dp checked

int g_unSystemIntegrity_unRcpPeriodicSaving_monitorThreadId;        // ID of the monitoring thread
int g_unSystemIntegrity_unRcpPeriodicSaving_loopDelay;              // Delay after each check cycle
int g_unSystemIntegrity_unRcpPeriodicSaving_maxManagerBlockingTime; // Manager gets killed if it blocks longer than this time
int g_unSystemIntegrity_unRcpPeriodicSaving_maxManagerRestarts;     // Max. number of retries to restart manager

string unRcpPeriodicSaving_alarmDescription;
shared_ptr<unSystemIntegrity_UpdateAlarmDescription> unRcpPeriodicSaving_updAlDesc = new unSystemIntegrity_UpdateAlarmDescription();

//@{

//------------------------------------------------------------------------------------------------------------------------
// unRcpPeriodicSaving_systemIntegrityInfo
/** Return the list of systemAlarm pattern. 
  
@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  CTRL

@return list of systemAlarm pattern
*/
dyn_string unRcpPeriodicSaving_systemIntegrityInfo()
{
  return makeDynString(unRcpPeriodicSaving_pattern);
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unRcpPeriodicSaving_Initialize
/**
Purpose:
Get the list of unRcpPeriodicSaving check by the systemIntegrity. The ones that are enabled.

@param dsResult: dyn_string, output, the list of enabled unRcpPeriodicSaving that are checked

Usage: Public

PVSS manager usage: CTRL (WCCOActrl)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
unSystemIntegrity_unRcpPeriodicSaving_Initialize(dyn_string &dsResult)
{
	dyn_string dsList;
	int len, i;
	string dpToCheck;
	bool enabled;

	// Get configuration data from unRcpPeriodicSaving_systemIntegrityInfo
	unSystemIntegrity_unRcpPeriodicSaving_getConfigData(true);

	// get the list of unRcpPeriodicSaving DP to checked
	dsList = dpNames(c_unSystemAlarm_dpPattern + unRcpPeriodicSaving_pattern + "*", c_unSystemAlarm_dpType);
	len = dynlen(dsList);
	for(i = 1; i<=len; i++) {
		dpToCheck = dpSubStr(dsList[i], DPSUB_DP);
		dpToCheck = substr(dpToCheck, strlen(c_unSystemAlarm_dpPattern), strlen(dpToCheck));
		dpGet(dsList[i]+".enabled", enabled);
		if(enabled)
			dynAppend(dsResult, dpToCheck);
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unRcpPeriodicSaving_HandleCommand
/**
Purpose:
handle any systemIntegrity command.

@param sDpe1: string, input, dpe
@param command: int, input, the systemIntegrity command
@param sDpe2: string, input, dpe
@param parameters: dyn_string, input, the list of parameters of the systemIntegrity command

Usage: Public

PVSS manager usage: CTRL (WCCOActrl)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/
unSystemIntegrity_unRcpPeriodicSaving_HandleCommand(string sDpe1, int command, string sDpe2, dyn_string parameters)
{
	dyn_string exceptionInfo;
	int i, len=dynlen(parameters);
	
	switch(command) {
		case UN_SYSTEMINTEGRITY_ADD: // add the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				unSystemIntegrity_unRcpPeriodicSaving_checking(parameters[i], true, true, exceptionInfo);
			}
			unSystemIntegrity_unRcpPeriodicSaving_manageMonitoringThread("start");
			break;
		case UN_SYSTEMINTEGRITY_DELETE: // delete the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				unSystemIntegrity_unRcpPeriodicSaving_checking(parameters[i], false, false, exceptionInfo);
				// remove dp from the alert list of applicationDP if it is in
				_unSystemIntegrity_modifyApplicationAlertList(c_unSystemAlarm_dpPattern+parameters[i], false, exceptionInfo);
				// delete the _UnSystemAlarm dps
				if(dpExists(c_unSystemAlarm_dpPattern+parameters[i]))
					dpDelete(c_unSystemAlarm_dpPattern+parameters[i]);
			}
			delay(1);
			unSystemIntegrity_unRcpPeriodicSaving_manageMonitoringThread("stop");
			break;
		case UN_SYSTEMINTEGRITY_ENABLE: // enable the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				if(dpExists(c_unSystemAlarm_dpPattern+parameters[i]))
					unSystemIntegrity_unRcpPeriodicSaving_checking(parameters[i], false, true, exceptionInfo);
			}
			unSystemIntegrity_unRcpPeriodicSaving_manageMonitoringThread("start");
			break;
		case UN_SYSTEMINTEGRITY_DISABLE: // disable the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				if(dpExists(c_unSystemAlarm_dpPattern+parameters[i]))
					unSystemIntegrity_unRcpPeriodicSaving_checking(parameters[i], false, false, exceptionInfo);
			}
			delay(1);
			unSystemIntegrity_unRcpPeriodicSaving_manageMonitoringThread("stop");
			break;
		case UN_SYSTEMINTEGRITY_DIAGNOSTIC: // give the list of _UnSystemAlarm DP and the state
			dpSetWait(unRcpPeriodicSaving_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.commandResult", command,
					unRcpPeriodicSaving_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.result", g_unSystemIntegrity_unRcpPeriodicSavingList);
			break;
		default:
			break;
	}

	if(dynlen(exceptionInfo) > 0) {
		if(isFunctionDefined("unMessageText_sendException")) {
			unMessageText_sendException("*", "*", "unSystemIntegrity_unRcpPeriodicSaving_HandleCommand", "user", "*", exceptionInfo);
		}
// handle any error in case the send message failed
		if(dynlen(exceptionInfo) > 0) {
			DebugN("unSystemIntegrity_unRcpPeriodicSaving_HandleCommand", exceptionInfo);
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unRcpPeriodicSaving_DataCallback
/**
* Callback function to get most recent configuration data
*
* @param sDPe
* @param dsResult
*/
public void unSystemIntegrity_unRcpPeriodicSaving_DataCallback(string sDPe, dyn_string &dsResult)
{
  unSystemIntegrity_unRcpPeriodicSaving_getConfigData(false);
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unRcpPeriodicSaving_getConfigData
/**
* Get configuration data and settings from unRcpPeriodicSaving_systemIntegrityInfo
*
* @param   bResetOnError   input,  true: set default values on error, false: don't set default values on error
*/
private void unSystemIntegrity_unRcpPeriodicSaving_getConfigData(bool bResetOnError)
{
  if(dpExists(unRcpPeriodicSaving_pattern + UN_SYSTEMINTEGRITY_EXTENSION))
  {
    dyn_string configData;
    dpGet(unRcpPeriodicSaving_pattern + UN_SYSTEMINTEGRITY_EXTENSION + ".config.data", configData);

    if(dynlen(configData) == c_unSystemIntegrity_unRcpPeriodicSaving_ConfigIndex_Count)
    {
      g_unSystemIntegrity_unRcpPeriodicSaving_loopDelay = (int)configData[c_unSystemIntegrity_unRcpPeriodicSaving_ConfigIndex_LoopDelay];
      g_unSystemIntegrity_unRcpPeriodicSaving_maxManagerBlockingTime = (int)configData[c_unSystemIntegrity_unRcpPeriodicSaving_ConfigIndex_MaxManagerBlockTime];
      g_unSystemIntegrity_unRcpPeriodicSaving_maxManagerRestarts = (int)configData[c_unSystemIntegrity_unRcpPeriodicSaving_ConfigIndex_MaxManagerRestarts];
    }
    else
    {
      if(bResetOnError==true)
      {
        dpSetWait(unRcpPeriodicSaving_pattern + UN_SYSTEMINTEGRITY_EXTENSION + ".config.data",
              makeDynString(c_unSystemIntegrity_unRcpPeriodicSaving_DefaultConfig_LoopDelay,
                            c_unSystemIntegrity_unRcpPeriodicSaving_DefaultConfig_MaxManagerBlockTime,
                            c_unSystemIntegrity_unRcpPeriodicSaving_DefaultConfig_MaxManagerRestarts)
             );
        g_unSystemIntegrity_unRcpPeriodicSaving_loopDelay = c_unSystemIntegrity_unRcpPeriodicSaving_DefaultConfig_LoopDelay;
        g_unSystemIntegrity_unRcpPeriodicSaving_maxManagerBlockingTime = c_unSystemIntegrity_unRcpPeriodicSaving_DefaultConfig_MaxManagerBlockTime;
        g_unSystemIntegrity_unRcpPeriodicSaving_maxManagerRestarts = c_unSystemIntegrity_unRcpPeriodicSaving_DefaultConfig_MaxManagerRestarts;
      }
      else
      {
        DebugTN("unSystemIntegrity_unRcpPeriodicSaving_getConfigData(): Inconsistency found in " + unRcpPeriodicSaving_pattern + UN_SYSTEMINTEGRITY_EXTENSION + ".config.data" + " No action has been taken.");
      }
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unRcpPeriodicSaving_checking
/**
Purpose:
This function register/de-register the callback funtion of unRcpPeriodicSaving manager. This function can also create the _unSystemAlarm_unRcpPeriodicSaving dp 
with the alarm config but it cannot delete it.

	@param sDp: string, input, data point name
	@param bCreate: bool, input, true create the dp and the alarm config if it does not exist, enable it
	@param bRegister: bool, input, true do a dpConnect, false do a dpDisconnect
	@param exceptionInfo: dyn_string, output, exception are returned here

Usage: Internal

PVSS manager usage: CTRL (WCCOActrl)

Constraints:
	. PVSS version: 3.0 
	. operating system: W2000, NT and Linux, but tested only under W2000 and Linux.
	. distributed system: yes.
*/

private void unSystemIntegrity_unRcpPeriodicSaving_checking(string sDp, bool bCreate, bool bEnable, dyn_string &exceptionInfo)
{
  // Remove the system name
  string sDpToCheck = unSystemIntegrity_removeSystemName(sDp);
  string sSysAlarmDp = c_unSystemAlarm_dpPattern + sDpToCheck;

  if(bCreate)
  {
    if(!dpExists(sSysAlarmDp))
    {
      dpCreate(sSysAlarmDp, c_unSystemAlarm_dpType);
	}
    string sSysAlarmDpe = sSysAlarmDp + ".alarm";
    dpSetDescription(sSysAlarmDpe, "unRecipe Periodic Saving Alarm:");
    unSystemIntegrity_unRcpPeriodicSaving_addAlertConfig(sSysAlarmDpe, exceptionInfo);
  }

  // handle alarm description connection/disconnection
  if (bEnable) {
    // register alarm description
    unSystemIntegrity_registerAlarmDescription(sSysAlarmDp + ".alarm", unRcpPeriodicSaving_updAlDesc, exceptionInfo);
    classConnect("unSystemIntegrity_unRcpPeriodicSaving_alarmDescriptionCB", unRcpPeriodicSaving_updAlDesc, unSystemIntegrity_UpdateAlarmDescription::updateAlarmDescription);
  } else {
    // unregister alarm description
    classDisconnect("unSystemIntegrity_unRcpPeriodicSaving_alarmDescriptionCB", unRcpPeriodicSaving_updAlDesc, unSystemIntegrity_UpdateAlarmDescription::updateAlarmDescription);
  }

  // Register dp and dpconnect to dsParam
  if(dynlen(exceptionInfo)<=0) {

    // add/remove integrity classes
    _unSystemIntegrity_setList(g_unSystemIntegrity_unRcpPeriodicSavingList, sSysAlarmDp, bEnable);

    // enable/disable and unmask/mask
    dpSetWait(sSysAlarmDp + ".enabled", bEnable);
    unAlarmConfig_mask(sSysAlarmDp + ".alarm", !bEnable, exceptionInfo);
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unRcpPeriodicSaving_addAlertConfig
/**
* Add discrete alert config to system alarm datapoint element
*
* @param sSysAlarmDpe    input, dpe for which we want to create the alert config
*	@param exceptionInfo   output, exception are returned here
*/
private void unSystemIntegrity_unRcpPeriodicSaving_addAlertConfig(string sSysAlarmDpe, dyn_string &exceptionInfo)
{
  // set discrete alarm config
  dyn_mixed alarmObject;
  fwAlertConfig_objectCreateDiscrete(
    alarmObject,
    makeDynString("OK",
                  "Manager status unknown",
                  "Max retries reached",
                  "Manager is blocking",
                  "Manager is stopped",
                  "No connection to DB",
                  "Failure Saving Recipe"),
    makeDynString("*",
                  c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_PMONWarning,
                  c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_MaxRetriesReached,
                  c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_ManagerBlocked,
                  c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_ManagerStopped,
                  c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_NoDbConnection,
                  c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_FailurePeriodicSaving),
    makeDynString("",
                  getSystemName()+"_unSystemIntegrityAlarm_lvl1.",
                  getSystemName()+"_unSystemIntegrityAlarm_lvl1.",
                  getSystemName()+"_unSystemIntegrityAlarm_lvl2.",
                  getSystemName()+"_unSystemIntegrityAlarm_lvl2.",
                  getSystemName()+"_unSystemIntegrityAlarm_lvl2.",
                  getSystemName()+"_unSystemIntegrityAlarm_lvl2."),
    "",
    makeDynString(""),
    "",
    false,
    makeDynBool(false,false,false,false,false,false,false),
    "",
    makeDynString("","","","","","",""),
    exceptionInfo,
    false,
    false,
    "",
    false); //exception info returned here
  if(dynlen(exceptionInfo)){
	DebugTN(__FUNCTION__, __LINE__, exceptionInfo);
	return;
  }
  //set the alarm to the dpe
  fwAlertConfig_objectSet(sSysAlarmDpe, alarmObject, exceptionInfo);
  if(dynlen(exceptionInfo)){
	DebugTN(__FUNCTION__, __LINE__, exceptionInfo);
	return;
  }
  //activate it
  fwAlertConfig_activate(sSysAlarmDpe, exceptionInfo);
  if(dynlen(exceptionInfo)){
	DebugTN(__FUNCTION__, __LINE__, exceptionInfo);
	return;
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unRcpPeriodicSaving_manageMonitoringThread
/**
* Starts and Stops the thread in which the checking is done
*
* @param command         input, start or stop the thread
*/
private void unSystemIntegrity_unRcpPeriodicSaving_manageMonitoringThread(string command)
{
  dyn_string dsList = dpNames(c_unSystemAlarm_dpPattern + unRcpPeriodicSaving_pattern + "*", c_unSystemAlarm_dpType);

  switch(command)
  {
    case "start":
      if(g_unSystemIntegrity_unRcpPeriodicSaving_monitorThreadId == 0)
      {
        if(dynlen(dsList)> 0)
        {
          g_unSystemIntegrity_unRcpPeriodicSaving_monitorThreadId = startThread("unSystemIntegrity_unRcpPeriodicSaving_checkAlert");
          //DebugN("***********STARTING THREAD*********");
        }
      }
      break;

    case "stop":
      for(int i =1; i<=dynlen(dsList); i++)
      {
        bool enable;
        dpGet(dsList[i] + ".enabled", enable);
        if(enable == true)
          return;
      }

      int err;
      if(g_unSystemIntegrity_unRcpPeriodicSaving_monitorThreadId != 0)
      {
        err = stopThread(g_unSystemIntegrity_unRcpPeriodicSaving_monitorThreadId);
        //DebugN("***********STOPPING THREAD*********");
      }

      if(err<0)
        DebugTN(__FUNCTION__, __LINE__, "unSystemIntegrity_unRcpPeriodicSaving_manageMonitoringThread() - stopping thread failed!");
      else
        g_unSystemIntegrity_unRcpPeriodicSaving_monitorThreadId = 0;

      break;

    default:
      break;
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unRcpPeriodicSaving_checkAlert
/**
* Check in a periodic cycle if the unRcpPeriodicSaving CTRL manager
* and connection to the DB work properly
*
* Following checks are done:
* ----------------------------------------------------------
* 1. Check the current state of the unRecipe CTRL manager.
*    Kill it if it is blocking longer than a specified time
* 2. Check the DB connection. Set an error if disconnected or
*    not accessible.
*/
private void unSystemIntegrity_unRcpPeriodicSaving_checkAlert()
{
  const string sManagerName = "WCCOActrl";
  const string sManagerOption = "unRecipe/unRcpPeriodicSaving.ctl -num " + UN_RCP_PERIODIC_SAVING_CTRL_MANAGER;

  string sManagerStartMode = "";
  int iManagerState;
  int iManagerIndex;

  int iRestartCounter = 0;
  bool bBlocking = false;
  time tBlockingStart;

  dyn_string exceptionInfo;

  while(1)
  {
    int iErrorCode;
    unSystemIntegrity_unRcpPeriodicSaving_presetErrorCode(iErrorCode, c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_OK, true);

    // --------------------------------------------------- 1. Check the unRecipe CTRL manager state --------------------------------------------------------
    // Get current manager status and manager process information
    unSystemIntegrity_getState(sManagerName, sManagerOption, iManagerState);
    if(iManagerState >=0 && unSystemIntegrity_unRcpPeriodicSaving_findManager(sManagerName, sManagerOption, sManagerStartMode, iManagerIndex) == true)
    {
      // Only pay attention to process information if PMON query is successful

      string sManagerAction = "";

      switch(iManagerState)
      {
        case SYSTEMINTEGRITY_MANAGER_STOPPED:
        {
          unSystemIntegrity_unRcpPeriodicSaving_throwError("ERROR", c_unSystemIntegrity_unRcpPeriodicSaving_Catalogue_ManagerNotRunning);
          unSystemIntegrity_unRcpPeriodicSaving_presetErrorCode(iErrorCode, c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_ManagerStopped);

          if (sManagerStartMode == "always") {
            // Start manager if it is not running and start mode requires a start
            sManagerAction = "start";
          }
          bBlocking = false;
          break;
        }
        case SYSTEMINTEGRITY_MANAGER_BLOCKED:
        {
          unSystemIntegrity_unRcpPeriodicSaving_throwError("ERROR", c_unSystemIntegrity_unRcpPeriodicSaving_Catalogue_ManagerIsBlocking);
          unSystemIntegrity_unRcpPeriodicSaving_presetErrorCode(iErrorCode, c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_ManagerBlocked);

          if(bBlocking == false) {
            tBlockingStart = getCurrentTime();
            bBlocking = true;
          }
          time tCurrent = getCurrentTime();
          if(((tCurrent - tBlockingStart) > g_unSystemIntegrity_unRcpPeriodicSaving_maxManagerBlockingTime) && (g_unSystemIntegrity_unRcpPeriodicSaving_maxManagerBlockingTime > 0)) {
            // Kill manager if it is blocking for more than a specified time
            sManagerAction = "kill";
          }
          break;
        }
        default: // SYSTEMINTEGRITY_MANAGER_RUNNING
        {
          bBlocking = false;
          iRestartCounter = 0;
          break;
        }
      }

      // Execute sManagerAction
      if((iRestartCounter < g_unSystemIntegrity_unRcpPeriodicSaving_maxManagerRestarts) || (g_unSystemIntegrity_unRcpPeriodicSaving_maxManagerRestarts < 0))
      {
        // g_unSystemIntegrity_unRcpPeriodicSaving_maxManagerRestarts = 0 --> kill never
        // g_unSystemIntegrity_unRcpPeriodicSaving_maxManagerRestarts < 0 --> kill always
        // g_unSystemIntegrity_unRcpPeriodicSaving_maxManagerRestarts > 0 --> kill couple of times

        switch(sManagerAction)
        {
          case "kill":
          {
            dyn_string dsExInfo;
            unSystemIntegrity_unRcpPeriodicSaving_throwError("INFO", c_unSystemIntegrity_unRcpPeriodicSaving_Catalogue_KillingManager);
            unSystemIntergity_manageManager(sManagerName, sManagerOption, SYSTEMINTEGRITY_KILL_COMMAND, iManagerIndex, dsExInfo);
            iRestartCounter++;
            break;
          }
          case "start":
          {
            dyn_string dsExInfo;
            unSystemIntegrity_unRcpPeriodicSaving_throwError("INFO", c_unSystemIntegrity_unRcpPeriodicSaving_Catalogue_StartingManager);
            unSystemIntergity_manageManager(sManagerName, sManagerOption, SYSTEMINTEGRITY_START_COMMAND, iManagerIndex, dsExInfo);
            iRestartCounter++;
            break;
          }
        }
      }
      else
      {
        unSystemIntegrity_unRcpPeriodicSaving_throwError("WARNING", c_unSystemIntegrity_unRcpPeriodicSaving_Catalogue_MaxRetriesReached, "(" + g_unSystemIntegrity_unRcpPeriodicSaving_maxManagerRestarts + ") Aborting current action: " + sManagerAction + " unRcpPeriodicSaving manager.");
        unSystemIntegrity_unRcpPeriodicSaving_presetErrorCode(iErrorCode, c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_MaxRetriesReached);
      }
    }
    else
    {
      unSystemIntegrity_unRcpPeriodicSaving_throwError("WARNING", c_unSystemIntegrity_unRcpPeriodicSaving_Catalogue_ReadPmonInfoFailed, "Process manager " + sManagerName + " " + sManagerOption + " might not be running.");
      unSystemIntegrity_unRcpPeriodicSaving_presetErrorCode(iErrorCode, c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_PMONWarning);
    }

    // --------------------------------------------------- 2. Check the DB connection for saving Recipes ---------------------------------------------------------------
	// Check the DB connection status by doing an implicit SQL query
	fwConfigurationDB_initialize("", exceptionInfo);
    if (!fwConfigurationDB_DBConfigured || dynlen(exceptionInfo)) {
      unSystemIntegrity_unRcpPeriodicSaving_throwError("ERROR", c_unSystemIntegrity_unRcpPeriodicSaving_Catalogue_NoDbConnection);
      unSystemIntegrity_unRcpPeriodicSaving_presetErrorCode(iErrorCode, c_unSystemIntegrity_unRcpPeriodicSaving_AlertLevel_NoDbConnection);
      dynClear(exceptionInfo);
    }

	   // Write ErrorCode into unSystemIntegrity unRcpPeriodicSaving alarm datapoint
    unSystemIntegrity_unRcpPeriodicSaving_writeErrorCode(iErrorCode);

    delay(g_unSystemIntegrity_unRcpPeriodicSaving_loopDelay);
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unRcpPeriodicSaving_presetErrorCode
/**
* Compare error code and set it if is higher than current error code
*
* @param iErrorCode     input/output, current error code
* @param iNewCode       input, new error code
* @param bForceNewCode  output, false: Write iErrorCode only if iNewCode is higher, true: Always write iErrorCode
*/
private void unSystemIntegrity_unRcpPeriodicSaving_presetErrorCode(int &iErrorCode, int iNewCode, bool bForceNewCode=false)
{
  if(bForceNewCode) {
    iErrorCode = iNewCode;
  }
  else {
    if(iNewCode > iErrorCode) {
      iErrorCode = iNewCode;
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unRcpPeriodicSaving_writeErrorCode
/**
* Write error code to unSystemIntegrity unRcpPeriodicSaving alarm datapoint
*
* @param iErrorCode     input, error code to write
*/
void unSystemIntegrity_unRcpPeriodicSaving_writeErrorCode(int iErrorCode)
{
  string dpeAlarm = c_unSystemAlarm_dpPattern + unRcpPeriodicSaving_pattern + ".alarm";

  if (dpExists(dpeAlarm) == false)
	  return;

  // Obtain the previous error code to determine if we have to trigger the update alarm description event
  int prevErrorCode;
  int rc = dpGet(dpeAlarm, prevErrorCode);
  dyn_errClass err = getLastError();
  if (dynlen(err)) {
	  DebugTN(__FUNCTION__, __LINE__, "Previous error code cannot be obtained from the alarm datapoint", err);
   return;
  }

  if (iErrorCode != prevErrorCode) {
    int rc = dpSetWait(dpeAlarm, iErrorCode);
    err = getLastError();
    if (dynlen(err)) {
	    DebugTN(__FUNCTION__, __LINE__, "Error code cannot be written to the alarm datapoint", err);
     return;
    }
  } else {
    // Update the alarm description by triggering an event when the previous error code is the same than the current one
    dyn_dyn_anytype alertInfo;
    string query = "SELECT ALERT '_alert_hdl.._value' FROM '" + dpeAlarm + "'";
    rc = dpQuery(query, alertInfo);
    dyn_errClass err = getLastError();
    if (dynlen(err)) {
      DebugTN(__FUNCTION__, __LINE__, "Alert information cannot be extracted from the query", err);
      return;
    }
    unSystemIntegrity_AlarmDescriptionTimestamp myTimeStamp;
    int lenResults = dynlen(alertInfo);
    if (lenResults >= 1) {
      for (int i = 2; i <= lenResults; i++) {
        myTimeStamp.timeStamp = alertInfo[i][2];
        myTimeStamp.aTimeStamp = alertInfo[i][2];
        myTimeStamp.dpeAlert = getAIdentifier(alertInfo[i][2]);
        myTimeStamp.alarmValue = alertInfo[i][3];
        if (!dpExists(myTimeStamp.dpeAlert)) {
          DebugTN(__FUNCTION__, __LINE__, "Alert datapoint element " + myTimeStamp.dpeAlert + " does not exist or has been removed.");
	         return;
        }
      }
    }
    // Trigger alarm description update
    unRcpPeriodicSaving_updAlDesc.triggerUpdateAlarmDesc(myTimeStamp.timeStamp, myTimeStamp.aTimeStamp, myTimeStamp.dpeAlert, myTimeStamp.alarmValue);
  }
}

//------------------------------------------------------------------------------------------------------------------------


// unSystemIntegrity_unRcpPeriodicSaving_throwError
/**
* Write special error to message log
*
* @param severity       input,  severity of the message
* @param code           input,  error code to write
* @param sDetails       input,  message details
*/
private void unSystemIntegrity_unRcpPeriodicSaving_throwError(string severity = "ERROR", int code = 1, string sDetails="")
{
  int prio = PRIO_WARNING;
  int type = ERR_CONTROL;

  switch(strtoupper(severity))
  {
    case "INFO":    prio=PRIO_INFO;    break;
    case "WARNING": prio=PRIO_WARNING; break;
    case "ERROR":   prio=PRIO_SEVERE;  break;
  }

  errClass err = makeError("unSystemIntegrity_unRcpPeriodicSaving", prio, type, code, sDetails);
  throwError(err);

  unRcpPeriodicSaving_alarmDescription = getCatStr("unSystemIntegrity_unRcpPeriodicSaving", "0000"+code);

  return;
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unRcpPeriodicSaving_findManager
/**
* Get process information of a specific manager
*
* @param manager     input, manager name
* @param commandLine input, manager option
* @param startMode   output, manager start mode
* @param managerPos  output, manager pmon idx
*
* @return            output, true=OK, false=ERROR
*/
private bool unSystemIntegrity_unRcpPeriodicSaving_findManager(string manager, string commandLine, string &startMode, int &managerPos)
{
  dyn_mixed managerInfo;
  fwInstallationManager_getProperties(manager, commandLine, managerInfo);
  int len = dynlen(managerInfo);
  if(len >= FW_INSTALLATION_MANAGER_PMON_IDX && len >= FW_INSTALLATION_MANAGER_START_MODE ) {
    managerPos = managerInfo[FW_INSTALLATION_MANAGER_PMON_IDX];
    startMode = managerInfo[FW_INSTALLATION_MANAGER_START_MODE];
    return true;
  }
  return false;
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unRcpPeriodicSaving_alarmDescriptionCB
/**
* Callback function to set the unRcpPeriodicSaving alarm description in the alertSet when
* the updateAlarmDescription class event has been triggered
*
* @param timeStamp         input, time, alert timestamp
* @param aTimeStamp        input, atime, alert atimestamp
* @param dpeAlert          input, string, alert datapoint element
* @param alarmValue        input, int, alarm value number
*/

public void unSystemIntegrity_unRcpPeriodicSaving_alarmDescriptionCB(time timeStamp, atime aTimeStamp, string dpeAlert, int alarmValue) {
  unSystemIntegrity_setAlarmDescription(unRcpPeriodicSaving_alarmDescription, timeStamp, aTimeStamp, dpeAlert);
}

//@}
